Package: clevRvis
Type: Package
Title: Visualization Techniques for Clonal Evolution
Version: 1.11.0
Authors@R: 
	person("Sarah", "Sandmann", , "sarah.sandmann@uni-muenster.de", role = c("aut", "cre"),comment=c(ORCID="0000-0002-5011-0641"))
Description: clevRvis provides a set of visualization
        techniques for clonal evolution. These include shark plots, dolphin
	plots and plaice plots. Algorithms for time point interpolation 
	as well as therapy effect estimation are provided. Phylogeny-aware
	color coding is implemented. A shiny-app for generating plots
	interactively is additionally provided.
License: LGPL-3
Encoding: UTF-8
Requires: shiny, ggraph, igraph, ggiraph, cowplot, htmlwidgets, readxl,
        dplyr, readr, purrr, tibble, patchwork, R.utils, shinyWidgets,
        colorspace, shinyhelper, shinycssloaders, ggnewscale,
        shinydashboard, DT, colourpicker, grDevices, methods, utils,
        stats, ggplot2, magrittr, tools
Imports: shiny, ggraph, igraph, ggiraph, cowplot, htmlwidgets, readxl,
        dplyr, readr, purrr, tibble, patchwork, R.utils, shinyWidgets,
        colorspace, shinyhelper, shinycssloaders, ggnewscale,
        shinydashboard, DT, colourpicker, grDevices, methods, utils,
        stats, ggplot2, magrittr, tools
VignetteBuilder: knitr
Suggests: knitr, rmarkdown, BiocStyle
biocViews: Software, ShinyApps, Visualization
BugReports: https://github.com/sandmanns/clevRvis/issues
URL: https://github.com/sandmanns/clevRvis
RoxygenNote: 7.2.1
git_url: https://git.bioconductor.org/packages/clevRvis
git_branch: devel
git_last_commit: 02b71a5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 20:33:06 UTC; biocbuild
Author: Sarah Sandmann [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5011-0641>)
Maintainer: Sarah Sandmann <sarah.sandmann@uni-muenster.de>

export(clevRvisShiny)
export(createSeaObject)
export(dolphinPlot)
export(extSharkPlot)
export(plaicePlot)
export(combinedPlot)
export(sharkPlot)
export(exploreTrees)

exportClasses(seaObject)
exportMethods(
    show,
    ytop, "ytop<-",
    ybtm, "ybtm<-",
    xpos, "xpos<-",
    col, "col<-",
    timepoints, "timepoints<-",
    fracTable, "fracTable<-",
    parents, "parents<-",
    nestLevels, "nestLevels<-",
    cloneFamily, "cloneFamily<-",
    cloneLabels, "cloneLabels<-",
    defaultLabels, "defaultLabels<-",
    originTimepoint, "originTimepoint<-"
)


import('shiny',except=c(validate,setProgress,dataTableOutput,renderDataTable,
                        runExample))
import('shinydashboard')
import('patchwork')
import('ggiraph') 
import('shinyWidgets')

importFrom('ggraph', "ggraph", "geom_edge_elbow")
importFrom('igraph',"graph_from_data_frame")
importFrom('cowplot',"plot_grid")
importFrom('htmlwidgets', "saveWidget")
importFrom('readxl', "read_excel")
importFrom('dplyr', "select", "filter", "as_data_frame")
importFrom('magrittr', "%>%")
importFrom('readr', "parse_number", "read_csv")
importFrom('purrr', "detect_index", "pluck")
importFrom('tibble',"add_column", "add_row")
importFrom('R.utils', "insert")
importFrom('colorspace', "rainbow_hcl", "darken")
importFrom('shinyhelper', 'helper', "observe_helpers")
importFrom('shinycssloaders', "withSpinner")
importFrom('ggnewscale', "new_scale")
importFrom('DT', "dataTableOutput", "datatable", "renderDataTable")
importFrom('colourpicker', "colourInput")
importFrom("grDevices", "colorRampPalette", "col2rgb")
importFrom("methods", "new", "representation", "setMethod", "setGeneric", 
            "is", "setClass", "show")
importFrom("stats", "spline")
importFrom("tools", "file_ext")
importFrom("utils", "tail")
importFrom("ggplot2", "geom_point", "aes", "scale_y_reverse", 
            "scale_x_reverse", "coord_flip")
importFrom("ggplot2", "ggplot", "scale_fill_manual", "theme", "geom_segment", 
            "geom_text", "geom_point")
importFrom("ggplot2", "expand_limits", "guides", "scale_color_discrete", 
            "scale_color_manual", "scale_size_area")
importFrom("ggplot2", "theme_void", "guide_legend", "geom_hline", 
            "scale_colour_manual", ".data")

<p align="center">
    <img height="150" src="https://uni-muenster.sciebo.de/s/RwS9PA1dj1Nmivn/download">
</p>

# clevRvis

clevRvis provides an extensive set of visualization techniques for clonal evolution. Three types of plots are available: 1) shark plots (basic trees, showing the phylogeny and optionally the cancer cell fraction CCF); 2) dolphin plots (advanced visualization, showing the phylogeny and the development of CCFs over time); 3) plaice plots (novel visualization, showing the phylogeny, the development of CCFs and the development of remaining healthy alleles, influenced by bi-allelic events, over time). Moreover, the tool provides algorithms for fully automatic interpolation of time points and estimation of therapy effect to approximate a tumor's development in the presence of few measured time points, as well as exploring alternative trees.

## Requirements
To run clevRvis, you need R Version 4.1.0 or higher.

##  Installation
clevRvis is currently under development. The latest version can be installed from github using the following command:

```
if (!requireNamespace("devtools", quietly=TRUE))
  install.packages("devtools")
devtools::install_github("sandmanns/clevRvis")
```

createSeaObject <- function(fracTable, parents, timepoints,
                            col = NULL, cloneLabels = NULL,
                            originTimepoint = NULL,
                            timepointInterpolation = TRUE,
                            therapyEffect = NULL) {

    if(!is.matrix(fracTable) || !is.numeric(fracTable)){
        stop("No valid input for fracTable provided.")
    }
    if(!is.vector(parents) || !is.numeric(parents)){
        stop("No valid input for parents provided.")
    }
    if(!is.vector(timepoints) || !is.numeric(timepoints)){
        stop("No valid input for timepoints provided.")
    }
    if(!is.null(col) && is.null(.isColor(col))){
        stop("No valid input for col provided.")
    }
    if(!is.null(cloneLabels) && !is.character(cloneLabels)){
        stop("No valid input for cloneLabels provided.")
    }
    if(!is.null(originTimepoint) && !is.numeric(originTimepoint)){
        stop("No valid input for originTimepoint provided.")
    }
    if(timepointInterpolation != TRUE && timepointInterpolation != FALSE){
        stop("No valid input for timepointInterpolation provided.")
    }
    if(!is.null(therapyEffect) && !is.numeric(therapyEffect)){
        stop("No valid input for therapyEffect provided.")
    }
    if(nrow(fracTable) != length(parents)){
        stop("Number of clones defined in the fractable does not match number
            of clones defined in parents (", nrow(fracTable), " vs ",
            length(parents), ").")
    }
    if(ncol(fracTable) != length(timepoints)){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", ncol(fracTable), 
            " vs ", length(timepoints), ").")
    }
    
    # nest levels
    nestLevels <- .getAllNestLevels(parents)

    ## Change the name of the clones to numbers (in fracTable)
    rownames(fracTable) <- seq_len(dim(fracTable)[1])

    ## timepoints as column names
    colnames(fracTable) <- timepoints

    # default clone labels are just Clone: + 1:numClones
    if (is.null(cloneLabels)) {
        cloneLabels <- paste('Clone:', as.character(seq_len(nrow(fracTable))))
        defaultLabels <- TRUE
    } else {defaultLabels <- FALSE}

    ## Check for single timepoints that timepoint estimation is true
    if (!timepointInterpolation & length(timepoints) == 1) {
        stop("To visualize clonal evolution from single timepoint
            timepointInterpolation mustbe set to TRUE")
    } else if (is.null(originTimepoint) & length(timepoints) == 1) {
        stop('To visualize clonal evolution from a single timepoint
            originTimepoint must be manually specified.')
    }

    # check origin timepoint
    if (is.null(originTimepoint)) {
        originTimepoint <- timepoints[1] - (timepoints[2] - timepoints[1])
    } else {
        if (!(is.numeric(originTimepoint))) {
            stop("originTimepoint must be a numeric value")
        }
        if (originTimepoint >= min(timepoints)) {
            stop("originTimepoint must be a timepoint before
                the first timepoint in the timepoints vector")
        }
    }

    ## Values lower than 0.1 are considered 0
    fracTable[which(fracTable < 0.1)] <- 0

    ## sanity checks on input data
    .validateInputs(fracTable, parents, cloneLabels)

    ## Therapy effect estimation
    if (!is.null(therapyEffect)) {
        fracTable <- .getTherapyEffect(fracTable = fracTable,
                                        timepoints = timepoints,
                                        parents = parents,
                                        therapyEffect = therapyEffect)

        timepoints <- as.numeric(colnames(fracTable))
    }


    ## Estimate new timepoints
    if (timepointInterpolation) {
        fracTable <- .estimateTimepoints(nestLevels = nestLevels,
                                        fracTable = fracTable,
                                        parents = parents,
                                        timepoints = timepoints,
                                        originTimepoint = originTimepoint,
                                        therapyEffect = therapyEffect)
        timepoints <- as.numeric(colnames(fracTable))
    }

    ## Recalculate timepoints if both (new timepoints,therapy effect) are
    ## estimated
    if (!is.null(therapyEffect) & timepointInterpolation) {
        timepoints <- .recalculateTimepoints(therapyEffect, timepoints)
        colnames(fracTable) <- timepoints
    }




    ## Change the name of the clones to numbers (in fracTable)
    rownames(fracTable) <- seq_len(dim(fracTable)[1])



    ## sanity checks on calculated data
    .validateInputs(fracTable, parents, cloneLabels)

    ## create the object
    seaObject <- new("seaObject", ytop = list(), ybtm = list(), col = c("NULL"),
                    timepoints = as.numeric(colnames(fracTable)),
                    fracTable = fracTable, parents = parents,
                    nestLevels = nestLevels, cloneLabels = cloneLabels,
                    defaultLabels = defaultLabels,
                    originTimepoint = originTimepoint
    )

    ## vector with each clones family
    seaObject <- .cloneFamVec(seaObject)

    # set default colors to start
    seaObject <- .setColors(seaObject, col = col)


    return(seaObject)
}

dolphinPlot <- function(seaObject, shape = 'spline', borderCol = NULL,
                        pos='center', vlines=NULL, vlineCol="#6E6E66",
                        vlab=NULL, vlabSize = 3,
                        separateIndependentClones = FALSE,
                        showLegend = FALSE, markMeasuredTimepoints = NULL,
                        main = NULL, mainPos = 'middle', mainSize = 5,
                        xlab = NULL, ylab = NULL, pad.left = 0.005,
                        annotations = NULL, annotSize = 3){
    if(is(seaObject, "seaObject") == FALSE){
        stop('No seaObject provided as input.')
    }
    if(!is.null(borderCol) && is.null(.isColor(borderCol))){
        stop("No valid input for borderCol provided.")
    }
    if(!is.null(vlines) && !is.numeric(vlines)){
        stop("No valid input for vlines provided.")
    }
    if(is.null(.isColor(vlineCol))){
        stop("No valid input for vlineCol provided.")
    }
    if(!is.null(vlab) && !is.character(vlab) && !is.numeric(vlab)){
        stop("No valid input for vlab provided.")
    }
    if(!is.numeric(vlabSize)){
        stop("No valid input for vlabSize provided.")
    }
    if(separateIndependentClones != TRUE && separateIndependentClones != FALSE){
        stop("No valid input for separateIndependentClones provided.")
    }
    if(showLegend != TRUE && showLegend != FALSE){
        stop("No valid input for showLegend provided.")
    }
    if(!is.null(markMeasuredTimepoints) && !is.numeric(markMeasuredTimepoints)){
        stop("No valid input for markMeasuredTimepoints provided.")
    }
    if(!is.null(main) && !is.character(main)){
        stop("No valid input for main provided.")
    }
    if(!is.numeric(mainSize)){
        stop("No valid input for mainSize provided.")
    }
    if(!is.null(xlab) && !is.character(xlab)){
        stop("No valid input for xlab provided.")
    }
    if(!is.null(ylab) && !is.character(ylab)){
        stop("No valid input for ylab provided.")
    }
    if(!is.null(pad.left) && !is.numeric(pad.left)){
        stop("No valid input for pad.left provided.")
    }
    if(!is.null(annotations) && !is.data.frame(annotations)){
        stop("No valid input for annotations provided.")
    }
    if(!is.numeric(annotSize)){
        stop("No valid input for annotSize provided.")
    }
    if(nrow(fracTable(seaObject)) != length(parents(seaObject))){
        stop("Number of clones defined in the fractable does not match number
            of clones defined in parents (", nrow(fracTable(seaObject)), " vs ",
            length(parents(seaObject)), ").")
    }
    if(ncol(fracTable(seaObject)) != length(timepoints(seaObject))){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", 
            ncol(fracTable(seaObject)), " vs ", length(timepoints(seaObject)), 
            ").")
    }
    
    x<-NULL
    y<-NULL
    name<-NULL
    ymin<-NULL
    ymax<-NULL
    tp<-NULL
    lab<-NULL

    ##get x and y values
    if (pos == 'center'){
        seaObject <- .layoutSeaObjectClones(seaObject,
                        separateIndependentClones = separateIndependentClones)
    } else if(pos== 'bottom'){
        seaObject <- .bottomLayout(seaObject,
                        separateIndependentClones = separateIndependentClones)
    } else if (pos == 'top'){
        seaObject <-.topLayout(seaObject,
                        separateIndependentClones = separateIndependentClones)
    } else{
        stop('Position (pos) must be either "center" or "bottom".',
                    pos,'is not a valid position.')
    }


    if (shape == 'polygon'){
        coordsTbl <- .getCoordsTable(seaObject)

    }else{
        if(shape == 'spline'){
            coordsTbl <- .getSmoothCoordsTbl(seaObject=seaObject,
                                            pad.left=pad.left)
        } else{
            stop('Shape must be wither "spline" or "polygon"')
        }
    }

    if(!is.null(borderCol)){
        dolphPlot <- ggplot(coordsTbl, aes(x = x, y = y)) +
            geom_polygon_interactive(aes(fill = name, group = name,
                                        tooltip = name, data_id = name),
                                    show.legend = showLegend,
                                    color = borderCol, size = 0.25
            )
    }else{
        dolphPlot <- ggplot(coordsTbl, aes(x = x, y = y)) +
            geom_polygon_interactive(aes(fill = name, group = name,
                                        tooltip = name, data_id = name),
                                    show.legend = showLegend
            )
    }

    dolphPlot <- dolphPlot +
        scale_fill_manual(values = seaObject@col, name = "Clone labels") +
        theme_void() + theme(legend.position = "left")


    if(!is.null(vlines)){
        vlinesData <- data.frame(vlines = vlines, ymin = rep(0,length(vlines)),
                                ymax = rep(100,length(vlines)))

        dolphPlot <- dolphPlot + geom_segment(mapping = aes(x = vlines,
                                                            xend=vlines,
                                                            y = ymin,
                                                            yend = ymax),
                                            data = vlinesData,
                                            color = vlineCol,
                                            show.legend = FALSE)

        if(!is.null(vlab)){
            vlinesData <- vlinesData %>% add_column(vlab = vlab)
            dolphPlot <- dolphPlot + geom_text(mapping = aes(x = vlines,
                                                            y = 100,
                                                            label = vlab,
                                                            vjust = -1),
                                            size = vlabSize,
                                            data = vlinesData) +
                expand_limits(y=100+vlabSize)
        }
    }

    if(!is.null(markMeasuredTimepoints)){
        measuredTpData <- data.frame(tp = markMeasuredTimepoints,
                                    y = rep(-2,
                                            length(markMeasuredTimepoints)))
        dolphPlot <- dolphPlot + geom_point(mapping = aes(x = tp, y = y),
                                            data = measuredTpData,
                                            shape = 'triangle',
                                            size = 2)
    }

    if(!is.null(main)){
        if(mainPos == 'middle'){
            mainCoords <- (min(seaObject@timepoints) +
                            max(seaObject@timepoints))/2
        } else{
            if(mainPos == 'left'){
                mainCoords <- min(seaObject@timepoints)

            }else{
                if(mainPos == 'right'){
                    mainCoords <- max(seaObject@timepoints)
                }else{
                    stop('main position (mainPos) must be "left",
                        "middle", or "right"')
                }
            }
        }

        mainY <- mainSize*2 + 100

        if(mainY < 110 & !is.null(vlab)){
            mainY <- 110
        }

        mainLabel <- data.frame(lab = main, x = mainCoords, y = mainY)

        dolphPlot <- dolphPlot + geom_text(mapping = aes(x = x, y = y,
                                                        label = lab),
                                                        size = mainSize,
                                                        data = mainLabel)
    }

    if(!is.null(xlab)){

        axisLabelX <- data.frame(lab = xlab,
                                x = ((min(seaObject@timepoints) +
                                max(seaObject@timepoints))/2),
                                y = -6)

        dolphPlot <- dolphPlot + geom_text(mapping = aes(x = x, y = y,
                                                        label = lab),
                                            data = axisLabelX)
    }

    if(!is.null(ylab)){
        pad <- max(coordsTbl$x)-min(coordsTbl$x)
        axisLabelY <- data.frame(lab = ylab, x = min(coordsTbl$x) - pad*0.18,
                                y = 50)
        yLine <- data.frame(x = min(coordsTbl$x) - pad*0.05, ymin = 0,
                            ymax =100)
        percentages <- data.frame(lab = '100%', x = min(coordsTbl$x) - pad*0.08,
                                y = 50)

        dolphPlot <- dolphPlot +
            geom_text(mapping = aes(x = x,y = y,label = lab,angle = 90),
                    data = axisLabelY) +
            geom_segment(mapping = aes(x = x, xend=x, y = ymin, yend = ymax),
                        data = yLine, color = 'black',
                        show.legend = FALSE, size = 0.2) +
            geom_text(mapping = aes(x = x, y = y, label = lab, angle = 90),
                    data = percentages, size=3)

    }

    ##Annotations
    if(!is.null(annotations)){
        if(sum(annotations$col %in% c('black','Black')) == 0){
            colorsScale <- 'white'
        } else {
            colorsScale <- c('black','white')
        }
        dolphPlot <- dolphPlot + geom_text(mapping = aes(x = x, y = y,
                                                        label = lab,
                                                        color = col),
                                        size = annotSize,
                                        data = annotations) +
            guides(color = 'none') +
            scale_color_discrete(type = colorsScale)
    }


    return(dolphPlot)
}

methods::setClass("seaObject",
                    representation(
                    ytop = "list",
                    ybtm = "list",
                    xpos = "list",
                    col = "character",
                    timepoints = "numeric",
                    fracTable = "matrix",
                    parents = "numeric",
                    nestLevels = "numeric",
                    cloneFamily = "numeric",
                    cloneLabels = "character",
                    defaultLabels = "logical",
                    originTimepoint = "numeric"
                    )
)

methods::setGeneric("ytop", function(x) standardGeneric("ytop"))
methods::setGeneric("ytop<-", 
                    function(x, value) standardGeneric("ytop<-"))

methods::setMethod("ytop", "seaObject", function(x) x@ytop)
methods::setMethod("ytop<-", "seaObject", function(x, value) {
    x@ytop <- value
    x
})

methods::setGeneric("ybtm", function(x) standardGeneric("ybtm"))
methods::setGeneric("ybtm<-", 
                    function(x, value) standardGeneric("ybtm<-"))

methods::setMethod("ybtm", "seaObject", function(x) x@ybtm)
methods::setMethod("ybtm<-", "seaObject", function(x, value) {
    x@ybtm <- value
    x
})

methods::setGeneric("xpos", function(x) standardGeneric("xpos"))
methods::setGeneric("xpos<-", 
                    function(x, value) standardGeneric("xpos<-"))

methods::setMethod("xpos", "seaObject", function(x) x@xpos)
methods::setMethod("xpos<-", "seaObject", function(x, value) {
    x@xpos <- value
    x
})

methods::setGeneric("col", function(x) standardGeneric("col"))
methods::setGeneric("col<-", 
                    function(x, value) standardGeneric("col<-"))

methods::setMethod("col", "seaObject", function(x) x@col)
methods::setMethod("col<-", "seaObject", function(x, value) {
    x@col <- value
    x
})

methods::setGeneric("timepoints", function(x) standardGeneric("timepoints"))
methods::setGeneric("timepoints<-", 
                    function(x, value) standardGeneric("timepoints<-"))

methods::setMethod("timepoints", "seaObject", function(x) x@timepoints)
methods::setMethod("timepoints<-", "seaObject", function(x, value) {
    x@timepoints <- value
    x
})

methods::setGeneric("fracTable", function(x) standardGeneric("fracTable"))
methods::setGeneric("fracTable<-", 
                    function(x, value) standardGeneric("fracTable<-"))

methods::setMethod("fracTable", "seaObject", function(x) x@fracTable)
methods::setMethod("fracTable<-", "seaObject", function(x, value) {
    x@fracTable <- value
    x
})

methods::setGeneric("parents", function(x) standardGeneric("parents"))
methods::setGeneric("parents<-", 
                    function(x, value) standardGeneric("parents<-"))

methods::setMethod("parents", "seaObject", function(x) x@parents)
methods::setMethod("parents<-", "seaObject", function(x, value) {
    x@parents <- value
    x
})

methods::setGeneric("nestLevels", function(x) standardGeneric("nestLevels"))
methods::setGeneric("nestLevels<-", 
                    function(x, value) standardGeneric("nestLevels<-"))

methods::setMethod("nestLevels", "seaObject", function(x) x@nestLevels)
methods::setMethod("nestLevels<-", "seaObject", function(x, value) {
    x@nestLevels <- value
    x
})

methods::setGeneric("cloneFamily", function(x) standardGeneric("cloneFamily"))
methods::setGeneric("cloneFamily<-", 
                    function(x, value) standardGeneric("cloneFamily<-"))

methods::setMethod("cloneFamily", "seaObject", function(x) x@cloneFamily)
methods::setMethod("cloneFamily<-", "seaObject", function(x, value) {
    x@cloneFamily <- value
    x
})

methods::setGeneric("cloneLabels", function(x) standardGeneric("cloneLabels"))
methods::setGeneric("cloneLabels<-", 
                    function(x, value) standardGeneric("cloneLabels<-"))

methods::setMethod("cloneLabels", "seaObject", function(x) x@cloneLabels)
methods::setMethod("cloneLabels<-", "seaObject", function(x, value) {
    x@cloneLabels <- value
    x
})

methods::setGeneric("defaultLabels", function(x) standardGeneric("defaultLabels"))
methods::setGeneric("defaultLabels<-", 
                    function(x, value) standardGeneric("defaultLabels<-"))

methods::setMethod("defaultLabels", "seaObject", function(x) x@defaultLabels)
methods::setMethod("defaultLabels<-", "seaObject", function(x, value) {
    x@defaultLabels <- value
    x
})

methods::setGeneric("originTimepoint", function(x) standardGeneric("originTimepoint"))
methods::setGeneric("originTimepoint<-", 
                    function(x, value) standardGeneric("originTimepoint<-"))

methods::setMethod("originTimepoint", "seaObject", function(x) x@originTimepoint)
methods::setMethod("originTimepoint<-", "seaObject", function(x, value) {
    x@originTimepoint <- value
    x
})

methods::setMethod("show","seaObject",
                    function(object)print(list(ytop = object@ytop,
                                                ybtm = object@ybtm,
                                                xpos = object@xpos,
                                                col = object@col,
                                                timepoints = object@timepoints,
                                                fracTable = object@fracTable,
                                                parents = object@parents,
                                                nestLevels = object@nestLevels,
                                                cloneFamily = object@cloneFamily,
                                                cloneLabels = object@cloneLabels,
                                                defaultLabels = object@defaultLabels,
                                                originTimepoint = object@originTimepoint)))

#' Validate some key assumptions about the seaObject's data
#'
#' @param fracTable A numeric matrix containing tumor fraction estimates for
#' all clones at all timepoints
#' @param parents An integer vector specifying parental relationships between
#' clones
#' @param cloneLabels A character vector of names to assign to each clone when
#' plotting a legend
#'
#' @return No return value - stops execution with an error if invalid inputs
#' are detected
#'

.validateInputs <- function(fracTable, parents, cloneLabels) {
    clones <-    seq_len(dim(fracTable)[1])
    timepts <- seq_len(dim(fracTable)[2])

    ##no cluster can go from present to absent and then back
    ##difference in clusters cannot go from present to absent and then back
    for (clone in clones) {
        startedClone <- FALSE
        endedClone <- FALSE
        startedDiff <- FALSE
        endedDiff <- FALSE

        for (timept in timepts) {
            ##check individual clones
            if (fracTable[clone, timept] > 0.000001) {
                if (startedClone & endedClone) {
                    stop("Clone", clone, "goes from present to absent
                        (fraction=0) and then back to present."
                    )
                }
                startedClone <- TRUE
            } else {
                if (startedClone) {
                    endedClone <- TRUE
                }
            }

            ##check difference in clusters
            diffr <-
                fracTable[clone, timept] -
                    sum(fracTable[which(parents == clone), timept])
            if (diffr > 0.0001) {
                if (startedDiff & endedDiff) {
                    stop("The difference between clone ",
                        clone,
                        "and its subclones
                        goes from present to absent (difference = 0) and
                        then back to present at timepoint",
                        timept,
                        ". Subclones can't have the same CCF as the parent
                        (difference = 0) and then have less
                        CCF again."
                    )
                }
                startedDiff <- TRUE
            } else {
                if (startedDiff) {
                    endedDiff <- TRUE
                }
            }
        }
    }

    ##clusters of entirely zero get a warning
    if (length(which(rowSums(fracTable) == 0)) > 0) {
        warning(
            "at least one cluster has fraction zero at all timepoints.
                        It will not be displayed"
        )
    }

    ##make sure that each timepoint doesn't sum to more than the parental value
    #at a given nest level (or 100% for level 0)
    for (timept in timepts) {
        independentClones <- which(parents == 0)
        if (sum(fracTable[independentClones, timept]) > 100.000001) {
            helper_temp <- paste(independentClones, collapse = ",")
            stop("clones with same nest level cannot have values that sum to
                more than 100%: Problem is in clusters ",helper_temp
            )
        }

        for (i in unique(parents)) {
            if (i > 0) {
                neighbors <- which(parents == i)
                if (sum(fracTable[neighbors, timept]) > fracTable[parents[neighbors[1]],timept] + 0.000001) {
                    helper_temp <- paste(neighbors,collapse = ",")
                    stop("clones with same parent cannot have values that
                        sum tomore than the percentage of the parent:
                        Problem is in clusters ", helper_temp,
                        " at timepoint ", timept
                    )
                }
            }
        }
    }

    ##number of clone labels is equal to the number of clones
    if (length(cloneLabels) != nrow(fracTable)) {
        stop("The number of cloneLabels provided must be equal to the
            number of clones")
    }

}


#' Fill the col slot of the seaObject
#'
#' @param seaObject A seaObject
#' @param col (optional) A vector with each clone's color
#'
#' @return A seaObject with the col slot filled

.setColors <- function(seaObject, col = NULL) {
    nclones <- nrow(seaObject@fracTable)
    if (!(exists("nclones"))) {
        warning(
            "Could not set colors, as the number of rows in the
            fracTable slot of the seaObject could not be calculated"
        )
        return(seaObject)
    }

    if (is.null(col)) {
        ##use default color scheme
        col <- .getRelatedColors(seaObject)

        seaObject@col <- col[seq_len(nclones)]
        return(seaObject)
    }

    ##else colors provided, check them for sanity
    if (length(col) != nrow(seaObject@fracTable)) {
        stop("number of colors provided must be equal to the number
            of clones (", nclones, ")"
        )
    }
    seaObject@col <- col
    return(seaObject)
}


#' Given an integer indicating the length of the longest branch, return a color
#' matrix with as many rows as the integer given
#'
#' @param lenBranch An integer vector specifying the length of the longest
#' branch in the clonal evolution tree
#'
#' @return A matrix of dimensions 25 columns (hue changes) and lenBranch rows
#' (darkness changes)
.getColorMatrix <- function(lenBranch) {
    ordr <- c(seq(18, 25), seq(1, 17))
    rbw <- rainbow_hcl(25, l = 85)[ordr]
    allColors <- c()
    for (col in rbw) {
        darkCol <- darken(col, 0.7)
        darkColors <- colorRampPalette(c(col, darkCol))(lenBranch)
        allColors <- c(allColors, darkColors)
    }
    colMatrix <- matrix(allColors, ncol = length(rbw), byrow = FALSE)

    return(colMatrix)
}



#' Given a seaObject, define the color of each clone taking into account the
#' phylogeny.
#'
#' @param seaObject A seaObject
#'
#' @return A character vector with each clones color
.getRelatedColors <- function(seaObject) {
    parents <- NULL

    lenBranch <- max(seaObject@nestLevels) + 1
    colorMatrix <- .getColorMatrix(lenBranch = lenBranch)
    nclones <- length(seaObject@cloneLabels)
    cloneCols <- rep(NA, nclones)
    parentClone <- -1
    parentX <- c(1, rep(NA, nclones))
    parentY <- c(0, rep(NA, nclones))
    cloneBranches <- .getAllBranches(seaObject@parents)

    ## first stablish the color for the initial clones (parent = 0)
    indepClones <- which(seaObject@parents == 0)
    if (length(indepClones) > 1) {
        step <-
            25 %/% length(indepClones) #as far away as possible in the matrix
        x <- 0
        for (clone in indepClones) {
            x <- x + step
            y <- 1
            if (x > 25) {
                x <- x %/% 25
            }
            cloneCols[clone] <- colorMatrix[y, x]
            parentX[clone + 1] <- x
            parentY[clone + 1] <- y
        }
        ##get which clones where not initial
        branchedClones <-
            seq_len(nclones)[-which(seq_len(nclones) %in% indepClones)]
    } else{
        branchedClones <- seq_len(nclones)
    }

    ##Deal with the non-initial clones
    for (clone in branchedClones) {
        parent <- seaObject@parents[clone]
        ##color change depends on the number of branches
        if (sum(seaObject@parents == parent) > 4) {
            branchStep <- 0
        } else {
            branchStep <- 2
        }
        ## If it's a branch (parent already in the vector)
        if (parent %in% parentClone) {
            counts <- sum(parentClone == parent) + branchStep
            modX <- FALSE
            ## alternate between the two sides of the matrix to variate the
            ## color, darken the color
            if (counts %% 2 == 1) {
                x <- parentX[parent + 1] + ((counts + 1) / 2)

                if (x %in% parentX) {
                    if (sum((which(parentX == x) - 1) %in% cloneBranches[[clone]]) == 0) {
                        parentsInBranch <-
                            which(names(table(parents)) %in% cloneBranches[[clone]])
                        counts <- counts +
                            sum(table(parents)[parentsInBranch] > 1)
                        if (counts %% 2 == 1) {
                            x <- parentX[parent + 1] +
                                ((counts + 1) / 2) + branchStep
                        } else{
                            x <- parentX[parent + 1] - (counts / 2) -
                                branchStep
                        }
                        modX <- TRUE
                    }
                }
            } else if (counts %% 2 == 0 & !modX) {
                x <- parentX[parent + 1] - (counts / 2)
                if (x %in% parentX) {
                    if (sum((which(parentX == x) - 1) %in% cloneBranches[[clone]]) == 0) {
                        parentsInBranch <-
                            which(names(table(parents)) %in% cloneBranches[[clone]])
                        counts <-
                            counts + sum(table(parents)[parentsInBranch] > 1)
                        x <- parentX[parent + 1] + (counts / 2)
                        modX <- TRUE
                    }
                }
            }
            ##For linear evolution (parent not in the vector) --> darken the
            ##color
        } else {
            x <- parentX[parent + 1]
        }
        y <- parentY[parent + 1] + 1

        ##Fix x and y values to be inside the matrix
        if (x > 25) {
            x <- x %/% 25
        }
        if (y > lenBranch) {
            y <- y %/% lenBranch
        }
        if (x == 0) {
            x <- 25
        }
        if (x < 0) {
            x <- 25 + x
        }


        ##check another clone doesn't have the same color
        repColor <- colorMatrix[y, x] %in% cloneCols
        if (repColor) {
            while (repColor) {
                if (x < 12) {
                    x <- x + 1 + branchStep
                } else {
                    x <- x - 1 - branchStep
                }

                repColor <- colorMatrix[y, x] %in% cloneCols
            }
        }

        cloneCols[clone] <- colorMatrix[y, x] ##Add color to the vector
        parentClone <-
            c(parentClone, seaObject@parents[clone]) #update parents vec
        parentX[clone + 1] <-
            x #update parents vectors with the clone x and y positions
        parentY[clone + 1] <- y
    }
    return(cloneCols)
}

#' Given the a list representing the parents of each clone, and the number
#' specifying which clone to test, returns how deeply it is nested
#'
#' @param parents An integer vector specifying parental relationships between
#' clones
#' @param x The integer specifying which subclone to calculate nest level for
#'
#' @return An integer representing how deeply this subclone is nested
#' @seealso .getAllNestLevels
.getNestLevel <- function(parents, x) {
    #sanity checks
    if (x > length(parents)) {
        stop("cannot have a parent that does not exist in list. parent =",
            x, ", length(parents) =", length(parents)
        )
    }
    if (x < 0) {
        stop("cannot have a value in parents of less than zero")
    }

    if (parents[x] == 0) {
        return(0)
    } else {
        return(.getNestLevel(parents, parents[x]) + 1)
    }
}



#' Given the a list representing the parents of each clone, return a vector
#' specifying how deeply each clone is nested
#'
#' @param parents An integer vector specifying parental relationships between
#' clones
#'
#' @return An integer vector representing how deeply each subclone is nested
#' @seealso .getNestLevel
#'
.getAllNestLevels <- function(parents) {
    nest.level <- c()
    for (i in seq_len(length(parents))) {
        nest.level <- c(nest.level, .getNestLevel(parents, i))
    }
    return(nest.level)
}



#' Given a seaObject, return the seaObject with the cloneFamily slot filled
#' with a numeric vector specifying which family each clone belongs to
#'
#' @param seaObject A seaObject
#'
#' @return A seaObject with the cloneFamily slot filled
#'
.cloneFamVec <- function(seaObject) {
    numFamilies <- sum(seaObject@parents == 0)
    listFams <- tail(.getFromTo(seaObject), numFamilies)
    cloneFam <- c()
    for (i in seq_len(length(seaObject@parents))) {
        for (j in seq_len(numFamilies)) {
            if (i %in% as.numeric(listFams[[j]])) {
                cloneFam <- c(cloneFam, j)
            }
        }
    }

    seaObject@cloneFamily <- cloneFam
    return(seaObject)
}



#'Given a cancer cell fraction table as a matrix,
#'return a matrix with (TRUE/FALSE) indicating each clone's possible parents
#'
#' @param fracTable a matrix with the cancer cell fraction of each clone at
#' each timepoint
#'
#' @return matrix with (TRUE/FALSE) indicating each clone's possible parents
#'
.possibleParents <- function(fracTable){
    ##limit options per position
    possible <- matrix(rep(TRUE, (nrow(fracTable)*nrow(fracTable))),
                    nrow = nrow(fracTable))
    row.names(possible) <- c(0:(nrow(fracTable)-1))

    fracTable[nrow(fracTable),] <- NA

    for(i in seq_len(ncol(fracTable))){
        while(sum(!is.na(fracTable[,i]))>1){
            small <- which.min(fracTable[,i])
            impossible <- which(fracTable[,i]>fracTable[small,i])
            for(j in impossible){
                possible[small+1,j] <- FALSE
            }
            fracTable[which.min(fracTable[,i]),i] <- NA
        }
    }
    for(i in seq_len(ncol(possible)-1)){
        possible[i+1,i] <- FALSE
    }

    return(possible)
}


#'validate when only one option for parent that it does not sum up to >100
#'
#' @param fracTable a matrix with the cancer cell fraction of each clone at
#' each timepoint
#' @param possible matrix with (TRUE/FALSE) indicating each clone's possible parents
#'
#' @return matrix with (TRUE/FALSE) indicating each clone's validated possible
#' parents
#'
.checkPossibleParents <- function(fracTable, possible){

    check<-which(colSums(possible)==1)
    checked<-c()
    while(!identical(check,checked)){
        for(i in setdiff(check,checked)){
            for(j in seq_len(ncol(fracTable))){
                compare_with<-fracTable[i,j]
                for(k in c(seq_len(length(fracTable[,j])))[-i]){
                    if((compare_with+fracTable[k,j])>100){
                        possible[which(possible[,i]==TRUE),k]<-FALSE
                    }
                }
            }
            checked<-c(checked,i)
        }
        check<-which(colSums(possible)==1)
    }

    possibilities<-list()
    for(i in seq_len(ncol(possible))){
        possibilities[[i]]<-names(possible[possible[,i]==TRUE,i])
    }
    return(possibilities)
}


#'Get the possible parental relations
#'
#' @param possibilities matrix with (TRUE/FALSE) indicating each clone's
#' validated possible parents
#' @param fracTable a matrix with the cancer cell fraction of each clone at
#' each timepoint
#' @param timepoints numeric vector with time points
#'
#' @return list of numeric vectors corresponding to the possible parental
#' relations
#'
.getParentOptions <- function(possibilities, fracTable, timepoints){
    temp<-tryCatch({
        combinations<-expand.grid(possibilities)
    }, error=function(e){
        return(NULL)
    })

    solutions <- list()
    found <- 1
    if(!is.null(temp)){
        if(nrow(combinations) < 20000){
            if(ncol(fracTable) == 1){

                for(i in seq_len(nrow(combinations))){
                    temp <- tryCatch({
                        x <- createSeaObject(
                        fracTable,
                        parents = as.numeric(t(as.matrix(combinations[i,]))),
                        timepoints,
                        timepointInterpolation = TRUE,
                        originTimepoint = -1)
                    }, error=function(e){
                        return(NULL)
                    })

                    if(!is.null(temp)){
                        solutions[[found]] <- as.numeric(t(as.matrix(combinations[i,])))
                        found <- found+1
                    }
                }
            }else{
                for(i in seq_len(nrow(combinations))){
                    temp <- tryCatch({
                        x <- createSeaObject(
                        fracTable,
                        parents = as.numeric(t(as.matrix(combinations[i,]))),
                        timepoints,
                        timepointInterpolation = TRUE)
                    }, error=function(e){
                        return(NULL)
                    })

                    if(!is.null(temp)){
                        solutions[[found]] <- as.numeric(t(as.matrix(combinations[i,])))
                        found <- found+1
                    }
                }
            }
            ##name vectors
            names(solutions) <- unlist(lapply(solutions,
                                        function(x) paste(x, collapse = ',')))

            return(solutions)
        }else{
            message("Too many possible options.")
        }
    }else{
        message("Too many possible options.")
    }
}





#### ESTIMATE TIMEPOINTS #######



#' get a list with clones present at a certain timepoint divided by families.
#'
#' @param parents A vector indicating the parental relations
#' @param fracTable Vector with CCFs of clones at a timepoint
#'
#' @return list with clones divided by families, first half of the list is
#' parent clones, second half are children clones.
#'
.getListFamilies <- function(parents, fracTable, listFam = NULL) {
    started <- FALSE #flag
    fam_n <- 1
    numFam <- sum(parents == 0) #number of expected families
    fam_v <- c()
    if (numFam == 1) {
        listFam <- which(fracTable != 0)
    }
    else{
        while (fam_n < numFam) {
            #which clones are present in the timepoint
            present <- which(fracTable != 0)
            for (clone in present) {
                parent <- parents[clone]

                #add vector to the list
                if (parent == 0 & started) {
                    listFam[[fam_n]] <- fam_v
                    fam_n <- fam_n + 1
                    fam_v <- c()
                    started <- FALSE
                }

                #start the vector
                if (parent == 0 & !(started)) {
                    started <- TRUE
                    fam_v <- c(fam_v, clone)
                }

                #Add clones to family vector while the lists are empty
                if (parent != 0 & started & length(listFam) == 0) {
                    fam_v <- c(fam_v, clone)
                }

                #if lists are not empty
                if (parent != 0 & started & length(listFam) > 0) {
                    ##idx is the position of the family where the clone belongs
                    idx <- detect_index(listFam, ~ parent %in% .)

                    #check the list to see if parent is there
                    if (idx != 0) {
                        listFam[[idx]] <- c(listFam[[idx]], clone)
                    }

                    if (idx == 0) {
                        fam_v <- c(fam_v, clone)
                    }

                }

            }

            #when we are done iterating add the remainings to the lists
            listFam[[fam_n]] <- fam_v
            fam_n <- fam_n + 1
        }
    }

    return(listFam)
}

#' given the first and last value and the length get a linear sequence vector
#'
#' @param first first value of the vector
#' @param last last value of the vector
#' @param length desired length of the vector (including given first and last
#' value)
#'
#' @return Numeric vector of length 'length' with linear sequence starting with
#' 'first' value and finishing with 'last'.
#' @seealso .interpolate
#' @examples
#' \donttest{
#' .linearseq(1,5,5)
#' .linearseq(90,10,3)
#' .linearseq(7,7,4)
#' }
.linearseq <- function(first, last, length) {
    # Compute step
    if (last > first) {
        c <- (last - first) / (length - 1)
        return(seq(first, last, c))
    } else if (last < first) {
        c <- (first - last) / (length - 1)
        return(rev(seq(last, first, c)))
    } else if (last == first) {
        return(rep(first, length))
    }
}


#' given a vector with NA values, fill in the empty slots so that, with the
#' filled slots it forms a linear sequence
#'
#' @param lt numeric vector with NA values
#'
#' @return Numeric vector with all NA slots filled
#' @seealso .linearseq
#' @examples
#' \donttest{
#' .interpolate(c(1,NA,NA,4,NA,8))
#' }
.interpolate <- function(lt) {
    # Find missing elements
    idx <- 1
    while (idx <= length(lt)) {
        if (is.na(lt[idx])) {
            # Found a gap (find the end of the missing sequence)
            begin_idx <- idx - 1
            end_idx <- idx + 1
            while (end_idx <= length(lt) & is.na(lt[end_idx])) {
                end_idx <- end_idx + 1
            }
            # Fill the gap with a linear sequence
            lt[begin_idx:end_idx] <-
                .linearseq(lt[begin_idx], lt[end_idx], end_idx - begin_idx + 1)
            # Update index
            idx <- end_idx + 1
        }
        else{
            idx <- idx + 1
        }

    }
    return(lt)
}


#' get the estimated CCFs and timepoints before the first measured timepoint
#'
#' @param listFam list of the present clones in the first measured timepoint
#' divided by families
#' @param originPoint A numeric value specifying the estimated timepoint where
#' the first clone emerged.
#' @param nestLevels An integer vector specifying how deeply a given clone is
#' nested in the overall hierarchy
#' @param nclones An integer value specifying the total number of clones
#' @param fracTable A numeric matrix containing tumor fraction estimates for
#' all clones at all timepoints
#' @param parents An integer vector specifying parental relationships between
#' clones
#'
#'
#' @return A list with the new estimated CCFs matrix and a vector with the new
#' estimated timepoints
#' @seealso .estimateTimepoints

.originTimepoints <-
    function(listFam,
            originPoint,
            nestLevels,
            nclones,
            fracTable,
            parents,
            timepoints) {
        newCCFtable <- data.frame(id = c(seq_len(nclones)))
        n <- unlist(listFam) #present clones in tp as vector
        n_l <- nestLevels[n] #nest levels of present clones
        numNewTp <-
            length(unique(n_l)) #number of new timepoints (before tp1)
        ##Vector with new estimated timepoints values
        newTimepoints <-
            .interpolate(c(originPoint, rep(NA, numNewTp - 1), timepoints[1]))

        ##k = {1,..,numNewTp} -> first estimated timepoint = originPoint,
        ##first clones with nestLevel = 0
        for (k in seq_len(numNewTp)) {
            newCCF <- c() #vector with new ccf
            for (l in seq_len(nclones)) {
                ##Case when: present clone, nest level is 0 and first new
                ##estimated tp
                if ((l %in% n) & nestLevels[l] == 0 & ncol(newCCFtable) == 1) {
                    newCCF <- c(newCCF, 0.1)
                }

                ##Case when: present clone and not first estimated tp
                else if ((l %in% n)    & ncol(newCCFtable) > 1) {
                    ## Clone wasn't present before
                    if (newCCFtable[l, ncol(newCCFtable)] == 0 &
                            nestLevels[l] == k - 1) {
                        newCCF <- c(newCCF, 0.1)
                    }
                    ##Clone was present before
                    else if (newCCFtable[l, ncol(newCCFtable)] > 0 |
                                    is.na(newCCFtable[l, ncol(newCCFtable)])) {
                        newCCF <- c(newCCF, NA)
                    }
                    ##Not its timepoint to emerge
                    else if (newCCFtable[l, ncol(newCCFtable)] == 0 &
                                    !(nestLevels[l] == k - 1)) {
                        newCCF <- c(newCCF, 0.0)
                    }
                }
                ##Case when: clone is not present
                else {
                    newCCF <- c(newCCF, 0.0)
                }

            }
            #Add the ccf calculated to the new fracTable
            newCCFtable <- newCCFtable %>% add_column(newCCF)

        }
        firstTpCCFs <- fracTable[, 1]
        ##calculate difference in ccfs
        absCCFs <- c()
        for (clone in seq_len(length(firstTpCCFs))) {
            children <- which(parents == clone)
            cloneCCF <- firstTpCCFs[clone] - sum(firstTpCCFs[children])
            absCCFs <- c(absCCFs, cloneCCF)
        }
        ##Add children's ccf to parent
        for (col in seq(2, ncol(newCCFtable))) {
            replaceCCF <- which(is.na(newCCFtable[, col]))
            for (clone in rev(replaceCCF)) {
                children <- which(parents == clone)
                newCCFtable[clone, col] <-
                    absCCFs[clone] + sum(newCCFtable[children, col])
            }
        }

        return(list(newccf = newCCFtable,
                                newtp = newTimepoints[-length(newTimepoints)]))
    }





#' get the estimated CCFs between two measured timepoints
#'
#' @param nclones An integer value specifying the total number of clones
#' @param newCCFtable A data frame with the previous estimated CCFs.
#' @param newClones A numeric vector specifying which of the present clones are
#' new in this measured timepoint
#' @param whichClones A numeric vector specifying which of the new clones
#* should
#' emerge in this estimated timepoint
#'
#'
#' @return A vector with the new estimated CCFs
#' @seealso .estimateTimepoints

.getNewCCFvector <-
    function(nclones, newCCFtable, newClones, whichClones) {
        newCCF <- c()
        for (clone in seq_len(nclones)) {
            #if it is already present
            if (newCCFtable[clone, ncol(newCCFtable)] > 0 |
                    is.na(newCCFtable[clone, ncol(newCCFtable)])) {
                newCCF <- c(newCCF, NA)
            }
            #if it is not present and should emerge
            else if (newCCFtable[clone, ncol(newCCFtable)] == 0 &
                            clone %in% newClones[whichClones]) {
                newCCF <- c(newCCF, 0.1)
            }
            #if it's not present and should remain not present
            else {
                newCCF <- c(newCCF, 0)
            }
        }
        return(newCCF)
    }



#' Get a new CCFs table with the estimated and measured CCFs
#'
#' @param nestLevels An integer vector specifying how deeply a given clone is
#' nested in the overall hierarchy
#' @param fracTable A numeric matrix containing tumor fraction estimates for
#' all clones at all timepoints
#' @param originPoint A numeric value specifying the estimated timepoint where
#' the first clone emerged
#' @param parents An integer vector specifying parental relationships between
#' clones
#' @param timepoints A numeric vector specifying the measured timepoints for
#' each column of the matrix
#'
#'
#' @return A matrix with the estimated and measured CCFs (and Timepoints as
#' column names)
#' @examples
#' \donttest {
#' .estimateTimepoints(nestLevels = c(0,1,2),
#'                     fracTable = matrix(c(20,10,0,50,30,25),ncol=2),
#' originTimepoint = -30, parents = c(0,1,2), timepoints = c(0,50))
#' }
.estimateTimepoints <-
    function(nestLevels,
                    fracTable,
                    originTimepoint,
                    parents,
                    timepoints,
                    therapyEffect = NULL) {
        nclones <- nrow(fracTable)
        presentClones <- c()
        cloneBranches <- .getAllBranches(parents)

        for (tp in seq_len(length(timepoints))) {
            prevClones <- presentClones
            listFam <- .getListFamilies(parents, fracTable[, tp])
            presentClones <- unlist(listFam)

            newClones <- presentClones[!presentClones %in% prevClones]
            #nestLevelsNewClones <- nestLevels[newClones]
            nestLevelsNewClones <- rep(NA, length(newClones))
            for (clone in seq_len(length(newClones))) {
                parentClone <- parents[newClones[clone]]
                if (!(parentClone %in% newClones)) {
                    nestLevelsNewClones[clone] <- 0
                } else {
                    parentNestLevel <-
                        nestLevelsNewClones[which(newClones == parentClone)]
                    nestLevelsNewClones[clone] <- parentNestLevel + 1
                }
            }
            numNewTp <- length(unique(nestLevelsNewClones))

            ##estimated timepoints before first measured tp
            if (tp == 1) {
                newValues <-
                    .originTimepoints(
                        listFam,
                        originPoint = originTimepoint,
                        nestLevels,
                        nclones,
                        fracTable,
                        parents = parents,
                        timepoints = timepoints
                    )
                newTimepoints <- newValues$newtp
                newCCFtable <- newValues$newccf

            } else{
                #for the rest of timepoints

                for (newTp in seq_len(numNewTp)) {
                    ##which clones (index) are the next to be added
                    whichClones <-
                        which(nestLevelsNewClones == min(nestLevelsNewClones))

                    ##the first new timepoint will be to modify the measured
                    ##one
                    if (newTp == 1) {
                        firstClones <- newClones[whichClones]
                        newCCFtable[firstClones, ncol(newCCFtable)] <- 0.1

                        ##Check that adding 0.1 to a child clone doesn't add to
                        ##more than parent clone overall

                for (clone in firstClones) {
                    for (inClone in rev(cloneBranches[[clone]])[seq_len(length(cloneBranches[[clone]]) - 1)]) {
                        if (parents[inClone] > 0) {
                            neighbours <- which(parents == parents[inClone])
                            neighboursCCF <-
                                sum(newCCFtable[neighbours, ncol(newCCFtable)])
                            parentCCF <-
                                newCCFtable[parents[inClone], ncol(newCCFtable)]
                            parentDiffCCF <- parentCCF - neighboursCCF
                            ##If it does increase 0.1 to parent
                            if (parentDiffCCF < 0) {
                                newCCFtable[parents[inClone], ncol(newCCFtable)] <-
                                    newCCFtable[parents[inClone], ncol(newCCFtable)] + 0.1
                            }
                        }
                    }
                }

                    } else{
                        ##for the rest of new timepoints loop through all
                        ##the clones
                        ##Estimate CCF vector
                        newCCF <- .getNewCCFvector(nclones, newCCFtable,
                                                    newClones, whichClones)

                        ##add the new column to the table
                        newCCFtable <- cbind(newCCFtable, newCCF)
                    }

                    ##update the list of clones to emerge
                    newClones <- newClones[-whichClones]
                    nestLevelsNewClones <-
                        nestLevelsNewClones[!nestLevelsNewClones == min(nestLevelsNewClones)]
                }

                ##Compute the new estimated timepoints values
                if (tp <= length(timepoints) & numNewTp > 1) {
                    btwTimepoints <- .linearseq(timepoints[tp - 1],
                                                timepoints[tp], numNewTp + 1)
                    newTimepoints <- c(newTimepoints,
                                        btwTimepoints[2:(length(btwTimepoints) - 1)])
                }

            }

            newCCFtable <- cbind(newCCFtable, fracTable[, tp])
            newTimepoints <- c(newTimepoints, timepoints[tp])

            if (tp > 1 & !(is.null(therapyEffect)) & numNewTp > 0) {
                ##therapyEffect is a vector with two timepoints
                if (length(therapyEffect) == 2) {
                    tpBefore <- therapyEffect[1]
                    tpAfter <- therapyEffect[2]
                    therapyEffectTP <- (tpBefore + tpAfter) / 2
                } else if (length(therapyEffect) == 1) {
                    therapyEffectTP <- therapyEffect
                    tpAfter <-
                        timepoints[which(timepoints == therapyEffectTP) + 1]
                }

                if (therapyEffectTP == timepoints[tp - 1] &
                        tpAfter == timepoints[tp]) {
                    newCCFtable <-
                        .getIntmCCF(nclones, parents, newCCFtable,
                                    fracTable, tp)
                } else{
                    newCCFtable <- t(apply(newCCFtable, 1, .interpolate))
                }
            } else{
                ##Fill the new fracTable with the intermediate values
                newCCFtable <- t(apply(newCCFtable, 1, .interpolate))
            }
        }

        newTimepoints <-    round(newTimepoints, 2)

        newCCFtable <- round(newCCFtable[, 2:ncol(newCCFtable)], 2)
        if (is.null(nrow(newCCFtable))) {
            newCCFtable <- matrix(newCCFtable, ncol = length(newCCFtable))
        }
        ##timepoints as column names
        colnames(newCCFtable) <- newTimepoints

        return(newCCFtable)
    }

.getIntmCCF <- function(nclones, parents, newCCFtable, fracTable, tp) {
        ##Fill in the NAs
        lastClones <- which(!(seq_len(nclones) %in% parents))
        ##Last clones assumed to evolve linear way
        if (length(lastClones) > 1) {
            CCFsLastClones <-
                t(apply(newCCFtable[lastClones, 2:ncol(newCCFtable)], 1,
                        .interpolate))
        } else {
            CCFsLastClones <-
                .interpolate(newCCFtable[lastClones, 2:ncol(newCCFtable)])
        }

        newCCFtable[lastClones, 2:ncol(newCCFtable)] <- CCFsLastClones


        ##calculate difference in ccfs
        absCCFs <- c()
        for (clone in seq_len(nclones)) {
            children <- which(parents == clone)
            cloneCCF <- fracTable[clone, tp] - sum(fracTable[children, tp])
            absCCFs <- c(absCCFs, cloneCCF)
        }
        ##Add children's ccf to parent
        for (col in seq(2, ncol(newCCFtable))) {
            replaceCCF <- which(is.na(newCCFtable[, col]))
            for (clone in rev(replaceCCF)) {
                children <- which(parents == clone)
                newCCFvalue <- absCCFs[clone] +
                                sum(newCCFtable[children, col])
                ##Keep the ccf if the estimated is 0 and the clone is present
                ##before and after
                if (newCCFvalue == 0 &
                    fracTable[clone, tp - 1] > 0 &
                    fracTable[clone, tp] > 0) {
                    newCCFtable[clone, col] <- fracTable[clone, tp - 1]
                } else {
                    newCCFtable[clone, col] <- newCCFvalue
                }
            }
        }

        return(newCCFtable)
    }

#### THERAPY EFFECT ######

#' Given a specific timepoint for estimating the therapy effect, get the before
#' and after measured timepoints
#'
#' @param therapyEffect A numeric value specifying the timepoint where to
#' calculate the estimated therapy effect
#' @param timepoints A numeric vector specifying the measured timepoints
#'
#' @return A list with two values: tpBefore (timepoint before) and tpAfter
#' (timepoint after)
#' @seealso .getTherapyEffect
.getTpBeforeAfter <- function(therapyEffect, timepoints) {
    i <- 1

    while (therapyEffect > timepoints[i]) {
        tpBefore <- timepoints[i]
        i <- i + 1
    }
    tpAfter <- timepoints[i]

    return(list(tpBefore = tpBefore, tpAfter = tpAfter))
}



#' Get which clones where present before therapy effect
#'
#' @param tpBefore A numeric value specifying the measured timepoint just
#' before the estimated therapy effect
#' @param fracTable A numeric matrix containing tumor fraction estimates for
#' all clones at all timepoints
#' @param timepoints A numeric vector specifying the measured timepoints
#'
#' @return A numeric vector with the indexes of the clones present before
#' therapy
#' @seealso .getTherapyEffect
.getClonesPresent <- function(tpBefore, fracTable, timepoints) {
    tp <- which(timepoints == tpBefore)

    return(which(fracTable[, tp] > 0))
}



#' Get a vector with the estimated therapy effect CCFs
#'
#' @param tpBefore A numeric value specifying the measured timepoint just
#' before the estimated therapy effect
#' @param tpAfter A numeric value specifying the measured timepoint just after
#' the estimated therapy effect
#' @param fracTable A numeric matrix containing tumor fraction estimates for
#' all clones at all timepoints
#' @param parents A numeric vector specifying the parental relations between
#' clones
#' @param presentClones An integer vector specifying the clones present during
#' therapy effect
#'
#' @return A numeric vector with the estimated therapy effect CCFs
#' @seealso .getTherapyEffect
.getTherapyCCF <- function(tpBefore, tpAfter,fracTable, parents,
                            presentClones, timepoints) {
        newCCF_vec <- c()
        indBefore <- which(timepoints == tpBefore)
        indAfter <- which(timepoints == tpAfter)
        checkMinTps <- c(indBefore, indAfter)
        ##Get CCF for clones present
        for (clone in seq_len(nrow(fracTable))) {
            newMinCCFs <- c()
            if (clone %in% presentClones) {
                for (tp in checkMinTps) {
                    children <- which(parents == clone)
                    CCFchildren <- sum(fracTable[children, tp])
                    newCCF <- fracTable[clone, tp] - CCFchildren
                    newMinCCFs <- c(newMinCCFs, newCCF)
                }
                newCCF_vec <- c(newCCF_vec, min(newMinCCFs))
            } else{
                newCCF_vec <- c(newCCF_vec, 0)
            }
        }
        #Update CCF for plotting (add present children ccf)
        for (clone in rev(seq_len(nrow(fracTable)))) {
            children <- which(parents == clone)
            newCCF_vec[clone] <-
                newCCF_vec[clone] + sum(newCCF_vec[children])
        }

        for (clone in presentClones) {
            if (newCCF_vec[clone] == 0 & fracTable[clone, indAfter] > 0) {
                newCCF_vec[clone] <- 0.2
            }
        }

        return(newCCF_vec)
    }



#' Get a vector with the estimated therapy effect CCFs
#'
#' @param fracTable A numeric matrix containing tumor fraction estimates for
#' all clones at all timepoints
#' @param timepoints A numeric vector specifying the measured timepoints
#' @param parents A numeric vector specifying the parental relations between
#' clones
#' @param therapyEffect A numeric value specifiying the timepoint on which to
#' estimate the therapy effect or a numeric vector with length=2 specifying
#' the consecutive measured timepoints between which to estimate
#' the therapy effect.
#'
#' @return A matrix with the estimated therapy effect CCFs and the measured
#' ones.
#' With timepoints as colnames.
#' @examples
#' \dontrun{
#' .getTherapyEffect(fracTable = matrix(c(90,10,0,47,35,30), ncol = 2),
#' timepoints = c(0,50), parents = c(0,1,2), therapyEffect = 25)
#' }
.getTherapyEffect <- function(fracTable, timepoints, parents, therapyEffect) {
        ##therapyEffect is a vector with two timepoints
        if (length(therapyEffect) == 2) {
            tpBefore <- therapyEffect[1]
            tpAfter <- therapyEffect[2]
            if (!(tpBefore %in% timepoints)) {
                stop('The first timepoint provided in therapyEffect is not a
                    valid timepoint. ', tpBefore,
                    ' is not a measured timepoint'
                )
            } else if (!(tpAfter %in% timepoints)) {
                stop('The second timepoint provided in therapyEffect is not a
                    valid timepoint. ', tpAfter, ' is not a measured timepoint'
                )
            } else{
                therapyEffectTP <- (tpBefore + tpAfter) / 2
            }

            ##therapyEffect is a single timepoint where to estimate the therapy effect
        } else if (length(therapyEffect == 1)) {
            if (therapyEffect > max(timepoints) |
                    therapyEffect < min(timepoints)) {
                stop('The therapyEffect value must be between two measured
                    timepoints. ', therapyEffect, ' is not a valid therapy
                    effect timepoint.'
                )
            } else {
                ##Timepoints before and after the therapy effect estimation tp
                tpBefore <- .getTpBeforeAfter(therapyEffect = therapyEffect,
                                            timepoints = timepoints)$tpBefore
                tpAfter <- .getTpBeforeAfter(therapyEffect = therapyEffect,
                                            timepoints = timepoints)$tpAfter
                therapyEffectTP <- therapyEffect
            }

        } else{
            stop('therapyEffect must be either a vector with two values
                (two measured timepoints) or a estimated therapy effect
                timepoint ', therapyEffect, ' is not a valid value'
            )
        }


        ##Clones present in the timepoints before
        presentClones <-
            .getClonesPresent(tpBefore = tpBefore,
                                fracTable = fracTable,
                                timepoints = timepoints)


        #CCFs of all clones at the therapy effect timepoint
        newCCF_vec <-
            .getTherapyCCF(
                tpBefore = tpBefore,
                tpAfter = tpAfter,
                fracTable = fracTable,
                parents = parents,
                presentClones = presentClones,
                timepoints = timepoints
            )

        ##Update timepoints and frac table
        newTimepoints <-
            insert(timepoints,
                        ats = which(timepoints == tpAfter),
                        values = therapyEffectTP)

        newFracTable <-
            as.data.frame(fracTable) %>% add_column(newCCF_vec,
                            .before = which(timepoints == tpAfter))

        newFracTable <- as.matrix(newFracTable)
        colnames(newFracTable) <- newTimepoints

        return(newFracTable)
    }




#' When therapy effect and timepoint estimation happen between the same measured
#' timepoints,
#' recalculate the estimated timepoints so there is equidistant development
#'
#' @param therapyEffect A numeric value specifiying the timepoint on which to
#' estimate the therapy effect
#'     or a numeric vector with length=2 specifying the consecutive measured
#'     timepoints between which to estimate
#'     the therapy effect.
#' @param timepoints A numeric vector specifying the timepoints for each column
#' of the matrix
#'
#' @return A numeric vector with the measured and estimated timepoints

.recalculateTimepoints <- function(therapyEffect, timepoints) {
    ##only if therapy effect is given as two measured timepoints
    if (length(therapyEffect) == 2) {
        tpIndex1 <- which(round(timepoints, 2) == therapyEffect[1])
        tpIndex2 <- which(round(timepoints, 2) == therapyEffect[2])
        intermediateTps <- tpIndex2 - tpIndex1

        if (intermediateTps > 2) {
            for (i in seq(tpIndex1 + 1, tpIndex2 - 1)) {
                timepoints[i] <- NA
            }

            timepoints <- round(.interpolate(timepoints), 2)
        }
    }
    return(timepoints)
}



#### SHARK + DOLPHIN ##########


#' get the shark plot layout of all non-independent clones
#'
#' @param seaObject A sea object
#'
#' @return plot with all relations showed as grey lines and white clones
#' @seealso sharkPlot
#'
.basicSharkPlot <- function(seaObject) {
    from <- c()
    to <- c()
    clones <- rownames(seaObject@fracTable)
    for (i in seq_along(seaObject@parents)) {
        parent <- seaObject@parents[i]
        if (parent != 0) {
            from <- c(from, clones[parent])
            to <- c(to, clones[i])
        }
    }

    ## create an edge list data frame giving the hierarchical structure of
    ##the clones
    edges <- data.frame(from, to)

    ## Create a graph object
    mygraph <- graph_from_data_frame(edges)

    ##create the interactive plot
    p <- ggraph(mygraph, layout = 'tree', circular = FALSE,) +
        geom_edge_elbow(color = 'light grey',
                                        edge_width = 2,
                                        strength = 0.8) +
        geom_point(
            size = 8,
            mapping = aes(x = .data$x, y = .data$y),
            show.legend = FALSE,
            color =    'white'
        ) +
        theme_void() + scale_y_reverse(expand = c(0.25, 0.25)) +
        coord_flip() +
        scale_x_reverse(expand = c(0.25, 0.25))

    return(p)
}




#' get a list divided with clones divided by families.
#'
#' @param seaObject A sea object
#'
#' @return list with clones divided by families, first half of the list is
#' parent clones, second half are children clones.
#'
.getFromTo <- function(seaObject) {
    p <- seaObject@parents
    c <- rownames(seaObject@fracTable)
    numTrees <- sum(p == 0)
    from <- c()
    to <- c()
    started <- FALSE #flag
    l_from <- list() #list with from vectors
    l_to <- list() #list with to vectors
    tree <- 1

    while (tree <= numTrees) {
        for (i in seq(1, length(p))) {
            #iterate along parents indexes

            parent <- p[i]

            ##when we reach a clone with parent 0 and we already started a
            ##family save the vectors to the list and reset
            if (parent == 0 & started) {
                l_from[[tree]] <- from
                l_to[[tree]] <- to
                tree <- tree + 1
                from <- c()
                to <- c()
                started <- FALSE
            }

            ##When we reach a clone with parent 0 start + add 0 (from) and
            ##parent clone (to)
            if (parent == 0 & !(started)) {
                started <- TRUE
                from <- c(from, 0)
                to <- c(to, c[i])
            }

            ##Add clones to from and to vectors while the lists are empty
            if (parent != 0 & started & length(l_from) == 0) {
                from <- c(from, c[parent])
                to <- c(to, c[i])
            }

            ##if lists are not empty
            if (parent != 0 & started & length(l_from) > 0) {
                ##idx is the position of the family where the clone belongs
                idx <- detect_index(l_to, ~ parent %in% .)

                ##if it belongs to a family --> append to the family
                if (idx != 0) {
                    l_from[[idx]] <- c(l_from[[idx]], c[parent])
                    l_to[[idx]] <- c(l_to[[idx]], c[i])
                }

                ##if it doesn't belong to any known family add to the new one
                if (idx == 0) {
                    from <- c(from, c[parent])
                    to <- c(to, c[i])
                }
            }
        }

        ##when we are done iterating add the remainings to the lists
        l_from[[tree]] <- from
        l_to[[tree]] <- to
        tree <- tree + 1

    }
    return(c(l_from, l_to))
}



#' get the independent clones labels and the count of how many independent
#' clones are there.
#'
#' @param numTrees An integer indicating how many families of clones are there
#' @param l_to A list with children clones divided by families.
#'
#' @return list with the independent clones labels and the count of how many
#' independent clones are there.
#' @seealso sharkPlot
.getIndependentCloneLabels <- function(numTrees, l_to) {
    count <- 0
    indep_clone_labs <- c()
    for (g in seq_len(numTrees)) {
        grp <- l_to[[g]]
        if (length(grp) == 1) {
            count <- count + 1
            indep_clone_labs <- c(indep_clone_labs, grp)
        }
    }
    returnList <-
        list('cloneLabs' = indep_clone_labs, 'count' = count)

    return(returnList)
}



#' get x and y positions for plotting independent clones
#'
#' @param count An integer indicating how many independent clones are there
#' @param parentTbl data frame containing x, y and clone label of the
#' non-independent clones
#' @param intPlot basic plot without the independent clones
#'
#' @return list with the independent clones x and y positions
#' @seealso sharkPlot
.getIndependentCloneXY <- function(count, parentTbl, intPlot) {
    indep_clone_ypos <- c()
    indep_clone_xpos <- c()
    for (c in seq_len(count)) {
        if (nrow(parentTbl) == 1) {
            min_xpos <- min(c(intPlot$data$x, indep_clone_xpos))
            indep_clone_xpos <- c(indep_clone_xpos, min_xpos - 1)
        }
        else{
            min_xpos <- min(c(intPlot$data$x, indep_clone_xpos))
            xpos <- min_xpos -
                (tail(parentTbl$x, 1) - abs(tail(parentTbl$x, 2)[1]))
            indep_clone_xpos <- c(indep_clone_xpos, xpos)

        }
    }
    indep_clone_ypos <- rep.int(max(intPlot$data$y), count)

    returnList <- list('x' = indep_clone_xpos, 'y' = indep_clone_ypos)

    return(returnList)
}




#' shark plot when all clones are independent
#'
#' @param cloneLabels Independent clones labels
#' @param ccf Vector with the clones CCFs at a specific timepoint
#' @param showLegend Boolean - whether to show the legend or not
#' @param cloneCols Vector with each clone colors
#'
#'
#' @return basic shark plot with all independent clones
#' @examples
#' \donttest {
#' .indepClonesPlot(cloneLabels = c('A','B','C'), showLegend = T,
#'                  cloneCols = c("#B0D8FF","#83B2D7","#588CAF"))
#' }
#' @seealso sharkPlot
.indepClonesPlot <-
    function(cloneLabels, showLegend = FALSE, cloneCols) {
        name <- as.factor(cloneLabels)
        x <- rep(0, length(cloneLabels))
        y <- seq(length(cloneLabels)) - 1
        data <- data.frame(name, x, y)

        p <- ggplot(data) +
            geom_point_interactive(
                size = 10,
                mapping = aes(
                    x = x,
                    y = y,
                    color = name,
                    tooltip = name,
                    data_id = name
                ),
                show.legend = showLegend
            ) +
            scale_colour_manual(values = cloneCols,
                                name = "Clone labels") + theme_void()

        return(p)
    }



###DOLPHIN

#' Generate key points of the layout that will be used for plotting with center
#' visualization
#'
#' @param seaObject A sea object with appropriate slots filled
#' (fracTable, parents, nestLevels)
#' @param separateIndependentClones Boolean - Should independently-arising
#' clones (with parent 0) be separated by blank space in the plot?
#'
#' @return A sea object with layout slots filled in
#'
#' @examples
#' \dontrun{
#' .layoutSeaObjectClones(seaObject)
#'
#' .layoutSeaObjectClones(seaObject, separateIndependentClones = TRUE)
#'
#' }
#'
.layoutSeaObjectClones <- function(seaObject, separateIndependentClones) {
        innerSpace <- lapply(rownames(seaObject@fracTable), .getInnerSpace,
                            seaObject)
        outerSpace <- .getOuterSpace(seaObject)

        ytop.vec <- c()
        ybtm.vec <- c()
        xpos.vec <- c()

        ##for each timepoint
        for (timepos in seq_len(length(seaObject@timepoints))) {
            timepoint <- seaObject@timepoints[timepos]

            ytop <- rep(NA, length(seaObject@parents))
            ybtm <- rep(NA, length(seaObject@parents))
            xpos <- rep(timepoint, length(seaObject@parents))

            ##starting with those with no parents, then moving through each
            ##existing parent
            parentsList <- 0
            while (length(parentsList) > 0) {
                parent <- parentsList[[1]]
                children <- which(seaObject@parents == parent)
                parentsList <- parentsList[-1]
                parentsList <- c(parentsList, children)
                numChildren <- length(children)
                spacing <- 0
                ##start at the bottom plus half the outer space
                y <- outerSpace[timepos] / 2


                ## (unless we are separating indpendent clones, in which case
                ## we divide outer spacing info in between and around
                if (separateIndependentClones & parent == 0) {
                    numZeros <- length(which(seaObject@parents == 0))
                    if (numZeros > 1 & outerSpace[timepos] > 0) {
                        spacing <- (outerSpace[timepos] / (numZeros + 1)) * 0.05
                    }

                }
                if (parent != 0) {
                    ##consider inner spacing if this is a subclone
                    y <- ybtm[parent]
                    spacing <- innerSpace[[parent]][timepos] /
                                            (.getNumChildren(seaObject, parent,
                                                            timepos) + 1)
                }

                ##for each clone that has this parent, get coords
                for (clone in children) {
                    ##clone absent, don't need to add positions
                    if (seaObject@fracTable[clone, timepos] == 0) {
                        xpos[clone] <- NA
                        ##smooth ending to dying clones
                        if (timepos > 1) {
                            if (seaObject@fracTable[clone, timepos - 1] > 0) {
                                ybtm[clone] <- y + spacing / 2
                                ytop[clone] <- y + spacing / 2
                                xpos[clone] <- timepoint - 0.25
                            }
                        }
                    } else {
                        #clone is still here, deal with it
                        ybtm[clone] <- y + spacing
                        y <- y + seaObject@fracTable[clone, timepos]
                        ytop[clone] <- y + spacing
                        y <- y + spacing
                    }
                }
            }
            ybtm.vec <- c(ybtm.vec, ybtm)
            ytop.vec <- c(ytop.vec, ytop)
            xpos.vec <- c(xpos.vec, xpos)
        }

        ##turn coords into a matrix so that we go by clone instead of by
        ##timepoint
        ybtm <- matrix(ybtm.vec, ncol = ncol(seaObject@fracTable))
        ytop <- matrix(ytop.vec, ncol = ncol(seaObject@fracTable))
        xpos <- matrix(xpos.vec, ncol = ncol(seaObject@fracTable))

        ybtm.list <- list()
        ytop.list <- list()
        xpos.list <- list()

        ##now, split into lists per clone
        for (i in seq_len(nrow(seaObject@fracTable))) {
            ybtm.list[[i]] <- ybtm[i, !is.na(ybtm[i, ])]
            ytop.list[[i]] <- ytop[i, !is.na(ytop[i, ])]
            xpos.list[[i]] <- xpos[i, !is.na(xpos[i, ])]
        }

        seaObject@ybtm <- ybtm.list
        seaObject@ytop <- ytop.list
        seaObject@xpos <- xpos.list

        return(seaObject)
    }



#' Given a sea object generate key points of the layout that will be used for
#' plotting with bottom visualization
#'
#' @param seaObject A sea object with appropriate slots filled
#' (fracTable, parents, nestLevels)
#' @param separateIndependentClones Boolean - Should independently-arising
#' clones (with parent 0) be separated by blank space in the plot?
#'
#' @return A sea object with layout slots filled in
#'
#' @examples
#' \dontrun{
#' .bottomLayout(seaObject)
#'
#' .bottomLayout(seaObject, separateIndependentClones = TRUE)
#' }
.bottomLayout <- function(seaObject, separateIndependentClones = FALSE) {
        cloneFam <- seaObject@cloneFamily ##vector with each clones family
        cloneBranches <- .getAllBranches(seaObject@parents)
        clonesOrder <- .getClonesOrder(
            nestLevels <- seaObject@nestLevels,
            parents <- seaObject@parents,
            cloneFam <- cloneFam
        )$clonesOrder
        deepestClone <- .getClonesOrder(
            nestLevels <- seaObject@nestLevels,
            parents <- seaObject@parents,
            cloneFam <- cloneFam
        )$deepestClone

        innerSpace <- lapply(rownames(seaObject@fracTable), .getInnerSpace,
                            seaObject)
        outerSpace <- .getOuterSpace(seaObject)

        ytop.vec <- c()
        ybtm.vec <- c()
        xpos.vec <- c()

        ##for each timepoint
        for (timepos in seq_len(length(seaObject@timepoints))) {
            timepoint <- seaObject@timepoints[timepos]

            ytop <- rep(NA, length(seaObject@parents))
            ybtm <- rep(NA, length(seaObject@parents))
            xpos <- rep(timepoint, length(seaObject@parents))

            for (parent in clonesOrder) {
                if (parent == 0) {
                    parentClones <- which(seaObject@parents == 0)
                    children <-
                        c(clonesOrder[2],
                        parentClones[-which(parentClones == clonesOrder[2])])
                } else{
                    children <-
                        clonesOrder[sort(match(which(seaObject@parents == parent),
                                    clonesOrder))]
                }
                spacing <- 0
                ##start at the bottom
                y <- 0

                if (parent != 0) {
                    ##consider inner spacing if this is a subclone
                    y <- ybtm[parent]
                    spacing <- innerSpace[[parent]][timepos] /
                                        (.getNumChildren(seaObject, parent,
                                                        timepos) + 1)
                }

                ##for each clone that has this parent, get coords
                for (clone in children) {
                    ##clone absent, don't need to add positions
                    if (seaObject@fracTable[clone, timepos] == 0) {
                        xpos[clone] <- NA
                        ##smooth ending to dying clones
                        if (timepos > 1) {
                            if (seaObject@fracTable[clone, timepos - 1] > 0) {
                                if (clone %in% cloneBranches[[deepestClone]]) {
                                    ybtm[clone] <- y
                                    ytop[clone] <- y + spacing / 2
                                    xpos[clone] <- timepoint - 0.25
                                } else{
                                    ybtm[clone] <- y + spacing / 2
                                    ytop[clone] <- y + spacing / 2
                                    xpos[clone] <- timepoint - 0.25
                                }
                            }
                        }
                    } else {
                        #clone is still here, deal with it
                        #Bottom family --> bottom visualization
                        if (separateIndependentClones & parent == 0) {
                            spacing <- outerSpace[timepos]
                            ybtm[clone] <- y
                            y <- y + seaObject@fracTable[clone, timepos]
                            ytop[clone] <- y
                            y <- y + spacing * 0.05
                        } else{
                            if (clone %in% cloneBranches[[deepestClone]]) {
                                ybtm[clone] <- y
                                y <- y + seaObject@fracTable[clone, timepos]
                                ytop[clone] <- y
                                y <- y + spacing
                            } else{
                                ##other families --> normal viz
                                ybtm[clone] <- y + spacing
                                y <- y + seaObject@fracTable[clone, timepos]
                                ytop[clone] <- y + spacing
                                y <- y + spacing
                            }
                        }


                    }
                }
            }
            ybtm.vec <- c(ybtm.vec, ybtm)
            ytop.vec <- c(ytop.vec, ytop)
            xpos.vec <- c(xpos.vec, xpos)
        }


        ##turn coords into a matrix so that we go by clone instead of by
        ##timepoint
        ybtm <- matrix(ybtm.vec, ncol = ncol(seaObject@fracTable))
        ytop <- matrix(ytop.vec, ncol = ncol(seaObject@fracTable))
        xpos <- matrix(xpos.vec, ncol = ncol(seaObject@fracTable))


        ybtm.list <- list()
        ytop.list <- list()
        xpos.list <- list()

        ##now, split into lists per clone
        for (i in clonesOrder[2:length(clonesOrder)]) {
            ybtm.list[[i]] <- ybtm[i, !is.na(ybtm[i, ])]
            ytop.list[[i]] <- ytop[i, !is.na(ytop[i, ])]
            xpos.list[[i]] <- xpos[i, !is.na(xpos[i, ])]
        }

        seaObject@ybtm <- ybtm.list
        seaObject@ytop <- ytop.list
        seaObject@xpos <- xpos.list

        return(seaObject)
    }




#' get the number of non-zero children at a particular timepoint
#'
#' @param seaObject A sea object
#' @param clone An integer representing the clone number to check
#' @param timepoint The timepoint at which to check
#'
#' @return the number of children with non-zero fractions
#'
.getNumChildren <- function(seaObject, clone, timepoint) {
    if (clone == 0) {
        return(0)
    }
    return(length(which(seaObject@fracTable[which(seaObject@parents == clone), timepoint] >
                                                0)))
}


#' Get the percentage of a clone that is only that clone and not occupied by
#' subclones
#'
#' @param clone The number of the clone to check
#' @param seaObject A fish object
#'
#' @return A number giving the percentage of this clone that is uniquely this
#' clone
.getInnerSpace <- function(clone, seaObject) {
    total <- seaObject@fracTable[as.numeric(clone), ]
    for (i in which(seaObject@parents == clone)) {
        total <- total - seaObject@fracTable[i, ]
    }
    return(total)
}

#' Get the amount of non-tumor space outside of all clones
#'
#' @param seaObject A sea object
#'
#' @return A numeric vector representing non-tumor space at each timepoint
#'
.getOuterSpace <- function(seaObject) {
    ##return the sums of all clones with parents of 0 at each timepoint
    z <- seaObject@fracTable[which(seaObject@parents == 0), ]
    if (is.vector(z)) {
        #only one row, just return it
        return(100 - z) ##Max minus parent clones CCFs
    }
    return(100 - colSums(z))
}




#' Get X and Y coordinates for polygon shape for all clones
#'
#' @param seaObject A sea object
#'
#' @return A data frame with the columns id, name (= clone Label), x and y

.getCoordsTable <- function(seaObject) {
    nclones <- nrow(seaObject@fracTable)
    y <- c()
    id <- c()
    x <- c()
    name <- c()
    for (clone in seq_len(nclones)) {
        cloneY <- c(seaObject@ytop[[clone]], rev(seaObject@ybtm[[clone]]))
        id <- c(id, rep(clone, length(cloneY)))
        x <- c(x, seaObject@xpos[[clone]], rev(seaObject@xpos[[clone]]))
        y <- c(y, cloneY)
        name <-
            c(name, rep(seaObject@cloneLabels[clone], length(cloneY)))
    }


    coordsTbl <- data.frame(
        id = as.factor(id),
        name = factor(name, levels = seaObject@cloneLabels),
        x = x,
        y = y
    )

    return(coordsTbl)
}





#' Get X and Y coordinates for spline shape for one clone
#'
#' @param xpos A vector of x values for control points
#' @param ytop A vector of y values for control points on the top
#' @param ybtm A vector of y values for control points on the bottom
#' @param pad.left A numeric amount of extra padding to add to the left side
#' of the shape
#' @param cloneLab A character value corresponding to the clone label
#'
#' @return A data frame with the columns name (= clone Label), x and y
#' @examples
#' \dontrun{
#' .getCloneCoords(xpos=c(0,30,75,150), ytop=c(100,51,51,99), ybtm=c(0,49,49,1),
#' cloneLab='Clone1',)
#' }
.getCloneCoords <- function(xpos, ytop, ybtm, pad.left = 0, cloneLab) {
    range <- max(xpos) - min(xpos)
    flank <- range * 0.0001

    xpos <- c(rbind(xpos - flank * 2, xpos - flank, xpos, xpos + flank, xpos +
                    flank * 2))
    #extra control points (x axis) around measured timepoint

    ybtm <- c(rbind(ybtm, ybtm, ybtm, ybtm, ybtm))
    ytop <- c(rbind(ytop, ytop, ytop, ytop, ytop))

    #Starting point (x,y)
    xst <- xpos[1] - pad.left
    yst <- (ytop[1] + ybtm[1]) / 2

    xst <- c(xst - flank * 2, xst, xst + flank * 2)
    yst <- c(yst, yst, yst)

    #top line
    top <- spline(c(xst, xpos), c(yst, ytop), n = 70)
    btm <- spline(c(xst, xpos), c(yst, ybtm), n = 70)
    #x and y coordinates for bottom line

    cloneName_vec <- rep(cloneLab, length(c(top$x, rev(btm$x))))
    coordsTbl <- data.frame(name = cloneName_vec,
                            x = c(top$x, rev(btm$x)),
                            y = c(top$y, rev(btm$y)))

    return(coordsTbl)
}




#' Get X and Y coordinates for spline shape for all clones
#'
#' @param seaObject A sea object
#' @param pad.left A numeric amount of extra padding to add to the left side
#' of the shape
#'
#' @return A data frame with the columns name (= clone Label), x and y

.getSmoothCoordsTbl <- function(seaObject, pad.left) {
    smoothCoords <- NULL
    pad <- (max(seaObject@timepoints) - min(seaObject@timepoints)) * pad.left

    for (clone in seq_len(length(seaObject@cloneLabels))) {
        cloneCoords <- .getCloneCoords(
            xpos = seaObject@xpos[[clone]],
            ytop = seaObject@ytop[[clone]],
            ybtm = seaObject@ybtm[[clone]],
            pad.left = pad,
            cloneLab = as.factor(seaObject@cloneLabels[clone])
        )

        smoothCoords <- rbind(smoothCoords, cloneCoords)
    }

    return(smoothCoords)
}




#### PLAICE PLOT ######


##Get branches from each clone to 0
#' Given the a list representing the parents of each clone, and the number
#' specifying which clone to test,
#' returns a vector with all its ancestral clones from the same branch
#'
#' @param parents An integer vector specifying parental relationships between
#' clones
#' @param x The integer specifying which subclone to get the branch of
#'
#' @return An integer vector with all the clone's ancestral clones from the
#' same branch
#' @seealso .getAllBranches
#'
.getBranch <- function(parents, x) {
    #sanity checks
    if (x > length(parents)) {
        stop("cannot have a parent that does not exist in list. parent =",
            x,", length(parents) =",length(parents)
        )
    }
    if (x < 0) {
        stop("cannot have a value in parents of less than zero")
    }

    if (parents[x] == 0) {
        return(c(0)) #return initial clone (normal cell)
    } else {
        ##return clone's parent and compute again
        return(c(.getBranch(parents, parents[x]), parents[x]))
    }
}



#' Given the a vector representing the parents of each clone, return a list of
#' vectors specifying which clones are in each clone's branch
#'
#' @param parents An integer vector specifying parental relationships between
#' clones
#'
#' @return List of vectors specifying which clones are in each clone's branch
#' @seealso .getNestLevel
#'
.getAllBranches <- function(parents) {
    branch <- list()
    for (i in seq_len(length(parents))) {
        branch[[i]] <- c(.getBranch(parents, i), i)
    }
    return(branch)
}



#' Get an integer vector specifying the order which to get each clone's x and
#' y positions
#' for the bottom visualization, so that the "longest" family is on the bottom.
#'
#' @param nestLevels An integer vector specifying how deeply a given clone is
#' nested in the overall hierarchy
#' @param parents An integer vector specifying parental relationships between
#' clones
#' @param cloneFam An integer vector specifying which family each clone belongs
#' to
#'
#' @return Vector with clone's order of layout
.getClonesOrder <- function(nestLevels, parents, cloneFam) {
    longestBranches <-
        which(nestLevels == max(nestLevels)) #deepest clones
    f <- table(cloneFam)[cloneFam[longestBranches]]
    ##choose the family with less dependent branched clones (more linear)
    if (length(unique(f)) == 1) {
        longestFamily <- as.numeric(names(f))[1]
        #deepest clone in the selected fam
        lastClone <-
            longestBranches[which(cloneFam[longestBranches] == longestFamily)][1]
    } else{
        longestFamily <- as.numeric(names(f[f == min(f)]))
        #deepest clone in the selected fam
        lastClone <-
            longestBranches[which(cloneFam[longestBranches] == longestFamily)]
    }


    subBranches <- .getAllBranches(parents)

    clonesOrder <- subBranches[[lastClone]]
    parentClones <- which(parents == 0)
    if (length(parentClones) > 1) {
        clonesOrder <-
            clonesOrder %>% insert(parentClones[-which(parentClones %in% clonesOrder)], ats = 3)
    }

    for (i in seq_len(length(parents))) {
        p <- clonesOrder[i]
        if (p != 0) {
            children <- which(parents == p)
            clonesOrder <- c(clonesOrder, children)
            if (length(children) == 0) {
                clonesOrder <- c(clonesOrder, parentClones)
            }
        }
        clonesOrder <- clonesOrder[!duplicated(clonesOrder)]

    }
    return(list(clonesOrder = clonesOrder, deepestClone = lastClone))
}






#' Given a sea object generate key points of the layout that will be used for
#' plotting with top visualization
#'
#' @param seaObject A sea object with appropriate slots filled
#' (fracTable, parents, nestLevels)
#' @param separateIndependentClones Boolean - Should independently-arising
#' clones (with parent 0) be separated by blank space in the plot?
#'
#' @return A sea object with layout slots filled in
#'
#' @examples
#' \dontrun{
#' .topLayout(seaObject)
#'
#' .topLayout(seaObject, separateIndependentClones = TRUE)
#' }
.topLayout <-
    function(seaObject, separateIndependentClones = FALSE) {
        cloneFam <- seaObject@cloneFamily ##vector with each clones family
        cloneBranches <- .getAllBranches(seaObject@parents)
        clonesOrder <- .getClonesOrder(
            nestLevels = seaObject@nestLevels,
            parents = seaObject@parents,
            cloneFam = cloneFam
        )$clonesOrder
        deepestClone <- .getClonesOrder(
            nestLevels = seaObject@nestLevels,
            parents = seaObject@parents,
            cloneFam = cloneFam
        )$deepestClone

        innerSpace <- lapply(rownames(seaObject@fracTable), .getInnerSpace,
                                    seaObject)
        outerSpace <- .getOuterSpace(seaObject)

        ytop.vec <- c()
        ybtm.vec <- c()
        xpos.vec <- c()

        ##for each timepoint
        for (timepos in seq_len(length(seaObject@timepoints))) {
            timepoint <- seaObject@timepoints[timepos]

            ytop <- rep(NA, length(seaObject@parents))
            ybtm <- rep(NA, length(seaObject@parents))
            xpos <- rep(timepoint, length(seaObject@parents))

            for (parent in clonesOrder) {
                if (parent == 0) {
                    parentClones <- which(seaObject@parents == 0)
                    children <-
                        c(clonesOrder[2],
                        parentClones[-which(parentClones == clonesOrder[2])])
                } else{
                    children <-
                        clonesOrder[sort(match(which(seaObject@parents == parent),
                                        clonesOrder))]
                }
                spacing <- 0
                ##start at the bottom
                y <- 0

                if (parent != 0) {
                    ##consider inner spacing if this is a subclone
                    y <- ybtm[parent]
                    spacing <- innerSpace[[parent]][timepos] /
                                            (.getNumChildren(seaObject, parent,
                                                            timepos) + 1)
                }

                ##for each clone that has this parent, get coords
                for (clone in children) {
                    ##clone absent, don't need to add positions
                    if (seaObject@fracTable[clone, timepos] == 0) {
                        xpos[clone] <- NA
                        ##smooth ending to dying clones
                        if (timepos > 1) {
                            if (seaObject@fracTable[clone, timepos - 1] > 0) {
                                if (clone %in% cloneBranches[[deepestClone]]) {
                                    ybtm[clone] <- y
                                    ytop[clone] <- y - spacing / 2
                                    xpos[clone] <- timepoint - 0.25
                                } else{
                                    ybtm[clone] <- y - spacing / 2
                                    ytop[clone] <- y - spacing / 2
                                    xpos[clone] <- timepoint - 0.25
                                }
                            }
                        }
                    } else {
                        #clone is still here, deal with it
                        #Bottom family --> bottom visualization
                        if (separateIndependentClones & parent == 0) {
                            spacing <- outerSpace[timepos]
                            ybtm[clone] <- y
                            y <- y - seaObject@fracTable[clone, timepos]
                            ytop[clone] <- y
                            y <- y - spacing * 0.05
                        } else{
                            if (clone %in% cloneBranches[[deepestClone]]) {
                                ybtm[clone] <- y
                                y <- y - seaObject@fracTable[clone, timepos]
                                ytop[clone] <- y
                                y <- y - spacing
                            } else{
                                ##other families --> normal viz
                                ybtm[clone] <- y - spacing
                                y <- y - seaObject@fracTable[clone, timepos]
                                ytop[clone] <- y - spacing
                                y <- y - spacing
                            }
                        }


                    }
                }
            }
            ybtm.vec <- c(ybtm.vec, ybtm)
            ytop.vec <- c(ytop.vec, ytop)
            xpos.vec <- c(xpos.vec, xpos)
        }


        ##turn coords into a matrix so that we go by clone instead of by
        ##timepoint
        ybtm <- matrix(ybtm.vec, ncol = ncol(seaObject@fracTable))
        ytop <- matrix(ytop.vec, ncol = ncol(seaObject@fracTable))
        xpos <- matrix(xpos.vec, ncol = ncol(seaObject@fracTable))


        ybtm.list <- list()
        ytop.list <- list()
        xpos.list <- list()

        ##now, split into lists per clone
        for (i in clonesOrder[2:length(clonesOrder)]) {
            ybtm.list[[i]] <- ybtm[i, !is.na(ybtm[i, ])]
            ytop.list[[i]] <- ytop[i, !is.na(ytop[i, ])]
            xpos.list[[i]] <- xpos[i, !is.na(xpos[i, ])]
        }

        seaObject@ybtm <- ybtm.list
        seaObject@ytop <- ytop.list
        seaObject@xpos <- xpos.list

        return(seaObject)
    }






##Get colors for the allele visualization (bottom half)
#' Given a sea object and an integer vector with which clone's color to fill
#' each clone return a vector with each clones colors for the bottom half of
#' plaice plot
#'
#' @param seaObject A sea object with appropriate slots filled
#' (fracTable, parents, nestLevels)
#' @param clonesToFill An integer vector with the index of the clone's color to
#' fill each clone. For example:
#' clonesToFill <- c(0,0,0,2,0,0) clone 4 (and its children) will be filled
#' with clone's 2 color
#'
#' @return A vector with the clones colors
#'
.getBottomCloneCols <- function(seaObject, clonesToFill) {
    btmCloneColors <- rep('white', nrow(seaObject@fracTable))
    branches <- .getAllBranches(seaObject@parents)
    for (clone in seq_len(length(clonesToFill))) {
        if (clonesToFill[clone] != 0) {
            children <- c()
            for (branch in branches) {
                if (clone %in% branch) {
                    children <- c(children, branch[2:length(branch)])
                    children <-
                        children[which(seaObject@nestLevels[children] > seaObject@nestLevels[clone])]
                }
            }
            children <-
                unique(children) #paint clone desired and "children"
            btmCloneColors[clone] <- seaObject@col[clonesToFill[clone]]
            for (child in children) {
                if (clonesToFill[child] == 0) {
                    btmCloneColors[child] <- seaObject@col[clonesToFill[clone]]
                }
            }
        }
    }
    return(btmCloneColors)
}

.isColor <- function(x) {
    tryCatch(col2rgb(x), 
            error = function(e) NULL)
}

.onLoad <- function(...) {
    shiny::addResourcePath("clevRvis", system.file("www", package="clevRvis"))
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----installation, eval=FALSE-------------------------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("clevRvis")

## ----loading------------------------------------------------------------------
library(clevRvis)

## ----inputs-------------------------------------------------------------------
##Example data
timepoints <- c(0,50,100)
parents <- c(0,1,1,3,0,5,6)
fracTable <- matrix(c(20,10,0,0,0,0,0,
                    40,20,15,0,30,10,0,
                    50,25,15,10,40,20,15),
                    ncol = length(timepoints))
seaObject <- createSeaObject(fracTable, parents, timepoints,
                            timepointInterpolation = FALSE)

## ----seaObject----------------------------------------------------------------
##seaObject with enabled time point estimation
seaObject_tp <- createSeaObject(fracTable, parents, timepoints,
                                timepointInterpolation = TRUE)

## ----thpEff-------------------------------------------------------------------
##seaObject with enabled time point estimation and therapy effect 
##estimation between time point 50 and 100
seaObject_te <- createSeaObject(fracTable, parents, timepoints,
                            timepointInterpolation = TRUE,
                            therapyEffect = c(50,100))

## ----individualColors---------------------------------------------------------
##seaObject with manually defined colors
seaObject_col <- createSeaObject(fracTable, parents, timepoints,
                                timepointInterpolation = TRUE,
                                col = rainbow(7))

## ----shark--------------------------------------------------------------------
#Basic shark plot showing legend and title
sharkPlot(seaObject_tp, showLegend = TRUE, main = 'Example Shark plot')

## ----sharkIndivColors---------------------------------------------------------
#Basic shark plot showing legend and title with manually defined color coding
sharkPlot(seaObject_col, showLegend = TRUE, main = 'Example Shark plot')

## ----extShark-----------------------------------------------------------------
##extended shark plot, showing CCF as point size only for measured 
###time points, legend and title
extSharkPlot(seaObject_tp, timepoints = timepoints, showLegend = TRUE, 
            main = 'Example Extended Shark plot')

## ----splineDolph--------------------------------------------------------------
##Dolphin plot, with vertical lines showing all time points, custom y axis 
##label and triangles indicating the measured time points
dolphinPlot(seaObject_tp, showLegend = TRUE, 
            vlines = timepoints(seaObject), 
            vlab = timepoints(seaObject), vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)',
            markMeasuredTimepoints = timepoints)

## ----polyDolph----------------------------------------------------------------
##Dolphin plot polygon shape
dolphinPlot(seaObject_tp, showLegend = TRUE, vlines = timepoints,
            vlab = timepoints, vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)', shape = 'polygon')


## ----bottomDolph--------------------------------------------------------------
##Dolphin plot bottom layout and separated independent clones
dolphinPlot(seaObject_tp, showLegend = FALSE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)', 
            separateIndependentClones = TRUE, pos = 'bottom')

## ----annotsDolph--------------------------------------------------------------
##Dolphin plot with annotations
annotsTable <- data.frame(x = c(50,75), y = c(15,50), 
                            col = c('black', 'white'), 
                            lab = c('Annotation clone 5', 'Annotation clone 2'))
dolphinPlot(seaObject_tp, showLegend = TRUE, main = 'Example Dolphin Plot', 
            vlines = timepoints, vlab = timepoints, vlabSize = 2, 
            ylab = 'Cancer cell fraction', annotations = annotsTable, 
            pos = 'bottom', separateIndependentClones = TRUE)

## ----thpEffDolph--------------------------------------------------------------
##Dolphin plot with enabled therapy effect estimation
##vertical lines show all time points, customized y axis label,
##triangles indicate the measured time points
dolphinPlot(seaObject_te, showLegend = TRUE, 
            vlines = slot(seaObject,"timepoints"), 
            vlab = slot(seaObject,"timepoints"), vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)',
            markMeasuredTimepoints = timepoints)

## ----SharkDolph---------------------------------------------------------------
##Basic shark plot linked to dolphin plot
combinedPlot(seaObject_tp, showLegend = TRUE, vlines = timepoints,
            vlab = timepoints, vlabSize = 2, ylab = 'Cancer cell fraction',
            separateIndependentClones = TRUE)

## ----plaice-------------------------------------------------------------------
#Plaice plot when all genes have at least one healthy copy
plaicePlot(seaObject_tp, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE, 
            separateIndependentClones = TRUE)

## ----plaiceAnnots-------------------------------------------------------------
#Plaice plot showing biallelic events + annotations
annotsTable <- data.frame(x = c(24,55), y = c(-40,-5), 
                            col = c('black', 'white'), 
                            lab = c('TP53', 'UBA1'))
plaicePlot(seaObject_tp, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE, 
            separateIndependentClones = TRUE, clonesToFill = c(0,0,1,0,0,6,0),
            annotations = annotsTable)

## ----plaiceThpEff-------------------------------------------------------------
##Plaice plot with enabled therapy effect estimation, 
##all genes are assumed to have at least one healthy copy
plaicePlot(seaObject_te, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE, 
            separateIndependentClones = TRUE)

## ----exploreTrees-------------------------------------------------------------
##Alternative, valid parental relations are determined
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                20, 00, 00, 00,
                30, 00, 20, 05,
                98, 00, 55, 40),
            ncol=length(timepoints))

trees <- exploreTrees(fracTable, timepoints)
trees

## ----example1,echo=FALSE------------------------------------------------------
library(knitr)
kable(data.frame(Clone=c("clone A","clone B","clone C","clone D"),
                Parent=c("normal cell","clone A","clone B","clone C")))

## ----example2,echo=FALSE------------------------------------------------------
library(knitr)
kable(data.frame(Clone=c("clone A","clone B","clone C","clone D","clone E"),
                Parent=c("normal cell","clone A","clone B","clone C",
                        "clone B")))

## ----example3-----------------------------------------------------------------
##Data used in a previous example
timepoints <- c(0,50,100)
fracTable <- matrix(c(20,10,0,0,0,0,0,
                    40,20,15,0,30,10,0,
                    50,25,15,10,40,20,15),
                    ncol = length(timepoints))

##Define input for shiny-app
input<-data.frame(Clones=paste0("Clone ",LETTERS[1:nrow(fracTable)]),
                    as.data.frame(fracTable))
names(input)[2:ncol(input)]<-timepoints

##Export data.frame 'input', e.g. as csv-file
##write.table(input, "Example3.csv", sep = ",", row.names = FALSE,quote = FALSE)

## ----example3Parents,echo=FALSE-----------------------------------------------
library(knitr)
kable(data.frame(Clone=c("clone A","clone B","clone C","clone D","clone E",
                        "Clone F","Clone G"),
                Parent=c("normal cell","clone A","clone A","clone C",
                        "Normal cell","Clone E","Clone F")))

## ----example4-----------------------------------------------------------------
##Data used in a previous example
timepoints <- c(0,50,100)
parents <- c(0,1,1,3,0,5,6)
fracTable <- matrix(c(20,10,0,0,0,0,0,
                    40,20,15,0,30,10,0,
                    50,25,15,10,40,20,15),
                    ncol = length(timepoints))

##Define input for shiny-app
input<-data.frame(Clones=paste0("Clone ",LETTERS[1:nrow(fracTable)]),
                    as.data.frame(fracTable),parents=parents)
names(input)[2:(ncol(input)-1)]<-timepoints

##Export data.frame 'input', e.g. as csv-file
##write.table(input, "Example3_withParents.csv", sep = ",", row.names = FALSE,
##quote = FALSE)

## ----session------------------------------------------------------------------
sessionInfo()

sharkPlot <- function(seaObject, showLegend = FALSE, main = NULL){
    
    if(is(seaObject, "seaObject") == FALSE){
        stop('No seaObject provided as input.')
    }
    if(showLegend != TRUE && showLegend != FALSE){
        stop("No valid input for showLegend provided.")
    }
    if(!is.null(main) && !is.character(main)){
        stop("No valid input for main provided.")
    }
    if(nrow(fracTable(seaObject)) != length(parents(seaObject))){
        stop("Number of clones defined in the fractable does not match number
            of clones defined in parents (", nrow(fracTable(seaObject)), " vs ",
            length(parents(seaObject)), ").")
    }
    if(ncol(fracTable(seaObject)) != length(timepoints(seaObject))){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", 
            ncol(fracTable(seaObject)), " vs ", length(timepoints(seaObject)), 
            ").")
    }
    
    name<-NULL
    lab<-NULL

    cloneLabels <- seaObject@cloneLabels
    ##If it's just one clone
    if (nrow(seaObject@fracTable)==1){
        df <- data.frame(name = cloneLabels, x = 0, y = 0)
        intPlot <- ggplot(df) + geom_point_interactive(size = 10,
                                                    mapping = aes(x = x,
                                                            y = y,
                                                            color = name,
                                                            tooltip = name,
                                                            data_id = name),
                                                    show.legend = showLegend)+
            scale_colour_manual(values = seaObject@col,
                                name = "Clone labels") + theme_void()

        if(!is.null(main)){
            mainDf <- data.frame(lab = main, x = 0, y = 1)
            intPlot <- intPlot + geom_text(mapping = aes(x = x, y = y,
                                                        label = lab),
                                            size = 5, data = mainDf) +
                expand_limits(y=-1)
        }
    }
    ##If all clones are independent
    else if(sum(seaObject@parents)==0){
        intPlot <- .indepClonesPlot(cloneLabels = cloneLabels,
                                    showLegend = showLegend,
                                    cloneCols = seaObject@col)
    }
    else{
        intPlot <- .basicSharkPlot(seaObject)

        ##Number of expected trees
        numTrees <- sum(seaObject@parents == 0)
        ##list with parent vectors divided by families
        l_from <- .getFromTo(seaObject)[seq_len(numTrees)]
        ##list with child vectors divided by families
        l_to <- tail(.getFromTo(seaObject),numTrees)
        ##get initial (parent) clones
        initialClones <- which(seaObject@parents==0)
        ##Table with the known parents (not independent)
        parentTbl <- intPlot$data %>%
            select(name, x, y) %>%
            filter(name %in% initialClones)

        ##Labels of independent clones
    indepCloneLabels <- .getIndependentCloneLabels(numTrees, l_to)$cloneLabs
        ##number of independent clones
        count <- .getIndependentCloneLabels(numTrees, l_to)$count

        ##x and y positions of the independent clones
        indepCloneXpos <- .getIndependentCloneXY(count, parentTbl, intPlot)$x
        indepCloneYpos <- .getIndependentCloneXY(count, parentTbl, intPlot)$y

        ##create new dataframe for the points
        plotData <- data.frame(x=intPlot$data$x, y=intPlot$data$y,
                                id = intPlot$data$name)

        if (count != 0){
            ##dataframe with the independent clones data
            indepClonesData <- data.frame(x = indepCloneXpos,
                                            y = indepCloneYpos,
                                            id = indepCloneLabels)

            ##bind the rows to the points dataframe
            plotData <- rbind(plotData, indepClonesData)
        }


        ##get the order of the sorted clones
        if (seaObject@defaultLabels){
            clone_ids <- parse_number(sort(plotData$id))
        }else{
            clone_ids <- sort(cloneLabels, index.return = TRUE)$ix
        }

        ##Add column with clone labels
        plotData <- plotData %>%
            add_column(name=cloneLabels[as.numeric(plotData$id)])


        ##Add independent clones to the plot
        intPlot <- intPlot +
            geom_point_interactive(size = 10, mapping = aes(x = x,
                                                            y = y,
                                                            color = name,
                                                            tooltip = name,
                                                            data_id = name),
                                    show.legend = showLegend,
                                    data=plotData)+
            scale_colour_manual(values = seaObject@col[clone_ids],
                                                    name = "Clone labels") +
            expand_limits(x=c(min(plotData$x)-0.25, max(plotData$x)+0.25),
                        y=c(min(plotData$y)-0.25, max(plotData$y)+0.25))

    }
    if(!is.null(main) & length(cloneLabels)>1){
        if(sum(seaObject@parents)==0){
            x <- mean(plotData$x, na.rm = TRUE)
            y <- max(plotData$y+1, na.rm = TRUE)
        }else{
            y <- mean(plotData$y, na.rm = TRUE)
            x <- min(plotData$x-1, na.rm = TRUE)
        }

        mainLabel <- data.frame(lab = main,x,y)

        intPlot <- intPlot + geom_text(mapping = aes(x = x, y = y,
                                                    label = lab),
                                        size = 5,
                                        data = mainLabel)
    }

    return(intPlot)
}

combinedPlot <- function(seaObject, shark = TRUE, dolphin = TRUE,
                        shape = 'spline', borderCol = NULL, vlines = NULL,
                        vlineCol="#6E6E66", vlab=NULL, vlabSize = 3,
                        pos = 'center',
                        separateIndependentClones=FALSE, showLegend = FALSE,
                        markMeasuredTimepoints = NULL, downloadWidget = NULL,
                        mainDph = NULL, mainPosDph = 'middle', mainSizeDph = 5,
                        mainShk = NULL,
                        xlab = NULL, ylab = NULL, pad.left = 0.005,
                        annotations = NULL, width = 12, height = 9){

    if(is(seaObject, "seaObject") == FALSE){
        stop('No seaObject provided as input.')
    }
    if(shark != TRUE && shark != FALSE){
        stop("No valid input for shark provided.")
    }
    if(dolphin != TRUE && dolphin != FALSE){
        stop("No valid input for dolphin provided.")
    }
    if(shape != "spline" && shape != "polygon"){
        stop("No valid input for shape provided.")
    }
    if(!is.null(borderCol) && is.null(.isColor(borderCol))){
        stop("No valid input for borderCol provided.")
    }
    if(!is.null(vlines) && !is.numeric(vlines)){
        stop("No valid input for vlines provided.")
    }
    if(is.null(.isColor(vlineCol))){
        stop("No valid input for vlineCol provided.")
    }
    if(!is.null(vlab) && !is.character(vlab) && !is.numeric(vlab)){
        stop("No valid input for vlab provided.")
    }
    if(!is.numeric(vlabSize)){
        stop("No valid input for vlabSize provided.")
    }
    if(pos != "center" && pos != "bottom" && pos != "top"){
        stop("No valid input for pos provided.")
    }
    if(separateIndependentClones != TRUE && separateIndependentClones != FALSE){
        stop("No valid input for separateIndependentClones provided.")
    }
    if(showLegend != TRUE && showLegend != FALSE){
        stop("No valid input for showLegend provided.")
    }
    if(!is.null(markMeasuredTimepoints) && !is.numeric(markMeasuredTimepoints)){
        stop("No valid input for markMeasuredTimepoints provided.")
    }
    if(!is.null(downloadWidget) && !is.character(downloadWidget)){
        stop("No valid input for downloadWidget provided.")
    }
    if(!is.null(mainDph) && !is.character(mainDph)){
        stop("No valid input for mainDph provided.")
    }
    if(mainPosDph != "left" && mainPosDph != "middle" && mainPosDph != "right"){
        stop("No valid input for mainPosDph provided.")
    }
    if(!is.numeric(mainSizeDph)){
        stop("No valid input for mainSizeDph provided.")
    }
    if(!is.null(mainShk) && !is.character(mainShk)){
        stop("No valid input for mainShk provided.")
    }
    if(!is.null(xlab) && !is.character(xlab)){
        stop("No valid input for xlab provided.")
    }
    if(!is.null(ylab) && !is.character(ylab)){
        stop("No valid input for ylab provided.")
    }
    if(!is.null(pad.left) && !is.numeric(pad.left)){
        stop("No valid input for pad.left provided.")
    }
    if(!is.null(annotations) && !is.data.frame(annotations)){
        stop("No valid input for annotations provided.")
    }
    if(!is.numeric(width) || width <= 0){
        stop("No valid input for width provided.")
    }
    if(!is.numeric(height) || height <= 0){
        stop("No valid input for height provided.")
    }
    if(nrow(fracTable(seaObject)) != length(parents(seaObject))){
        stop("Number of clones defined in the fractable does not match number
            of clones defined in parents (", nrow(fracTable(seaObject)), " vs ",
            length(parents(seaObject)), ").")
    }
    if(ncol(fracTable(seaObject)) != length(timepoints(seaObject))){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", 
            ncol(fracTable(seaObject)), " vs ", length(timepoints(seaObject)), 
            ").")
    }

    tooltip_css <- "color:black;padding:10px;border-radius:10px 20px 10px 20px;"

    if(shark & dolphin){
        splot <- sharkPlot(seaObject, showLegend = showLegend, main = mainShk)
        fplot <- dolphinPlot(seaObject, vlines = vlines, vlineCol = vlineCol,
                            vlab = vlab, shape = shape, borderCol = borderCol,
                            markMeasuredTimepoints = markMeasuredTimepoints,
                            main = mainDph, mainPos=mainPosDph,
                            vlabSize = vlabSize,
                            mainSize = mainSizeDph, xlab = xlab,
                            ylab = ylab, pad.left = pad.left, pos=pos,
                        separateIndependentClones = separateIndependentClones,
                            annotations = annotations)

        ##interactive plot to html widget
        gPlot <- girafe(ggobj = fplot+splot, width_svg = width,
                        height_svg = height) %>%
            girafe_options(opts_sizing(rescale = FALSE),
                            opts_hover_inv(css = "opacity:0.3;"),
                            opts_hover(css = "opacity:1;"),
                            opts_tooltip(css = tooltip_css, use_fill=TRUE))
    }

    else{
        if(shark & !dolphin){
            p <- sharkPlot(seaObject, showLegend = showLegend, main = mainShk)
        }
        if(dolphin & !shark){
            p <- dolphinPlot(seaObject, showLegend = showLegend,
                            vlabSize = vlabSize,
                            vlines = vlines, vlineCol = vlineCol,
                            vlab = vlab, shape = shape, borderCol = borderCol,
                            markMeasuredTimepoints = markMeasuredTimepoints,
                            main = mainDph, mainPos=mainPosDph, pos = pos,
                            mainSize = mainSizeDph, xlab = xlab,
                            ylab = ylab, pad.left = pad.left,
                        separateIndependentClones = separateIndependentClones,
                            annotations = annotations)
        }

        gPlot <- girafe(ggobj = p, width_svg = width, height_svg = height) %>%
            girafe_options(opts_sizing(rescale = FALSE),
                            opts_hover_inv(css = "opacity:0.3;"),
                            opts_hover(css = "opacity:1;"),
                            opts_tooltip(css = tooltip_css, use_fill=TRUE))
    }

    if(!is.null(downloadWidget)){
        htmlwidgets::saveWidget(gPlot, downloadWidget,
                                                        selfcontained = TRUE)
    }


    return(gPlot)
}

extSharkPlot <- function(seaObject, showLegend = FALSE, main = NULL,
                        timepoints = NULL, width = 10, interactivePlot = TRUE){
    
    if(is(seaObject, "seaObject") == FALSE){
        stop('No seaObject provided as input.')
    }
    if(showLegend != TRUE && showLegend != FALSE){
        stop("No valid input for showLegend provided.")
    }
    if(!is.null(main) && !is.character(main)){
        stop("No valid input for main provided.")
    }
    if(!is.null(timepoints) && !is.numeric(timepoints)){
        stop("No valid input for timepoints provided.")
    }
    if(!is.numeric(width) || width <= 0){
        stop("No valid input for width provided.")
    }
    if(interactivePlot != TRUE && interactivePlot != FALSE){
        stop("No valid input for interactivePlot provided.")
    }
    if(nrow(fracTable(seaObject)) != length(parents(seaObject))){
        stop("Number of clones defined in the fractable does not match number
            of clones defined in parents (", nrow(fracTable(seaObject)), " vs ",
            length(parents(seaObject)), ").")
    }
    if(ncol(fracTable(seaObject)) != length(timepoints(seaObject))){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", 
            ncol(fracTable(seaObject)), " vs ", length(timepoints(seaObject)), 
            ").")
    }
    
    if(is.null(timepoints)){
        timepoints <- seaObject@timepoints ## by default all timepoints
    }
    ##Which ccfs to show
    fracTable <- seaObject@fracTable[,which(seaObject@timepoints %in% timepoints)]

    if (!is.array(fracTable)){
        if(length(timepoints)==1){
            y <- rev(seq_len(length(fracTable)))
            x <- rep(0, length(fracTable))
        } else{
            y <-    rep(0, length(fracTable))
            x <- seq_len(length(fracTable))
        }
        lab <- as.factor(seaObject@cloneLabels)
    } else {
        y <- rep(rev(seq_len(nrow(fracTable))),ncol(fracTable))
        lab <- as.factor(seaObject@cloneLabels[seq_len(nrow(fracTable))])
        x <- rep(seq_len(ncol(fracTable)),each = nrow(fracTable))
    }
    ccf <- as.numeric(fracTable)
    ##Data frame for plotting ccf as size
    df <- data.frame(lab, x, y, ccf)

    if (length(which(ccf == 0))>0){
        df <- df[-which(ccf == 0),] ##don't plot ccf = 0
    }

    ##to plot size from 0 to 100
    df <- df %>% add_row(x=NA, y=NA, lab=NA, ccf=0)
    df <- df %>% add_row(x=NA, y=NA, lab=NA, ccf=100)

    #get sorted
    if (seaObject@defaultLabels){
        clone_ids <- parse_number(sort(unique(as.character(df$lab))))
    }else{
        clone_ids <- sort(seaObject@cloneLabels, index.return = TRUE)$ix
    }

    if(!is.array(fracTable)){
        if(length(timepoints)==1){
            dfTP <- data.frame(x=0, y = max(df$y, na.rm = TRUE)+1,
                                lab = timepoints)
        } else{
            dfTP <- data.frame(x=seq_len(length(fracTable)),
                                y = rep(max(df$y, na.rm = TRUE)+1,
                        length(fracTable)),
                                lab = timepoints)
        }
    } else{
        ##Column labels df
        dfTP <- data.frame(x=seq_len(ncol(fracTable)),
                            y = rep(max(df$y, na.rm = TRUE)+1,
                            ncol(fracTable)),
                            lab = timepoints)
    }
    ##Interactive plot
    dotplot <- ggplot() +
                geom_point_interactive(mapping = aes(x = x, y = y, color = lab,
                                                    size = ccf, tooltip = ccf,
                                                    data_id = lab),
                                        data = df[!is.na(df$lab),],
                                        show.legend = showLegend) +
        scale_colour_manual(values = seaObject@col[clone_ids],
                                                name = "Clone labels",
                                                na.translate = FALSE) +
        scale_size_area(name='Relative CCFs', max_size = 15) + theme_void() +
        expand_limits(x=c(min(df$x, na.rm = TRUE)-0.5,
                        max(df$x, na.rm = TRUE)+0.5),
                        y=c(min(df$y, na.rm = TRUE)-0.5,
                            max(df$y, na.rm = TRUE)+0.5)) +
        guides(colour = guide_legend(override.aes = list(size=5))) +
        geom_text(mapping = aes(x = x, y = y, label = lab),
                            data = dfTP)

    if(!is.array(fracTable)){
        dotplot <- dotplot + expand_limits(y=-1)
    }
    ##Add basic shark plot to the left side
    shkPlt <- sharkPlot(seaObject, main=main)

    ##Interactive
    if (interactivePlot){
    tooltip_css <- "color:black;padding:10px;border-radius:10px 20px 10px 20px;"

        girafe(ggobj = shkPlt+dotplot, width_svg = width,
                        height_svg = 10) %>%
                girafe_options(opts_sizing(rescale = FALSE),
                                opts_hover_inv(css = "opacity:0.3;"),
                                opts_hover(css = "opacity:1;"),
                                opts_tooltip(css = tooltip_css,
                                use_fill=TRUE))
    } else {
        plot_grid(shkPlt,dotplot)
    }

}

clevRvisShiny <- function() {
    shinyApp(
        ui=shinyUI(
            # Define UI for application
            shinyUI(dashboardPage(
                title = 'clevR-vis',
                ###HEADER
                dashboardHeader(
                    title = span(div(
                        column(2, img(src = "clevRvis/logo_version3_black.png", 
                                        width = '50px')),
                        column(8,"clevRvis"),
                        column(2,img(src = "clevRvis/logoIMI.png", width = 
                                        "50px"))
                    )),
                    titleWidth = '100%',
                    
                    #instructions ######
                    dropdownMenu(
                        type = "notifications",
                        headerText = strong("INSTRUCTIONS"),
                        icon = icon("question"),
                        badgeStatus = NULL,
                        notificationItem(
                            text = 'Start at the "Inputs" section (left menu) 
                            and upload a file (excel or csv) containing the 
                            clones cancer cell fractions.',
                            icon = icon('download')
                        ),
                        notificationItem(
                            text = 'Specify the time points, clone labels and 
                            parental relations and validate them by clicking 
                            "Submit Inputs".',
                            icon = icon('toggle-on')
                        ),
                        notificationItem(
                            text = 'When inputs are valid, move to the 
                            "seaObject options" section and select the desired 
                            estimations and click the "submit" button.',
                            icon = icon('filter')
                        ),
                        notificationItem(
                            text = 'Check the Cancer Cell Fractions table 
                            created, including the parental relations. If 
                            everything looks right you can now plot the data.',
                            icon = icon('table')
                        ),
                        notificationItem(
                            text = 'On the "Plots" section you can decide which 
                            plots you want to visualize. Each subsection shows a 
                            kind of plot and the corresponding filters.',
                            icon = icon('chart-area')
                        ),
                        notificationItem(
                            text = 'To add annotations to "Dolphin or Plaice 
                            plot" go to "Annotations" section (left menu). There 
                            you find a subsection for each clone where you can 
                            specify the text and position of the annotation.',
                            icon = icon('pencil')
                        )
                    )
                ),
                
                ###SIDEBAR
                dashboardSidebar(
                    width = 400,
                    sidebarMenu(
                        id = 'sidebar',
                        menuItem('Outputs', tabName = 'outputs', 
                                icon = icon('table')),
                        
                        br(),
                        
                        h4(' INPUTS & OPTIONS', style = 'color: #343a40;'),
                        
                        ##Inputs PANEL #####
                        menuItem(
                            'Inputs',
                            icon = icon('download'),
                            
                            br(),
                            
                            ##fractable (CCFs) file as excel or csv
                            fileInput('fileCCF','Cancer Cell Fractions (CCFs) 
                            table (xlsx, xls, csv files supported)',
                                    accept = c('.csv','.xls','.xlsx')),
                            
                            br(),
                            
                            ##timepoints vector
                            prettySwitch(
                                'tpCols', 'First row contains time point labels 
                                (numeric)?',
                                status = 'primary', slim = TRUE, value = TRUE) %>%
                                helper(
                                    type = "inline",
                                    icon = 'circle-question',
                                    style = "position: absolute; left: 40px;",
                                    title = "Defining time points",
                                    content = c("<p><strong>Does the first row 
                                    of the input file correspond to the time 
                                                points?</strong></p>",
                                                "Time points must be numeric 
                                                values!",
                                                "To modify the time points from 
                                                the app the input
                            file must not contain them.")
                                ),
                            
                            conditionalPanel(
                                condition = '!input.tpCols',
                                prettySwitch('dfltTP', 'Use default time points? 
                                            (1,2...)',
                                            status = 'primary', slim = TRUE),
                                conditionalPanel(
                                    condition = '!input.dfltTP',
                                    textInput('timepoints','Vector with time points') %>%
                                        helper(
                                            type = "inline",
                                            style = "position: absolute; left: 
                                            170px;",
                                            icon = 'circle-question',
                                            title = "Defining manual time 
                                            points",
                                            content = c("Write the time points 
                                            in the text box separated by commas",
                                                        "Time points must be 
                                                        numeric values",
                                                        "To add the time points 
                                                        from the app the input
                                file must not contain them.")
                                        )
                                )
                            ),
                            
                            br(),
                            
                            #labels
                            prettySwitch('labRows', 'First column contains clone 
                                        labels?',
                                        status = 'primary', slim = TRUE, value = TRUE) %>%
                                helper(
                                    type = "inline",
                                    icon = 'circle-question',
                                    style = "position: absolute; left: 40px;",
                                    title = "Defining Clone Labels",
                                    content = c("<p><strong>Does the first 
                                    column of the input file
                                correspond to the clone labels?</strong></p>",
                                                "To modify the clone labels from 
                                                the app the
                                input file must not contain them.")
                                ),
                            
                            conditionalPanel(
                                condition = '!input.labRows',
                                prettySwitch('cloneLabs', 'Use default clone 
                                            labels',
                                            status = 'primary', slim = TRUE),
                                conditionalPanel(
                                    condition = '!input.cloneLabs',
                                    textInput('cloneLabsValues',
                                            'Manual custom labels for clones')
                                )
                            ),
                            
                            br(),
                            
                            ##parental relations
                            radioGroupButtons(
                                'parentCol', 'How to define parental 
                                relations?',
                                choiceNames = c('Column "parents" included',
                                                'Define parental relations 
                                                interactively'),
                                choiceValues = c('column','interactive')
                            )  %>%
                                helper(
                                    type = "inline",
                                    icon = 'circle-question',
                                    style = "position: absolute; left: 100px;",
                                    title = "How to define parental relations",
                                    content = c("<p><strong>Does the input table 
                                    contain a column
                            of parental relations?</strong></p>",
                                                'The column of parental 
                                                relations must be
                            named "parents". And it can be in any position
                            except the first column if clone labels
                            are also defined in the input table.',
                                                'Parental relations must be 
                                                defined by integer
                            values, indicating the position of the parent clone
                            (first row =  1). To indicate a clone emerging from
                            normal cell, the value must be 0.',
                                                "To interactively define 
                                                parental relations, the
                            input table must not contain them.",
                                                "<p><strong>The input table 
                                                doesn't contain a column
                            of parental relations?</strong></p>",
                                                'A dropdown for each clone will 
                                                appear with the
                            possible parents to interactively define the
                            parental relations.')
                                ),
                            
                            ##TEMPORAL check parents
                            #textOutput('checkColNames'),
                            # textOutput('checkParents'),
                            
                            conditionalPanel(
                                condition = 'input.parentCol == "interactive" & !input.expTrees',
                                h5(tags$b('Interactively define parental 
                                        relations:')),
                                htmlOutput('askTblParents'),
                                uiOutput('parents')
                            ),
                            
                            br(),
                            
                            prettySwitch(
                                'expTrees', 'Explore alternative trees',
                                status = 'primary', slim = TRUE, value = FALSE) %>%
                                helper(
                                    type = "inline",
                                    icon = 'circle-question',
                                    style = "position: absolute; left: 40px;",
                                    title = "Defining time points",
                                    content = c("<p><strong>Exploring 
                                    alternative
                            trees</strong></p>",
                                                "When switched on a dropdown 
                                                list will appear
                            containing the possible parental relations for the
                            table of Cancer Cell Fractions uploaded")
                                ),
                            
                            ##TEMPORAL check
                            # textOutput('checkAltTrees'),
                            
                            conditionalPanel(
                                condition = 'input.expTrees',
                                selectInput('optParents','Alternative parental
                                            relations:',
                                            choices = c('a','b'))
                            ),
                            
                            br(), hr(),
                            
                            actionBttn("subInputs", "Submit inputs",
                                        style = "jelly", color = "success", 
                                        size = 'sm'),
                            
                            useSweetAlert(),
                            
                            br()
                            
                        ), #end of inputs panel
                        
                        ##filters PANEL #####
                        menuItem(
                            'seaObject options',
                            icon = icon('filter'),
                            
                            #timepoints interpolation
                            prettySwitch('tpEstim', 'Extra time points 
                                        interpolation',
                                        status = 'primary', slim = TRUE, 
                                        value = TRUE),
                            conditionalPanel(
                                condition = 'input.tpEstim',
                                ##Custom original timepoint
                                prettySwitch('customOgTP', 'Custom time point of 
                                            origin',
                                            status = 'primary', slim = TRUE, 
                                            value = FALSE),
                                
                                conditionalPanel(condition = 'input.customOgTP',
                                                numericInput('ogTP', 'Time 
                                                            point of origin:',
                                                            max = 0,value=-10))
                            ),
                            
                            hr(),
                            
                            #therapy effect estimation
                            prettySwitch('therapyEffect', 'Therapy Effect 
                                        estimation',
                                        status = 'primary', slim = TRUE),
                            
                            conditionalPanel(
                                condition = 'input.therapyEffect',
                                radioGroupButtons("thpEf_opt", label = "",
                                                choiceNames = c("Specific time 
                                                                point",
                                                                "Two measured 
                                                                time points"),
                                                choiceValues = c(1,2),
                                                justified = TRUE
                                ),
                                
                                conditionalPanel(
                                    condition = 'input.thpEf_opt == 1',
                                    sliderInput('tpThp',
                                                'Select time point to estimate 
                                                therapy effect:',
                                                min = 0, max = 0, value = 0)),
                                
                                conditionalPanel(
                                    condition = 'input.thpEf_opt == 2',
                                    sliderTextInput('MtpThp','Select measured 
                                                    time points:',
                                                    choices = c(1,2), 
                                                    selected = c(1,2),
                                                    grid = TRUE))
                            ),
                            
                            hr(),
                            
                            #ccf or proportions switch
                            radioGroupButtons('ccfVSprop', 'Which proportions 
                                            to show?',
                                            choiceNames = c('CCF_clone',
                                                            'CCF_clone - sum(CCF_children)'),
                                            choiceValues = c('ccf', 'prop')) %>%
                                helper(
                                    type = "inline",
                                    icon = 'circle-question',
                                    style = "position: absolute; left: 200px;",
                                    title = "Proportions to visualize in 
                                    seaObject table",
                                    content = c("<p><strong>CCF_clone</strong></p>",
                                                "Each value represents the 
                                                absolute proportion of a
                            certain clone at a certain time point. The sum of
                            all CCF values at one timepoint can be >100% as
                            each CCF value includes the proportion of the
                            corresponding clone and its children",
                                                "",
                                                "<p><strong>CCF_clone - sum(CCF_children)</strong>
                            </p>",
                                                "Each value represents the 
                                                proportion of only the
                            corresponding clone. The sum of all proportion
                            values at one timepoint can NOT be >100% as the
                            proportions are calculated like:",
                                                "CCF_clone - sum(CCF_children)",
                                                "",
                                                "<p><strong>Example:</strong></p>",
                                                "CloneA -> CloneB -> CloneC",
                                                "At time point 1:",
                                                "CCF_clone: CloneA = 70%,  
                                                CloneB = 50%,
                            CloneC = 10%",
                                                "CCF_clone - sum(CCF_children): 
                                                CloneA = 20%,
                            CloneB = 40%, CloneC = 10%")
                                ),
                            
                            hr(),
                            
                            ##button to submit all changes
                            actionBttn("submitSea", "Submit", style = "jelly",
                                        color = "success", size = 'sm'),
                            useSweetAlert()
                        ), #end of seaObject filter panel
                        
                        ##PLOTS PANEL #####
                        menuItem(
                            'Plots',
                            icon = icon('chart-area'),
                            ##shark plot
                            menuItem(
                                'Shark Plot', tabName = 'shkDph',
                                prettySwitch("shark", "Show Shark Plot",
                                            status = 'primary', slim = TRUE),
                                
                                ##FILTERS ONLY VISIBLE WHEN SHARK PLOT IS 
                                ##SELECTED
                                conditionalPanel(
                                    condition = 'input.shark',
                                    h4('Shark plot filters'),
                                    prettySwitch('legendShk', 'Show legend',
                                                status = 'primary',
                                                slim = TRUE),
                                    textInput('mainShk', 'Main'),
                                    prettySwitch('extendedShk', 'Extended Shark 
                                                Plot',
                                                status = 'primary', 
                                                slim = TRUE),
                                    conditionalPanel(
                                        condition = 'input.extendedShk',
                                        checkboxGroupInput('showTps', 
                                                            'Shown time points',
                                                            choices = c('Measured time points',
                                                                        'Estimated time points',
                                                                        'Custom'),
                                                            selected = 'Measured time points'),
                                        #only show if custom Vlines is selected
                                        conditionalPanel(
                                            condition="$.inArray('Custom', input.showTps) > -1",
                                            textInput('customShowTps',
                                                    'Add manual existing time 
                                                    points to show')
                                        )
                                    )
                                )
                            ), #end of shark menuItem
                            
                            ##dolphin plot
                            menuItem(
                                'Dolphin Plot',
                                prettySwitch('dolphin', 'Show Dolphin Plot',
                                            status = 'primary', slim = TRUE),
                                
                                ##FILTERS ONLY VISIBLE WHEN DOLPHIN PLOT IS 
                                ##SELECTED
                                conditionalPanel(
                                    condition="input.dolphin",
                                    h4('Dolphin plot filters'),
                                    prettySwitch('legendDol', 'Show legend',
                                                status = 'primary',
                                                slim = TRUE),
                                    prettySwitch('sepIndCl', 'Separate 
                                                independent clones?',
                                                status = 'primary',
                                                slim = TRUE),
                                    textInput('mainDol', 'Main'),
                                    radioGroupButtons('pos', 'Layout position',
                                                    choices = c('center', 
                                                                'bottom'),
                                                    selected = 'center', 
                                                    justified = TRUE),
                                    radioGroupButtons('shape', 'Dolphin plot 
                                                    shape',
                                                    choices = c('spline',
                                                                'polygon'),
                                                    justified = TRUE),
                                    conditionalPanel(
                                        condition = "input.shape == 'spline'",
                                        sliderInput('pad', 'Pad Left', 
                                                    min = 0.001, max = 0.25,
                                                    step = 0.001, value = 0.005)
                                    ),
                                    
                                    prettySwitch('axisLab', 'Axis labels',
                                                status = 'primary',
                                                slim = TRUE),
                                    conditionalPanel(condition = 'input.axisLab',
                                                    textInput('xlab', 
                                                                'x axis label'),
                                                    textInput('ylab', 
                                                                'y axis label')),
                                    colourInput('borderCol', 'Clones border 
                                                color',
                                                value = '#FFFFFF00', 
                                                allowTransparent = TRUE),
                                    checkboxGroupInput('vlines', 'Vertical lines 
                                                        x position',
                                                        choices = c('Measured 
                                                                time points',
                                                                'Estimated 
                                                                time points',
                                                                'Custom'),
                                                        selected = 'Measured time 
                                                        points'),
                                    #only show if custom Vlines is selected
                                    conditionalPanel(
                                        condition="$.inArray('Custom', input.vlines) > -1",
                                        textInput('customVlines', 'Add manual 
                                                    vertical lines')
                                    ),
                                    ##only show if vlines is not NULL
                                    conditionalPanel(
                                        condition ="input.vlines.length > 0",
                                        colourInput('vlineCol', 'Vertical lines 
                                                    color',
                                                    value = '#6E6E66'),
                                        checkboxGroupInput('vlab', 'Vertical 
                                                            lines labels',
                                                            choices = c('Measured time points',
                                                                    'Estimated time points',
                                                                    'Custom'),
                                                            selected = 'Measured time points')
                                    ),
                                    #only show if custom vlabs is selected
                                    conditionalPanel(
                                        condition ="$.inArray('Custom', input.vlab) > -1",
                                        textInput('customVlabs',
                                                    'Add manual labels for 
                                                    vertical lines')
                                    ),
                                    ##only show if vlabs is not NULL
                                    conditionalPanel(
                                        condition ="input.vlab.length > 0",
                                        sliderInput('vlabSize', 'Size of the 
                                                    vertical lines labels',
                                                    min = 1, max = 10, 
                                                    value = 5)
                                    ),
                                    checkboxGroupInput('symbols', 'Show symbol underneath',
                                                    choices = c('Measured time points',
                                                                'Estimated time points',
                                                                "Custom")),
                                    #only show if custom symbols is selected
                                    conditionalPanel(
                                        condition ="$.inArray('Custom', input.symbols) > -1",
                                        textInput('customSymbols',
                                                    'Add manual x positions 
                                                    showing symbols'))
                                )
                            ), ##end of dolph menu item
                            
                            ##Plaice plot
                            menuItem(
                                'Plaice Plot', tabName = 'plaice',
                                prettySwitch('plaice', 'Show Plaice Plot',
                                            status = 'primary', slim = TRUE),
                                
                                ##FILTERS ONLY VISIBLE WHEN PLAICE PLOT IS 
                                ##SELECTED
                                conditionalPanel(
                                    condition="input.plaice",
                                    h4('Plaice plot filters'),
                                    prettySwitch('legendPlc', 'Show legend',
                                                status = 'primary',
                                                slim = TRUE),
                                    prettySwitch('sepIndClPlc', 
                                                'Separate independent clones?',
                                                status = 'primary',slim = TRUE),
                                    prettySwitch('axisLabPlc', 'Axis labels',
                                                status = 'primary',slim = TRUE),
                                    prettySwitch('fillClones', 'Mark biallelic 
                                                events',
                                                status = 'primary',slim = TRUE) %>%
                                        helper(
                                            type = "inline",
                                            icon = 'circle-question',
                                            style = "position: absolute; left: 177px;",
                                            title = "How to color biallelic 
                                            events?",
                                            content = c("To show biallelic 
                                            events have happened in a
                                clone you can color them.",
                                "<body><div><li> A not colored clone means
                                there is at least one copy of a healthy allele
                                </li> <li> A clone (e.g Clone 2) colored with
                                the color of another clone (e.g Clone 1) means a
                                new variant (in Clone 2) is causing a biallelic
                                event on the same gene that was affected by the
                                clone which color you are using (Clone 1).</li>
                                <ol> <i>For example</i>: consider linear clonal
                                evolution of 2 clones -- A and B.
                                Clone A is characterized by a mutation in
                                <i>TP53</i>, clone B by a deletion 17p.
                                The two variants are overlapping. If they affect
                                different alleles, no healthy allele of
                                <i>TP53</i>  remains in all cells belonging to
                                clone B. You may chose to color clone B in the
                                lower plot to indicate a decrease in healthy
                                alleles of <i>TP53</i> as the CCF of clone B
                                increases. Instead, if both variants affect the
                                same allele, one healthy copy of <i>TP53</i>
                                remains -- independent of the CCF of clones A
                                and B. Thus, no clone should be colored in the
                                lower plot.</li></ol> <li> A clone colored with
                                its own color means it implies on itself a
                                biallelic event,or it affects the only
                                available X- or Y- chromosome in male subjects.
                                </li> </div></body>")
                                        ),
                                    ##temp check
                                    #textOutput('clonesfillvector'),
                                    
                                    conditionalPanel(
                                        condition = 'input.fillClones',
                                        uiOutput('fillVec')
                                    ),
                                    
                                    textInput('mainPlc', 'Main'),
                                    radioGroupButtons('shapePlc', 'Plaice plot 
                                                        shape',
                                                        choices = c('spline',
                                                                    'polygon'),
                                                        justified = TRUE),
                                    colourInput('borderColPlc', 'Clones border 
                                                color',
                                                value = '#000000',  
                                                allowTransparent = TRUE),
                                    checkboxGroupInput('vlinesPlc', 'Vertical 
                                                        lines x position',
                                                        choices = c('Measured time points',
                                                                'Estimated time points',
                                                                'Custom'),
                                                        selected = 'Measured time points'),
                                    #only show if custom Vlines is selected
                                    conditionalPanel(
                                        condition="$.inArray('Custom', input.vlinesPlc) > -1",
                                        textInput('customVlinesPlc', 
                                                    'Add manual vertical lines')
                                    ),
                                    ##only show if vlines is not NULL
                                    conditionalPanel(
                                        condition ="input.vlinesPlc.length > 0",
                                        colourInput('vlineColPlc', 'Vertical 
                                                    lines color',
                                                    value = '#6E6E66'),
                                        checkboxGroupInput('vlabPlc', 'Vertical lines labels',
                                                        choices = c('Measured time points',
                                                                    'Estimated time points',
                                                                    'Custom'),
                                                        selected = 'Measured time points'),
                                        #only show if custom vlabs is selected
                                        conditionalPanel(
                                            condition ="$.inArray('Custom', input.vlabPlc) > -1",
                                            textInput('customVlabsPlc',
                                                        'Add manual labels for 
                                                        vertical lines')
                                        ),
                                        ##only show if vlabs is not NULL
                                        conditionalPanel(
                                            condition ="input.vlabPlc.length > 0",
                                            sliderInput('vlabSizePlc',
                                                        'Size of the vertical 
                                                        lines labels',
                                                        min = 1, max = 10, 
                                                        value = 5)
                                        )
                                    ),
                                    
                                    checkboxGroupInput('symbolsPlc', 'Show 
                                                        symbol underneath:',
                                                        choices = c('Measured time points',
                                                                'Estimated time points',
                                                                "Custom")),
                                    #only show if custom symbols is selected
                                    conditionalPanel(
                                        condition ="$.inArray('Custom', input.symbolsPlc) > -1",
                                        textInput('customSymbols',
                                                    'Add manual x positions where 
                                                    to show symbols')
                                    )
                                )
                            ) #end of plaice plot menu item
                        ), #end of plots panel
                        
                        ###### Annotations ####
                        menuItem('Annotations', icon = icon('pencil'),
                                prettySwitch('addAnnot', 'Add annotations',
                                                status = 'primary', 
                                                slim = TRUE),
                                menuItemOutput('annot')
                        ), ##end of annotations panel
                        
                        br(), br(), br(), br(), br(), br(), br(), br(), br(), 
                        br(),
                        
                        ###Tutorial ####
                        menuItem("Tutorial",
                                icon = icon("circle-question"),
                                tabName = "instructions"),
                        
                        ###About us ####
                        menuItem("About Us",
                                icon = icon("info"),
                                tabName = "information")
                    ) #end of sidebar menu
                ), #end of sidebar
                
                
                # Outputs #####
                dashboardBody(
                    includeCSS(system.file(package="clevRvis", "www", 
                                            "appStyle.css")),
                    tabItems(
                        tabItem(
                            tabName = 'outputs',
                            tabBox(id = "box_general",
                                width = NULL, height = NULL,
                                tabPanel('Input table',
                                            textOutput('warnParents'),
                                            tags$head(tags$style("#warnParents{color: red;
                                font-size: 20px;
                                font-style: italic;
                                }")
                                            ),
                                            h3('Input table'),
                                            textOutput('inTableAsk'),
                                            DT::dataTableOutput('OGccfTable'),
                                            h3('Parental relations:'),
                                            textOutput('colParents'),
                                            tags$head(tags$style("#colParents{color: black;
                                font-size: 17px;
                                }"))
                                ),
                                    tabPanel('seaObject table',
                                            h3('seaObject Cancer Cell Fractions 
                                            table'),
                                            textOutput('inInputsAsk'),
                                            DT::dataTableOutput('ccfTable')),
                                    tabPanel('Plots',
                                            tabBox(id = 'box_plots',
                                                    width = NULL, height = NULL,
                                                    tabPanel('Shark & Dolphin plots',
                                                            textOutput('inShkDphAsk'),
                                                            shinycssloaders::withSpinner(
                                                                girafeOutput('intPlot'))
                                                    ),
                                                    tabPanel('Plaice plot',
                                                            textOutput('inPlaiceAsk'),
                                                            shinycssloaders::withSpinner(
                                                                girafeOutput('plaicePlot')))
                                            )
                                    )
                            )),
                        
                        
                        ## tutorial
                        tabItem(
                            tabName = 'instructions',
                            fluidRow(
                                align="center",
                                HTML('<h1><i class="fas fa-book"></i> <b> Tutorial </b></h1>')),
                            br(),
                            
                            fluidRow(column(1),
                                    column(10,
                                            box(
                                                width = NULL,
                                                height = NULL,
                                                includeHTML(system.file(package = "clevRvis", 
                                                                        "www" ,
                                                                        "tutorial_text.html")))),
                                    column(1))
                        ), #end of instructions tab
                        
                        ## About Us
                        tabItem(
                            tabName = "information",
                            fluidRow(
                                align="center",
                                HTML('<h1><i class="fas fa-info"></i> <b> About Us</b></h1>')),
                            br(),
                            fluidRow(column(1),
                                    column(10,
                                            box(
                                                width = NULL,
                                                height = NULL,
                                                includeHTML(system.file(package = "clevRvis", 
                                                                        "www" ,
                                                                        "aboutUs_text.html")))),
                                    column(1))
                        ) #end of about us
                    ),
                    tags$head(
                        tags$link(
                            rel = "stylesheet",
                            type = "text/css",
                            href = "appStyle.css")
                    )
                ) #end of body
            ))
            
            
        ),
        server=function(input, output, session) {
            vlines <- NULL
            vlabs <- NULL
            symbs <- NULL
            xlab <- NULL
            ylab <- NULL
            annotsTbl <- NULL
            
            ##helpers
            observe_helpers(session = session)
            
            # First Pop Up ###############################
            observeEvent("", {
                showModal(modalDialog(
                    includeHTML(system.file(package = "clevRvis", "www" ,
                                            "intro_text.html")),
                    easyClose = TRUE,
                    footer = tagList(
                        actionButton(inputId = "modalButton",
                                    label = "UNDERSTOOD",
                                    icon = icon("thumbs-up"),
                                    color = "success")
                    )
                ))
            })
            
            observeEvent(input$modalButton,{
                removeModal()
            })
            ######
            
            ##Ask for input table
            output$inTableAsk <- renderText({
                if (!is.null(input$fileCCF)){
                    t <- NULL
                } else {
                    t <- 'Please upload a Cancer Cell Fractions table first 
                    ("Inputs").'
                }
            })
            
            
            ##Save the input table in reactive value
            rawFile <- reactive({
                    req(input$fileCCF)
                    file <- input$fileCCF
                    ext <- tools::file_ext(file$datapath)
                    
                    if (ext == 'xlsx' | ext == 'xls') {
                        if (input$tpCols){
                            fracTable <-
                                as.matrix(read_excel(file$datapath,
                                                    col_names = TRUE))
                        } else {
                            fracTable <-
                                as.matrix(read_excel(file$datapath,
                                                    col_names = FALSE))
                        }
                        
                    }else if (ext == 'csv'){
                        if (input$tpCols){
                            fracTable <- as.matrix(read_csv(file$datapath))
                        } else {
                            fracTable <-
                                as.matrix(read_csv(file$datapath, 
                                                    col_names = FALSE))
                        }
                        
                    }else {
                        stop('File must be an excel (.xls / .xlsx) or a 
                            CSV (.csv) ',
                            file$datapath, ' is not a valid file.')
                    }
                    
                return(fracTable)
                
            })
            
            #Check input options selected
            output$warnParents <- renderText({
                df <- rawFile()
                suppressWarnings( 
                    if(!input$tpCols & sum(df[1,]=='parents', 
                                            na.rm = TRUE) == 1){
                        t <- "Please indicate through the corresponding switch 
                        that the first
            row of the input file contains the numeric time points"
                    } else if(is.na(sum(as.numeric(df[,1]))) & ! input$labRows){
                        t <- "Please indicate throught the corresponding switch 
                        that the first
            column of the input file contains clone labels"
                    } else if(input$parentCol=='column' & (sum(colnames(df)=='parents')==0)){
                        t <- 'Please select "Define parental relations 
                        interactively"'
                    } else if(input$parentCol == 'interactive' &
                            (sum(colnames(df)=='parents')==1)){
                        t <- 'The input file contains the column "parents", to 
                        interactively
            define parental relations, upload a file without the column "parents"'
                    } else { t <- NULL}
                )
            })
            
            ##Process ccf table
            fracTableR <- reactive({
                suppressWarnings({ 
                    req(input$fileCCF)
                    fracTable <- rawFile()
                    checkFirstCol <- sum(as.numeric(fracTable[,1]))
                    
                    if(sum(colnames(fracTable)=='parents') == 1){
                        pCol <- which(colnames(fracTable)=='parents')
                        fracTable <- fracTable[,-pCol]
                    }
                    if(sum(fracTable[1,]=='parents', na.rm = TRUE) == 1){
                        pCol <- which(fracTable[1,]=='parents')
                        fracTable <- fracTable[,-pCol]
                        coln <- fracTable[1,]
                        fracTable <- fracTable[2:nrow(fracTable),]
                    }
                    if (is.na(checkFirstCol)){ #input$labRows){
                        rown <- fracTable[,1]
                        coln <- colnames(fracTable)[2:ncol(fracTable)]
                        if(dim(fracTable)[1]==1){
                            fracTable <- dplyr::as_data_frame(matrix(
                                fracTable[,2:ncol(fracTable)], ncol = length(coln)))
                            fracTable <- apply(fracTable, 2, as.numeric)
                            fracTable <- dplyr::as_data_frame(matrix(
                                fracTable, ncol = length(fracTable)))
                            
                        } else {
                            fracTable <- dplyr::as_data_frame(fracTable[,2:ncol(fracTable)])
                            fracTable <- apply(fracTable, 2, as.numeric)
                            fracTable <- round(fracTable, 2)
                            
                        }
                        rownames(fracTable) <- rown
                        colnames(fracTable) <- coln
                    }
                }) #end of suppressWarnings
                
                return(fracTable)
                
            }) #end of fracTable processing
            
            ##Show ccf table OUTPUT
            output$OGccfTable <- DT::renderDataTable({
                    df <- fracTableR()
                    if (!input$tpCols){
                        tps <- getTimepoints()
                        if (length(tps) < ncol(df)){
                            colnames(df) <- c(NULL,tps)
                        } else {
                            colnames(df) <- tps
                        }
                    }
                    if (!input$labRows){
                        cloneLabs <- getCloneLabs()
                        rownames(df) <- cloneLabs
                    }
                    
                    DT::datatable(round(df,2),
                                extensions = 'Buttons', options = list(
                                    dom = 'Blfrtip',  
                                    buttons = list('copy', 'print', list(
                                        extend = 'collection',
                                        buttons = c('csv', 'excel'),
                                        text = 'Download')),
                                    scrollX=TRUE, scrollCollapse=TRUE))
            })
            
            ##Show parent vector as text output
            output$colParents <- renderText({
                df <- rawFile()
                if(input$expTrees | input$parentCol=='interactive' ){
                    p_vec <- getParents()
                } else if(input$parentCol=='column'){ #column of parental 
                    #relations
                    if(sum(colnames(df)=='parents')==1){
                        pCol <- which(colnames(df)=='parents')
                        p_vec <- as.numeric(df[,pCol])
                    } else {p_vec <- NULL}
                }
                p_vec
                
            })
            
            output$outExample <- renderText({
                a <- input$exampleData
                a
            })
            
            ##Create inputs for specifying parental relations
            output$parents <- renderUI({
                df <- fracTableR()
                nclones <- nrow(df)
                cloneLabs <- getCloneLabs()
                
                lapply((seq_len(nclones)), function(i) {
                    list(pickerInput(paste0('p', i), label = cloneLabs[i],
                                    choices = c('Normal cell',cloneLabs)[-(i+1)],
                                    width = 'fit')
                    )
                }) #end of lapply
            }) # end of renderUI
            
            ##Ask for input table
            output$askTblParents <- renderText({
                if (is.null(input$fileCCF) & (input$parentCol=='interactive')){
                    t <- paste('Please upload an input Cancer Cell 
                            Fractions table',
                            'to specify parental relations', sep = '<br/>')
                } else {
                    t <- NULL
                }
            })
            
            ##Get clone labels
            getCloneLabs <- reactive({
                if (input$labRows){ #first col is clone labs
                    cloneLabs <- rownames(fracTableR())
                } else if (input$cloneLabs){ # default clone labs
                    cloneLabs <- paste('Clone:',
                                    as.character(seq_len(nrow(fracTableR()))))
                } else { #not default clone labs
                    cloneLabs <- strsplit(input$cloneLabsValues, ",")[[1]]
                    
                }
                
                return(cloneLabs)
            })
            
            ##Get Timepoints
            getTimepoints <- reactive({
                df <- fracTableR()
                
                if (input$tpCols){ #first row is tps
                    if (colnames(df)[1]=='...1'){
                        tp <- colnames(df)[2:ncol(df)]
                    } else {
                        tp <- colnames(df)
                    }
                    
                } else if (input$dfltTP){ #default tps
                    tp <- seq_len(ncol(df))
                    
                } else { #manually specified tps
                    tp <- as.numeric(strsplit(input$timepoints, ",")[[1]])
                }
                return(as.numeric(tp))
            })
            
            
            
            ##Get parents vector
            getParents <- eventReactive(input$subInputs,{
                df <- rawFile()
                
                if (input$expTrees){ #using explore alternative trees
                    p_vec <- getAltTrees() %>% pluck(input$optParents)
                } else if(input$parentCol=='column'){ #column of parental 
                    #relations
                    if(sum(colnames(df)=='parents')==1){
                        pCol <- which(colnames(df)=='parents')
                        p_vec <- as.numeric(df[,pCol])
                    } else {p_vec <- NULL}
                } else { #manually defined parental relations
                    cloneLabs <- getCloneLabs()
                    p_vec <- c()
                    for (j in seq_len(length(cloneLabs))) {
                        p <- input[[paste0("p",j)]]
                        if (p == 'Normal cell') {
                            p_vec <- c(p_vec,0)
                        } else {
                            p_id <- which(cloneLabs == p)
                            p_vec <- c(p_vec, p_id)
                        }
                    }
                }
                return(p_vec)
            })
            
            ##Get list with alternative parental relations
            getAltTrees <- reactive({
                df <- fracTableR()
                # if (input$parentCol == 'column'){
                #   df <- df[,-which(colnames(df)=="parents")]
                # }
                tps <- getTimepoints()
                x <- exploreTrees(fracTable = df, timepoints = tps)
                if (is.null(x))
                    x <- "No possible trees"
                return(x)
            })
            
            ##Temp check
            # output$checkAltTrees <- renderText({
            #   x <- getAltTrees()
            #   paste('tree', names(x))
            # })
            
            ##Update dropdown (selectInput) explore trees
            observe({
                x <- getAltTrees()
                
                updateSelectInput(session, "optParents",
                                    label = 'Alternative parental relations:',
                                    choices = names(x)
                )
                
            })
            
            ####CHECKS
            # output$checkColNames <- renderText({
            #   df <- fracTableR()
            #   colnames(df)
            # })
            # output$checkParents <- renderText({
            #   parents <- getParents()
            #   paste('parents',parents)
            # })
            
            #get reactive vector with warning messages
            validateInputs <- eventReactive(input$subInputs, {
                fracTable <- fracTableR()
                # if(input$parentCol=='column'){#one of the columns is parental 
                #relations
                #   pCol <- which(colnames(fracTable)=='parents')
                #   fracTable <- fracTable[,-pCol]
                # }
                parents <- getParents()
                cloneLabels <- getCloneLabs()
                warns <- c()
                clones <- seq_len(dim(fracTable)[1])
                timepts <- seq_len(dim(fracTable)[2])
                given_tps <- getTimepoints()
                
                if(is.null(parents)){
                    warns <- c(warns, 'Please define parental relations 
                    correctly.
                    No parental relations were defined.')
                }
                
                ##Defined timepoints must be numeric values
                if(sum(is.na(as.numeric(given_tps)))>0){
                    warns <- c(warns,'At least one of the timepoints defined is 
                    not a
                    numeric value. ALL TIMEPOINTS MUST BE NUMERIC VALUES.')
                }
                
                ##no cluster can go from present to absent and then back
                ##difference in clusters cannot go from present to absent and 
                ##then back
                for(clone in clones){
                    startedClone <- FALSE
                    endedClone <- FALSE
                    startedDiff <- FALSE
                    endedDiff <- FALSE
                    
                    for(timept in timepts){
                        ##check individual clones
                        if(fracTable[clone,timept] > 0){
                            if(startedClone & endedClone){
                                warns <- c(warns,paste("Clone",clone,"goes from 
                                present
                        to absent (fraction=0) and then back to present."))
                            }
                            startedClone <- TRUE
                        } else {
                            if(startedClone){
                                endedClone <- TRUE
                            }
                        }
                        
                        ##check difference in clusters
                        diffr <- fracTable[clone,timept] -
                            sum(fracTable[which(parents==clone),timept])
                        if( diffr > 0){
                            if(startedDiff & endedDiff){
                                warns <- c(warns,paste("The difference between",
                                                        clone,
                                                        "and its subclones goes 
                                                        from present to absent
                    (difference=0) and then back to present. Subclones can't
                    have the same CCF as the parent (difference = 0) and then
                    have less CCF again."))
                            }
                            startedDiff <- TRUE
                        } else {
                            if(startedDiff){
                                endedDiff <- TRUE
                            }
                        }
                    }
                }
                
                ##clusters of entirely zero get a warning
                if(length(which(rowSums(fracTable) == 0)) > 0){
                    warns <- c(warns, "At least one cluster has fraction zero 
                    at all
            timepoints. It will not be displayed")
                }
                
                ##each timepoint doesn't sum to more than the parental value
                #at a given nest level (or 100% for level 0)
                for(timept in timepts){
                    neighbors <- which(parents == 0)
                    if(sum(fracTable[neighbors,timept]) > 100){
                        warns <- c(warns, paste("clones with same nest level 
                        cannot
                have values that sum to more than 100%: Problem is in
                clusters ", paste(neighbors,collapse=",")))
                    }
                    
                    for(i in unique(parents)){
                        if(i > 0){
                            neighbors <- which(parents==i)
                            if(sum(fracTable[neighbors,timept]) >
                                fracTable[parents[neighbors[1]], timept]){
                                warns <- c(warns, paste("clones with same 
                                parent cannot
                        have values that sum to more than the percentage of the
                        parent: Problem is in clusters ",
                                                        paste(neighbors,
                                                            collapse=","),
                                                        "at timepoint",
                                                        timept))
                                
                            }
                        }
                    }
                }
                
                ##the number of clone labels is equal to the number of clones
                if(length(cloneLabels) != nrow(fracTable)){
                    warns <- c(warns, paste("number of cloneLabels provided must
            be equal to the number of clones"))
                }
                return(unique(warns))
            })
            
            ##Success / warning messages
            observeEvent(input$subInputs,{
                warns <- validateInputs()
                if (length(warns) == 0){
                    sendSweetAlert(
                        session = session,
                        title = "Success!!",
                        text = "Parental relations have been successfully 
                        defined",
                        type = "success"
                    )
                } else {
                    sendSweetAlert(
                        session = session,
                        title = "WARNING...",
                        text = warns,
                        type = "error"
                    )
                }
            })
            
            ##Update default original timepoint (manual choice)
            observeEvent(input$subInputs,{
                timepoints <- getTimepoints()
                if (length(timepoints)==1){
                    val <- -30
                } else {
                    val <- timepoints[1] - (timepoints[2] - timepoints[1])
                }
                updateNumericInput(session = session,
                                    inputId = 'ogTP',
                                    max = min(timepoints)-1,
                                    value = val)
            })
            
            ##Update slider to select measured timepoints for therapy estimation
            observeEvent(input$subInputs,{
                timepoints <- getTimepoints()
                updateSliderTextInput(session = session,
                                    inputId = 'MtpThp',
                                    choices = timepoints,
                                    selected = c(timepoints[1],timepoints[2]))
            })
            
            ##Update slider to select specific timepoint for therapy estimation
            observeEvent(input$subInputs,{
                timepoints <- getTimepoints()
                updateSliderInput(session = session, inputId = 'tpThp',
                                    min = min(timepoints),
                                    max = max(timepoints),
                                    value = 0,
                                    step = 0.5)
            })
            
            ##Ask for submit seaObject (click button pls)
            observeEvent("", {
                output$inInputsAsk <- renderText({
                    t <- 'Please upload a Cance Cell Fractions table first 
                    ("Inputs" ->
        "Submit inputs"). Then, configure the seaObject ("seaObject options" ->
        "Submit") to visualize the seaObject Cancer Cell Fractions table.'
                })
            })
            ##Remove text asking for submit seaObject
            observeEvent(input$submitSea,{
                output$inInputsAsk <- renderText({
                    t <- NULL
                })
            })
            
            ##Validate sea Object
            validateSeaObjectFilters <- eventReactive(input$submitSea, {
                
                tps <- getTimepoints()
                warns <- c()
                if(input$therapyEffect){
                    if (input$thpEf_opt == 1){ ##thp effect is specified 
                        #timepoint
                        if (input$tpThp %in% tps){
                            warns <- c(warns, 'Specific timepoint defined for 
                            therapy
                    effect can not be a measured timepoint')
                        }
                    } else { ##thp eff defined as two measured tps
                        therapyEffectMtps <- as.numeric(input$MtpThp)
                        if (which(tps==therapyEffectMtps[2]) -
                            which(tps==therapyEffectMtps[1])>1){
                            warns <- c(warns, 'Measured Timepoints selected for 
                            therapy
                            effect must be contiguous, no other measured
                            timepoint must be in between.')
                        } else if (which(tps==therapyEffectMtps[2]) -
                                    which(tps==therapyEffectMtps[1])==0){
                            warns <- c(warns, 'Measured Timepoints selected for 
                            therapy
                            effect can not be the same one. To select a specific
                            timepoint click on "Specific Timepoint" when choosing
                            the type of Therapy Effect Estimation.')
                        }
                    }
                }
                
                if(length(tps)==1 & !input$tpEstim){
                    warns <- c(warns, 'To visualize clonal evolution from single
            timepoint extra timepoints estimation must be selected.')
                }
                if (length(tps)==1 & !input$customOgTP){
                    warns <- c(warns, "To visualize clonal evolution from a 
                    single
            timepoint a timepoint of origin must be manually specified.")
                }
                return(warns)
            })
            
            ##check inputs are there (and correct) before clicking submit sea 
            ##object
            observeEvent(input$submitSea,{
                err <- validateSeaObjectFilters()
                if (!input$subInputs){
                    sendSweetAlert(
                        session = session,
                        title = "ERROR!",
                        text = 'Submit inputs before clicking submit on 
                        seaObject filters',
                        type = "error"
                    )
                } else if (length(err)>0){
                    sendSweetAlert(
                        session = session,
                        title = "ERROR!",
                        text = err,
                        type = "error"
                    )
                } else {
                    sendSweetAlert(
                        session = session,
                        title = "SUCCESS!!",
                        text = 'All in order',
                        type = "success"
                    )
                }
            })
            
            #reactive object SeaObject will update every time we click submit
            seaObjectR <- eventReactive(input$submitSea,{
                err <- validateSeaObjectFilters()
                fracTable <- fracTableR()
                timepoints <- getTimepoints()
                cloneLabels <- getCloneLabs()
                parents <- getParents()
                if(length(err)==0){
                    if(input$therapyEffect){
                        if (input$thpEf_opt == 1){
                            therapyEffect <- input$tpThp
                        } else if (input$thpEf_opt == 2){
                            therapyEffect <- as.numeric(input$MtpThp)
                        }
                    } else{
                        therapyEffect <- NULL}
                    
                    if(input$customOgTP){
                        ogTimepoint <- input$ogTP
                    } else{
                        ogTimepoint <- NULL
                    }
                    
                    
                    seaObject <- createSeaObject(fracTable = fracTable,
                                                timepoints = timepoints,
                                                parents = parents,
                                                timepointInterpolation = input$tpEstim,
                                                therapyEffect = therapyEffect,
                                                cloneLabels = cloneLabels,
                                                originTimepoint = ogTimepoint)
                }
                
                
                return(seaObject)
                
            }) #end of create SeaObject
            
            
            ##TABLE WITH CCF AFTER SEAOBJECT IS CREATED
            observeEvent(input$submitSea,{
                output$ccfTable <- DT::renderDataTable({
                    seaObject <- seaObjectR()
                    df <- dplyr::as_data_frame(seaObject@fracTable)
                    parents <- isolate(getParents())
                    if(input$ccfVSprop == "prop"){
                        df_new <- df
                        for (i in seq_len(ncol(df))){
                            for (j in seq_len(nrow(df))){
                                df_new[j,i] <- df[j,i] - sum(df[which(parents==j),i])
                            }
                        }
                        df <- df_new
                    }
                    suppressWarnings({
                        df <- df %>% add_column(parents)
                        cloneLabs <- getCloneLabs()
                        rownames(df) <- cloneLabs
                        DT::datatable(df, 
                                    extensions = 'Buttons', options = list(
                            scrollX=TRUE, scrollCollapse=TRUE,
                            dom = 'Blfrtip',  
                            buttons = list('copy', 'print', list(
                                extend = 'collection',
                                buttons = c('csv', 'excel'),
                                text = 'Download')
                            ))
                        )
                    }) #end of suppressWarnings
                })
            })
            
            ####PLOTS####
            
            ##Ask to activate the switches to visualize the plots (shark 
            ##dolphin)
            output$inShkDphAsk <- renderText({
                if(!input$shark & !input$dolphin){
                    t <- 'Please configure the seaObject first
        ("seaObject options" -> "Submit"). Then, select the plots to visualize
        ("Plots" -> switches for Shark, Dolphin and Plaice Plots).'
                } else {t <- NULL}
            })
            
            
            ##INTERACTIVE PLOTS
            output$intPlot <- renderGirafe({
                req(input$shark | input$dolphin )
                seaObject <- seaObjectR()
                timepoints <- getTimepoints()
                if(input$dolphin){
                    ##Vertical Lines and Labels
                    vlines <- c()
                    vlabs <- c()
                    symbs <- c()
                    if('Measured time points' %in% input$vlines){
                        vlines <- c(vlines,timepoints)
                        if('Measured time points' %in% input$vlab){
                            vlabs <- c(vlabs, timepoints)
                        }
                    }
                    if('Measured time points' %in% input$symbols){
                        symbs <- c(symbs, timepoints)
                    }
                    if('Estimated time points' %in% input$vlines){
                        estimatedTimepoints <-
                            seaObject@timepoints[!seaObject@timepoints %in% timepoints]
                        vlines <- c(vlines, estimatedTimepoints)
                        if('Estimated time points' %in% input$vlab){
                            vlabs <- c(vlabs, estimatedTimepoints)
                        }
                    }
                    if('Estimated time points' %in% input$symbols){
                        estimatedTimepoints <-
                            seaObject@timepoints[!seaObject@timepoints %in% timepoints]
                        symbs <- c(symbs, estimatedTimepoints)
                    }
                    if('Custom' %in% input$vlines){
                        customVlines <-
                            as.numeric(strsplit(input$customVlines, ",")[[1]])
                        vlines <- c(vlines,customVlines)
                    }
                    if('Custom' %in% input$vlab){
                        customVlabs <- strsplit(input$customVlabs, ",")[[1]]
                        vlabs <- c(vlabs, customVlabs)
                    }
                    if('Custom' %in% input$symbols){
                        customSymbs <-
                            as.numeric(strsplit(input$customSymbols, ",")[[1]])
                        symbs <- c(symbs,customSymbs)
                    }
                    if(input$axisLab){
                        if(length(input$xlab)>0){
                            xlab <- input$xlab
                        }else{ xlab <- NULL}
                        if(length(input$ylab)>0){
                            ylab <- input$ylab
                        }else{ ylab <- NULL}
                    }else{
                        xlab <- NULL
                        ylab <- NULL
                    }
                }
                if(input$legendDol | input$legendShk){
                    legend <- TRUE
                }else {legend <- FALSE}
                
                if (input$addAnnot){
                    annotsTbl <- getAnnots()
                } else {annotsTbl <- NULL}
                
                if (input$extendedShk & !input$dolphin){
                    tps <- c()
                    if('Measured time points' %in% input$showTps){
                        tps <- c(tps,timepoints)
                    }
                    if('Estimated time points' %in% input$showTps){
                        estimatedTimepoints <-
                            seaObject@timepoints[!seaObject@timepoints %in% timepoints]
                        tps <- c(tps, estimatedTimepoints)
                    }
                    if('Custom' %in% input$showTps){
                        customTps <- as.numeric(strsplit(input$customShowTps, ",")[[1]])
                        tps <- c(tps,customTps)
                    }
                        extSharkPlot(seaObject,
                                                showLegend = input$legendShk,
                                                main = input$mainShk,
                                                timepoints = tps)
                } else {
                        try(
                            combinedPlot(seaObject, dolphin = input$dolphin,
                                        shark = input$shark, vlines = vlines,
                                        mainShk = input$mainShk, 
                                        showLegend = legend,
                                        shape = input$shape, 
                                        borderCol = input$borderCol,
                                        vlineCol=input$vlineCol, vlab=vlabs,
                                        markMeasuredTimepoints = symbs, 
                                        pos = input$pos,
                                        separateIndependentClones = input$sepIndCl,
                                        mainDph = input$mainDol, 
                                        vlabSize = input$vlabSize,
                                        xlab = xlab, ylab = ylab, 
                                        annotations = annotsTbl,
                                        pad.left = input$pad)
                        )
                }
            })
            
            ##Create inputs for specifying clones to fill in plaiceplot
            output$fillVec <- renderUI({
                cloneLabs <- getCloneLabs()
                nclones <- length(cloneLabs)
                
                lapply((seq_len(nclones)), function(i) {
                    list(pickerInput(paste0('color', i), label = cloneLabs[i],
                                    choices = c('Empty',cloneLabs),
                                    width = 'fit')
                    )
                }) #end of lapply
            }) # end of renderUI
            
            ##Get clones to fill vector
            getClonesToFill <- reactive({
                req(input$fillClones)
                cloneLabs <- getCloneLabs()
                f_vec <- c()
                for (j in seq_len(length(cloneLabs))) {
                    c <- input[[paste0("color",j)]]
                    if (c == 'Empty') {
                        f_vec <- c(f_vec,0)
                    } else {
                        f_vec <- c(f_vec,which(cloneLabs == c))
                    }
                }
                return(f_vec)
            })
            
            #Ask to activate the plaiceplot switch to visualize the plot
            output$inPlaiceAsk <- renderText({
                if(!input$plaice){
                    t <- 'Please configure the seaObject first
        ("seaObject options" -> "Submit"). Then, select the plots to visualize
        ("Plots" -> switches for Shark, Dolphin and Plaice Plots).'
                } else {t <- NULL}
            })
            
            ##PLAICEPLOT
            output$plaicePlot <- renderGirafe({
                req(input$plaice)
                seaObject <- seaObjectR()
                timepoints <- getTimepoints()
                estimatedTimepoints <-
                    seaObject@timepoints[!seaObject@timepoints %in% timepoints]
                
                ##vector clones to fill
                if (input$fillClones){
                    fillClones <- getClonesToFill()
                } else {
                    fillClones <-rep(0,length(seaObject@col))
                }
                ##annotations
                if (input$addAnnot){
                    annotsTbl <- getAnnots()
                } else {annotsTbl <- NULL}
                
                ##Vertical Lines, Labels and symbols
                vlines <- c()
                vlabs <- c()
                symbs <- c()
                if('Measured time points' %in% input$vlinesPlc){
                    vlines <- c(vlines,timepoints)
                }
                if('Measured time points' %in% input$vlabPlc){
                    vlabs <- c(vlabs, timepoints)
                }
                if('Measured time points' %in% input$symbolsPlc){
                    symbs <- c(symbs, timepoints)
                }
                if('Estimated time points' %in% input$vlinesPlc){
                    vlines <- c(vlines, estimatedTimepoints)
                }
                if('Estimated time points' %in% input$vlabPlc){
                    vlabs <- c(vlabs, estimatedTimepoints)
                }
                if('Estimated time points' %in% input$symbolsPlc){
                    symbs <- c(symbs, estimatedTimepoints)
                }
                if('Custom' %in% input$vlinesPlc){
                    customVlines <-
                        as.numeric(strsplit(input$customVlinesPlc, ",")[[1]])
                    vlines <- c(vlines,customVlines)
                }
                if('Custom' %in% input$vlabPlc){
                    customVlabs <- strsplit(input$customVlabsPlc, ",")[[1]]
                    vlabs <- c(vlabs, customVlabs)
                }
                if('Custom' %in% input$symbolsPlc){
                    customSymbs <-
                        as.numeric(strsplit(input$customSymbolsPlc, ",")[[1]])
                    symbs <- c(symbs,customSymbs)
                }
                    plaicePlot(seaObject, clonesToFill = fillClones,
                            shape = input$shapePlc,
                            borderCol = input$borderColPlc, vlines=vlines,
                            vlineCol = input$vlineColPlc, vlab=vlabs,
                            vlabSize = input$vlabSizePlc, 
                            annotations=annotsTbl,
                            separateIndependentClones = input$sepIndClPlc,
                            showLegend = input$legendPlc,
                            markMeasuredTimepoints = symbs,
                            main = input$mainPlc, ylab = input$axisLabPlc)
            })
            
            
            
            ##Create inputs for annotations
            output$annot <- renderMenu({
                req(input$submitSea & req(input$addAnnot))
                tps <- seaObjectR()@timepoints
                cloneLabs <- getCloneLabs()
                nclones <- length(cloneLabs)
                sidebarMenu(
                    lapply((seq_len(nclones)), function(i) {
                        list(menuItem(paste(cloneLabs[i]),
                                    ##Annotations text
                                    textInput(paste0('annot',i),
                                                paste('Annotations',
                                                    cloneLabs[i])),
                                    ##Color
                                    switchInput(paste0("col",i), "Text color",
                                                labelWidth = "80px",
                                                onLabel = 'White', 
                                                offLabel = 'Black',
                                                onStatus = 'default'
                                    ),
                                    ##X position
                                    sliderInput(paste0('x', i), 
                                                label = 'X position',
                                                max = round(max(tps),0)+((max(tps)-min(tps))/2),
                                                min = round(min(tps),0)-((max(tps)-min(tps))/2),
                                                value = 0, step=(max(tps)-min(tps))*0.05
                                    ),
                                    ##Y position
                                    sliderInput(paste0('y', i), 
                                                label = 'Y position',
                                                max = 110,
                                                min = -110,
                                                value = 50, step=1)
                                
                        )
                        
                        )
                    }) #end of lapply
                )#end of sidebarMenu
            }) # end of renderMenu
            
            ##Generate the annotations table
            getAnnots <- reactive({
                
                cloneLabs <- getCloneLabs()
                x_vec <- c()
                y_vec <- c()
                annot_vec <- c()
                col_vec <- c()
                for (j in seq_len(length(cloneLabs))) {
                    x_vec <- c(x_vec, input[[paste0("x",j)]])
                    y_vec <- c(y_vec, input[[paste0("y",j)]])
                    annot_vec <- c(annot_vec, input[[paste0("annot",j)]])
                    if (input[[paste0('col',j)]]){
                        col_vec <- c(col_vec, 'White')
                    } else {
                        col_vec <- c(col_vec, 'Black')
                        
                    }
                    #col_vec <- c(col_vec, input[[paste0('col',j)]])
                    
                }
                annot_df <- data.frame(x = x_vec, y = y_vec, lab = annot_vec,
                                        col = col_vec)
                
                return(annot_df)
            })
            
            
            session$onSessionEnded(function() {
                stopApp()
            })
        }
        
    )
}

exploreTrees <- function(fracTable, timepoints){

    if(!is.matrix(fracTable) || !is.numeric(fracTable)){
        stop("No valid input for fracTable provided.")
    }
    if(!is.vector(timepoints) || !is.numeric(timepoints)){
        stop("No valid input for timepoints provided.")
    }
    if(ncol(fracTable) != length(timepoints)){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", ncol(fracTable), 
            " vs ", length(timepoints), ").")
    }
    
    #get matrix (TRUE/FALSE) indicating each clone's possible parents
    possible <- .possibleParents(fracTable)

    #when only one option for parent -> check where it sums up to >100
    possibilities <- .checkPossibleParents(fracTable, possible)

    #final validity check
    #is number of options to consider <20,000?
    solutions <- .getParentOptions(possibilities, fracTable, timepoints)

    return(solutions)
}

plaicePlot <- function(seaObject, shape = 'spline', borderCol = 'black',
                        vlines=NULL, vlineCol="#6E6E66",
                        vlab=NULL, vlabSize = 3,
                        separateIndependentClones = FALSE,
                        clonesToFill = NULL,
                        showLegend = FALSE, markMeasuredTimepoints = NULL,
                        main = NULL, mainPos = 'middle', mainSize = 5,
                        xlab = NULL, ylab = FALSE, pad.left = 0.005,
                        annotations = NULL, annotationsSize=3,
                        interactivePlot = TRUE){
    
    if(is(seaObject, "seaObject") == FALSE){
        stop('No seaObject provided as input.')
    }
    if(!is.null(borderCol) && is.null(.isColor(borderCol))){
        stop("No valid input for borderCol provided.")
    }
    if(!is.null(vlines) && !is.numeric(vlines)){
        stop("No valid input for vlines provided.")
    }
    if(is.null(.isColor(vlineCol))){
        stop("No valid input for vlineCol provided.")
    }
    if(!is.null(vlab) && !is.character(vlab) && !is.numeric(vlab)){
        stop("No valid input for vlab provided.")
    }
    if(!is.numeric(vlabSize)){
        stop("No valid input for vlabSize provided.")
    }
    if(separateIndependentClones != TRUE && separateIndependentClones != FALSE){
        stop("No valid input for separateIndependentClones provided.")
    }
    if(!is.null(clonesToFill) && !is.numeric(clonesToFill)){
        stop("No valid input for clonesToFill provided.")
    }
    if(showLegend != TRUE && showLegend != FALSE){
        stop("No valid input for showLegend provided.")
    }
    if(!is.null(markMeasuredTimepoints) && !is.numeric(markMeasuredTimepoints)){
        stop("No valid input for markMeasuredTimepoints provided.")
    }
    if(!is.null(main) && !is.character(main)){
        stop("No valid input for main provided.")
    }
    if(!is.numeric(mainSize)){
        stop("No valid input for mainSize provided.")
    }
    if(!is.null(xlab) && !is.character(xlab)){
        stop("No valid input for xlab provided.")
    }
    if(ylab != TRUE && ylab != FALSE){
        stop("No valid input for ylab provided.")
    }
    if(!is.null(pad.left) && !is.numeric(pad.left)){
        stop("No valid input for pad.left provided.")
    }
    if(!is.null(annotations) && !is.data.frame(annotations)){
        stop("No valid input for annotations provided.")
    }
    if(!is.numeric(annotationsSize)){
        stop("No valid input for annotationsSize provided.")
    }
    if(interactivePlot != TRUE && interactivePlot != FALSE){
        stop("No valid input for interactivePlot provided.")
    }
    if(nrow(fracTable(seaObject)) != length(parents(seaObject))){
        stop("Number of clones defined in the fractable does not match number
            of clones defined in parents (", nrow(fracTable(seaObject)), " vs ",
            length(parents(seaObject)), ").")
    }
    if(ncol(fracTable(seaObject)) != length(timepoints(seaObject))){
        stop("Number of time points defined in the fractable does not match 
            number of time points defined in timepoints (", 
            ncol(fracTable(seaObject)), " vs ", length(timepoints(seaObject)), 
            ").")
    }
    
    x<-NULL
    y<-NULL
    name<-NULL
    ymin<-NULL
    ymax<-NULL
    tp<-NULL
    lab<-NULL


    ##get x and y values for bottom visualization (clone evo)
    seaObject <- .bottomLayout(seaObject,
                        separateIndependentClones = separateIndependentClones)

    if (shape == 'polygon'){
        coordsTblBtm <- .getCoordsTable(seaObject)

    }else if(shape == 'spline'){
        coordsTblBtm <- .getSmoothCoordsTbl(seaObject=seaObject,
                                        pad.left=pad.left)
    } else{
        stop('Shape must be wither "spline" or "polygon"')
    }

    ##get x and y values for top visualization (alleleaware)
    seaObject <-.topLayout(seaObject,
                        separateIndependentClones = separateIndependentClones)

    if (shape == 'polygon'){
        coordsTblTop <- .getCoordsTable(seaObject)

    }else if(shape == 'spline'){
    coordsTblTop <- .getSmoothCoordsTbl(seaObject=seaObject, pad.left=pad.left)
    }

    ##Color vector for allele marking
    btmCloneColors <- .getBottomCloneCols(seaObject, clonesToFill=clonesToFill)


    ##Plaice plot
    plcPlot <- ggplot(coordsTblBtm, aes(x = x, y = y)) +
        geom_polygon_interactive(aes(fill = name, group = name,
                                    tooltip = name,
                                    data_id = name),
                                show.legend = showLegend,
                                color = borderCol,
                                size = 0.25
        ) +
        scale_fill_manual(values = c(seaObject@col),
                                            name = "Clone labels") +
        theme_void() + theme(legend.position = "left")+
        ##new scale for alleleaware coloring
        new_scale('fill') +
        geom_polygon_interactive(aes(fill = name, group = name,
                                    tooltip = name, data_id = name),
                                color = borderCol,
                                size = 0.25,
                                data = coordsTblTop) +
        scale_fill_manual(values = btmCloneColors) +
        guides(fill='none') + #no legend
        geom_hline(yintercept = 0, size = 0.7)



    ##VERTICAL LINES
    if(!is.null(vlines)){
        vlinesData <- data.frame(vlines = vlines,
                                ymin = rep(-100,length(vlines)),
                                ymax = rep(100,length(vlines)))

        plcPlot <- plcPlot + geom_segment(mapping = aes(x = vlines,
                                                        xend=vlines,
                                                        y = ymin,
                                                        yend = ymax),
                                        data = vlinesData, color = vlineCol,
                                        show.legend = FALSE)

        if(!is.null(vlab)){
            vlinesData <- vlinesData %>% add_column(vlab = vlab)
            plcPlot <- plcPlot + geom_text(mapping = aes(x = vlines,
                                                        y = 100,
                                                        label = vlab,
                                                        vjust = -1),
                                        size = vlabSize,
                                        data = vlinesData) +
                expand_limits(y=100+vlabSize)
        }
    }

    ##BOTTOM TRIANGLES
    if(!is.null(markMeasuredTimepoints)){
        measuredTpData <- data.frame(tp = markMeasuredTimepoints,
                                    y = rep(-102,
                            length(markMeasuredTimepoints)))
        plcPlot <- plcPlot + geom_point(mapping = aes(x = tp, y = y),
                                        data = measuredTpData,
                                        shape = 'triangle',
                                        size = 2)
    }

    ##MAIN TITLE
    if(!is.null(main)){
        if(mainPos == 'middle'){
            mainCoords <- (min(seaObject@timepoints) +
                            max(seaObject@timepoints))/2
        } else{
            if(mainPos == 'left'){
                mainCoords <- min(seaObject@timepoints)

            }else{
                if(mainPos == 'right'){
                    mainCoords <- max(seaObject@timepoints)
                }else{
                    stop('main position (mainPos) must be "left", "middle",
                        or "right"')
                }
            }
        }

        mainY <- mainSize*4 + 100

        if(mainY < 110 & !is.null(vlab)){
            mainY    <- 110
        }

        mainLabel <- data.frame(lab = main, x = mainCoords, y = mainY)

        plcPlot <- plcPlot + geom_text(mapping = aes(x = x, y = y,
                                                    label = lab),
                                    size = mainSize,
                                    data = mainLabel)
    }

    ##X AXIS LABEL
    if(!is.null(xlab)){

        axisLabelX <- data.frame(lab = xlab,
                                x = ((min(seaObject@timepoints) +
                                        max(seaObject@timepoints))/2),
                                y = -106)

        plcPlot <- plcPlot + geom_text(mapping = aes(x = x, y = y,
                                                    label = lab),
                                        data = axisLabelX)
    }

    ##Y AXIS LABEL (+ LINES AND %)
    if(ylab){
        pad <- max(coordsTblTop$x)-min(coordsTblTop$x)
        yLine <- data.frame(x = rep(min(coordsTblTop$x) - pad*0.05,2),
                                                ymin = c(0.5,-0.5),
                                                ymax = c(100,-100))
        percentages <- data.frame(lab = c('0%','100%','100%','0%'),
                                x = rep(min(coordsTblTop$x) - pad*0.1,2),
                                y = c(4,100,-4,-100))
        ylabs <- data.frame(lab=c('Remaining healthy allele',
                                'Cancer Cell Fraction'),
                            x = rep(min(coordsTblTop$x) - pad*0.15,2),
                            y = c(-50,50))

        plcPlot <- plcPlot +
            geom_segment(mapping = aes(x = x, xend=x, y = ymin, yend = ymax),
                                    data = yLine, color = 'black',
                        show.legend = FALSE) +
            geom_text(mapping = aes(x = x, y = y, label = lab),
                                data = percentages,
                                size=2.5) +
            geom_text(mapping = aes(x = x, y = y, label = lab, angle = 90),
                                data = ylabs,
                                size=3.7)

    }
    ##Annotations
    if(!is.null(annotations)){
        if(sum(annotations$col %in% c('black','Black')) == 0){
            colorsScale <- 'white'
        } else {
            colorsScale <- c('black','white')
        }
        plcPlot <- plcPlot + geom_text(mapping = aes(x = x, y = y,
                                                    label = lab,
                                                    color = col),
                                        size = annotationsSize,
                                        data = annotations) +
            guides(color = 'none') +
            scale_color_discrete(type = colorsScale)
    }

    if (interactivePlot){
        ###MAKE INTERACTIVE
    tooltip_css <- "color:black;padding:10px;border-radius:10px 20px 10px 20px;"

        plcPlot <- girafe(ggobj = plcPlot, width_svg = 10, height_svg = 10) %>%
            girafe_options(opts_sizing(rescale = FALSE),
                                        opts_hover_inv(css = "opacity:0.3;"),
                                        opts_hover(css = "opacity:1;"),
                                        opts_tooltip(css = tooltip_css,
                    use_fill=TRUE))
    }
    return(plcPlot)
}

---
title: "clevRvis Vignette"
output: 
    BiocStyle::html_document:
    toc: true
    toc_depth: 2
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown
vignette: >
    %\VignetteIndexEntry{ClEvR Viz vignette}
    %\VignetteEncoding{UTF-8}  
    %\VignetteEngine{knitr::rmarkdown}
editor_options: 
    markdown: 
    wrap: 80
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Introduction

Clonal evolution describes the development of a tumor over time, considering the principle of 'survival of the fittest' on a cellular level. The development of cellular heterogeneity is of special interest with respect to cancer as it can be associated with therapy response, prognosis and survival of a patient. For example, relapse may be caused by an aggressive subclone.

The study of clonal evolution is based on mutations detected in a tumor sample. Ideally, data on mutations being present or absent is available for more than one time point in the cause of a patient's disease. In a first analysis step, mutations have to be clustered to define clones. Every clone is commonly characterized by the cancer cell fraction (CCF) for every time point. The clones can, subsequently, be arranged as phylogenetic trees to represent the tumor's evolution. A final, but essential step in the analysis of clonal evolution, significantly contributing to interpretation of the results, is its visualization.

clevRvis provides an extensive set of visualization techniques for clonal evolution. Three types of plots are available: 1) shark plots (basic trees, showing the phylogeny and optionally the cancer cell fraction CCF); 2) dolphin plots (advanced visualization, showing the phylogeny and the development of CCFs over time); 3) plaice plots (novel visualization, showing the phylogeny, the development of CCFs and the development of remaining healthy alleles, influenced by bi-allelic events, over time). Moreover, the tool provides algorithms for fully automatic interpolation of time points and estimation of therapy effect to approximate a tumor's development in the presence of few measured time points, as well as exploring alternative trees.

clevRvis provides all common features for visualization also available in established R-packages like 'timescape'. These features have been revised from scratch and optimized (e.g. supporting branched independent evolution, supporting visualization of a single clone, extensive validity check of the input, switch between bottom- and centered-layout). Furthermore, a wide set of additional features is available (e.g. automatic phylogeny-aware color-coding, algorithms for time point interpolation and therapy effect estimation, graphical user interface).

## Requirements

To run clevRvis, you need R (Version 4.1.0 or higher).

## Installation

To install clevRvis, type the following commands in R:

```{r installation, eval=FALSE, }
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("clevRvis")
```

## Running clevRvis

After installation, the package can be loaded into R by typing

```{r loading }
library(clevRvis)
```

clevRvis is available as a shiny GUI. You can run the shiny version either by RStudio executing the function `clevRvisShiny()`. Additionally, all functions for classical use in R are available:

1.  `createSeaObject()`: Create a seaObject

2.  `sharkPlot()`: Generate a basic graph visualization of clonal evolution

3.  `extSharkplot()`: Generate an extended graph visualization of clonal evolution

4.  `dolphinPlot()`: Generate a detailed visualization of clonal evolution

5.  `combinedPlot()`: Generate a combined basic graph and detailed visualization of clonal evolution

6.  `plaicePlot()`: Generate an allele-aware visualization of clonal evolution

7.  `exploreTrees()`: Generate alternative parental relations to explore alternative trees

# Examplary use of clevRvis

## createSeaObject()

clevRvis needs a seaObject for the visualization of clonal evolution by means of any available plot. When generating the seaObject extra time points may be interpolated and/or therapy effect estimated.

### Usage

    createSeaObject(fracTable,parents,timepoints,col=NULL,cloneLabels=NULL,
                    originTimepoint=NULL,timepointInterpolation=TRUE,
                    therapyEffect=NULL)

-   `fracTable` A numeric matrix containing tumor fraction estimates for all clones at all time points.
-   `parents` An integer vector specifying parental relationships between clones.
-   `timepoints` A numeric vector specifying the time points for each column of the matrix.
-   `col` (optional) A vector of colors to use when plotting each clone.
-   `cloneLabels` (optional) A character vector of names to assign to each clone when plotting a legend.
-   `originTimepoint` (optional) Time point when the first clone emerges (must be before the first measured time point).
-   `timepointInterpolation` When set to true extra time points will be interpolated between measured time points and before the first measure timepoint to improve the visualization (default: TRUE).
-   `therapyEffect` (optional) A single numeric value indicating the time point when to estimate the effect of therapy or a numeric vector containing two consecutive measured time points, therapy effect timepoint will be in the middle.

### Details

The basis for all plotting functions included in clevRvis is a seaObject. It contains information on the CCFs for all clones, at all measured time points. Additionally, parental information on the clones is included.

Additional time points may be interpolated, therapy effect estimated when generating seaObjects.

Time point interpolation is generally recommended to improve visualization of clonal evolution. When having less time points than clones, or many new clones emerging in one single measured time point, the extra time point interpolation is strongly recommended to visualize the clonal evolution properly. If there is only one measured time point, time point interpolation is needed and the time point of origin must be manually specified, as there is no way of calculating it.

To visualize the effect of therapy on the clones' CCFs in case of missing measured data, a fully automatic approach for therapy effect estimation is available. When creating the seaObject, a specific time point can be defined (between two measured time points) or two measured time points can be selected (new therapy effect time point will be in the middle) for the estimation of the therapy effect.

A seaObject with all relevant slots filled is returned.

#### Time point interpolation

For an improved visualization of clonal evolution, enabling time point interpolation is recommended.

#### Therapy effect estimation

When generating the seaObject, the effect of therapy can also be estimated. There's two options to define the therapy effect time point:

-   The therapyEffect is defined as a single numeric value in between measured time points -\> the therapy effect will be estimated at the given time point

-   The therapyEffect is defined as a vector containing two measured time points -\> the therapy effect will be estimated at the midpoint between them

### Examples

```{r inputs}
##Example data
timepoints <- c(0,50,100)
parents <- c(0,1,1,3,0,5,6)
fracTable <- matrix(c(20,10,0,0,0,0,0,
                    40,20,15,0,30,10,0,
                    50,25,15,10,40,20,15),
                    ncol = length(timepoints))
seaObject <- createSeaObject(fracTable, parents, timepoints,
                            timepointInterpolation = FALSE)
```

```{r seaObject}
##seaObject with enabled time point estimation
seaObject_tp <- createSeaObject(fracTable, parents, timepoints,
                                timepointInterpolation = TRUE)
```

```{r thpEff}
##seaObject with enabled time point estimation and therapy effect 
##estimation between time point 50 and 100
seaObject_te <- createSeaObject(fracTable, parents, timepoints,
                            timepointInterpolation = TRUE,
                            therapyEffect = c(50,100))
```

```{r individualColors}
##seaObject with manually defined colors
seaObject_col <- createSeaObject(fracTable, parents, timepoints,
                                timepointInterpolation = TRUE,
                                col = rainbow(7))
```

## sharkPlot()

A shark plot shows the basic graph visualization of clonal evolution with nodes representing clones and edges indicating their evolutionary relations.

### Usage

    sharkPlot(seaObject, showLegend, main)

-   `seaObject` A seaObject.
-   `showLegend` A boolean indicating whether to show the legend or not (default: FALSE).
-   `main` A string corresponding to the plot''s main title.

### Details

A shark plot is the basic approach for visualization: common trees, with nodes representing clones and edges indicating their evolutionary relation. Phylogeny can be directly deduced from these plots.

Shark plots also offer an extension to visualize the changes in CCF along time for each clone. CCFs of each clone (rows) at each time point (columns) are shown as points next to the basic shark plot (see `extSharkPlot`).

### Examples

```{r shark}
#Basic shark plot showing legend and title
sharkPlot(seaObject_tp, showLegend = TRUE, main = 'Example Shark plot')
```

```{r sharkIndivColors}
#Basic shark plot showing legend and title with manually defined color coding
sharkPlot(seaObject_col, showLegend = TRUE, main = 'Example Shark plot')
```

## extSharkPlot()

An extended shark plot shows the basic graph visualization of clonal evolution with nodes representing clones and edges indicating their evolutionary relations and additional visualization of CCFs.

### Usage

    extSharkPlot(seaObject, showLegend, main, timepoints, width, interactivePlot)

-   `seaObject` A seaObject.
-   `showLegend` A boolean indicating whether to show the legend or not (default: FALSE).
-   `main` A string corresponding to the plot's main title (default: NULL).
-   `timepoints` By default, all time points available in the seaObject are visualized. Optionally, a selected set of available time points can be chosen.
-   `width` An integer value indicating the width of the widget plot (default: 10).
-   `interactivePlot` A boolean defining whether the plot should be interactive (default: TRUE; if using this function to export the extended shark plot, e.g. by png(), define interactivePlot = FALSE).

### Details

An extended shark plots consists of two elements:

1.  A basic shark plot: common trees, with nodes representing clones and edges indicating their evolutionary relation. Phylogeny can be directly deduced from these plots.

2.  Additionally, CCFs of each clone (rows) at each time point (columns) are shown as points next to the basic shark plot. The size of each point correlates with the CCF at the corresponding clone and time point.

Both plots are linked in an interactive widget.

### Examples

```{r extShark}
##extended shark plot, showing CCF as point size only for measured 
###time points, legend and title
extSharkPlot(seaObject_tp, timepoints = timepoints, showLegend = TRUE, 
            main = 'Example Extended Shark plot')
```

## dolphinPlot()

Dolphin plots provide a detailed visualization of clonal evolution. Plots show the development of all clones over time (x axis) and their clonal prevalences (y axis).

### Usage

    dolphinPlot(seaObject, shape, borderCol, pos, vlines, vlineCol, vlab, 
                vlabSize, separateIndependentClones, showLegend, 
                markMeasuredTimepoints, main, mainPos, mainSize, xlab, 
                ylab, pad.left, annotations, annotSize)

-   `seaObject` A seaObject.
-   `shape` The type of shape to construct the plot out of. The options are "spline" and "polygon" (default: "spline"").
-   `borderCol` A color for the border line. If "NULL" then no border will be drawn (default: NULL).
-   `pos` Plotting position of the clones. Options are "center" or "bottom" (default: "center").
-   `vlines` A vector of positions at which to draw vertical lines (default: NULL).
-   `vlineCol` A color value for the vertical lines (default: "#6E6E66").
-   `vlab` A character vector containing labels for each of the vertical lines (default: NULL).
-   `vlabSize` An integer value for the vertical labels size (default: 3).
-   `separateIndependentClones` Boolean defining whether independently-arising clones (with parent 0) should be separated by blank space in the plot (default: FALSE).
-   `showLegend` A boolean indicating whether to show a legend at the left side of the plot (default: FALSE).
-   `markMeasuredTimepoints` A vector of x positions at which to draw triangles on the bottom of the plot (default: NULL).
-   `main` A string corresponding to the plot's main title (default: NULL).
-   `mainPos` A string defining the title's position. Options are 'left', 'middle' or 'right', always above the plot (default: "middle").
-   `mainSize` An integer value defining the size of the title (default: 5).
-   `xlab` A string defining the label of the x axis (default: NULL).
-   `ylab` A string defining the label of the y axis. Automatically, a vertical line showing 100% will be plotted (default: NULL).
-   `pad.left` The amount of "ramp-up" to the left of the first time point. Given as a fraction of the total plot width (default: 0.005).
-   `annotations` A data.frame with: columns "x" (x position), "y" (y position), "lab" (annotation text) and "col" (color of the text either black or white) (default: NULL).
-   `annotSize` An integer value defining the size of the annotations (default: 3).

### Details

Dolphin plots displays detailed information on clonal evolution, showing the development of all clones over time (x-axis) and their clonal prevalence (y-axis). Information on phylogeny, CCFs and time course characterizing a clonal evolution are jointly visualized in this single plot.

Several basic options for customizing dolphin plots are available, e.g. switching between spline and polygon shape, bottom or central visualization, annotations, separating independent clones, adding vertical lines and labels, changing border and vertical lines colors, etc.

Dolphin plots may be chosen to be plotted along with basic shark plots (see `combinedPlot()`).

### Examples

```{r splineDolph}
##Dolphin plot, with vertical lines showing all time points, custom y axis 
##label and triangles indicating the measured time points
dolphinPlot(seaObject_tp, showLegend = TRUE, 
            vlines = timepoints(seaObject), 
            vlab = timepoints(seaObject), vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)',
            markMeasuredTimepoints = timepoints)
```

```{r polyDolph}
##Dolphin plot polygon shape
dolphinPlot(seaObject_tp, showLegend = TRUE, vlines = timepoints,
            vlab = timepoints, vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)', shape = 'polygon')

```

```{r bottomDolph}
##Dolphin plot bottom layout and separated independent clones
dolphinPlot(seaObject_tp, showLegend = FALSE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)', 
            separateIndependentClones = TRUE, pos = 'bottom')
```

To add annotations to clones in the plot, a table must be created containing the following columns: \* `x` x position of the annotation text \* `y` y position of the annotation text \* `col` color of the text (options are 'black' or 'white') \* `lab` text of the annotation

```{r annotsDolph}
##Dolphin plot with annotations
annotsTable <- data.frame(x = c(50,75), y = c(15,50), 
                            col = c('black', 'white'), 
                            lab = c('Annotation clone 5', 'Annotation clone 2'))
dolphinPlot(seaObject_tp, showLegend = TRUE, main = 'Example Dolphin Plot', 
            vlines = timepoints, vlab = timepoints, vlabSize = 2, 
            ylab = 'Cancer cell fraction', annotations = annotsTable, 
            pos = 'bottom', separateIndependentClones = TRUE)
```

```{r thpEffDolph}
##Dolphin plot with enabled therapy effect estimation
##vertical lines show all time points, customized y axis label,
##triangles indicate the measured time points
dolphinPlot(seaObject_te, showLegend = TRUE, 
            vlines = slot(seaObject,"timepoints"), 
            vlab = slot(seaObject,"timepoints"), vlabSize = 2, 
            ylab = 'Cancer cell fractions (CCFs)',
            markMeasuredTimepoints = timepoints)
```

## combinedPlot()

Given a sea object containing layout information, a shark and dolphin plot can be plotted together - linked and interactive.

### Usage

    combinedPlot(seaObject_tp, shark, dolphin, shape, borderCol, vlines, 
                vlineCol, vlab, vlabSize, pos, separateIndependentClones,
                showLegend, markMeasuredTimepoints, downloadWidget, 
                mainDph, mainPosDph, mainSizeDph, mainShk, xlab, ylab, 
                pad.left, annotations, width, height)

-   `seaObject` A seaObject.
-   `shark` A boolean defining whether or not to draw a shark plot (default: TRUE).
-   `dolphin` A boolean defining whether or not to draw a dolphin plot (default: TRUE).
-   `shape` The type of shape to construct the plot out of. The options are "spline" and "polygon" (default: "spline"").
-   `borderCol` A color for the border line. If "NULL" then no border will be drawn (default: NULL).
-   `pos` Plotting position of the clones. Options are "center", "bottom" or "top" (default: "center").
-   `vlines` A vector of positions at which to draw vertical lines (default: NULL).
-   `vlineCol` A color value for the vertical lines (default: "#6E6E66").
-   `vlab` A character vector containing labels for each of the vertical lines (default: NULL).
-   `vlabSize` An integer value for the vertical labels size (default: 3).
-   `separateIndependentClones` Boolean defining whether independently-arising clones (with parent 0) should be separated by blank space in the plot (default: FALSE).
-   `showLegend` A boolean indicating whether to show a legend at the left side of the plot (default: FALSE).
-   `markMeasuredTimepoints` A vector of x positions at which to draw triangles on the bottom of the plot (default: NULL).
-   `downloadWidget` File to safe HTML to (default: NULL).
-   `mainDph` A string corresponding to the dolphin plot's main title (default: NULL).
-   `mainPosDph` A string defining the dolphin plot's title position. Options are 'left', 'middle' or 'right', always above the plot (default: "middle").
-   `mainSizeDph` An integer value defining the size of the dolphin plot's title (default: 5).
-   `mainShk` A string corresponding to the shark plot's main title (default: NULL).
-   `xlab` A string defining the label of the x axis (default: NULL).
-   `ylab` A string defining the label of the y axis. Automatically, a vertical line showing 100% will be plotted (default: NULL).
-   `pad.left` The amount of "ramp-up" to the left of the first time point. Given as a fraction of the total plot width (default: 0.005).
-   `annotations` A data.frame with: columns "x" (x position), "y" (y position), "lab" (annotation text) and "col" (color of the text either black or white) (default: NULL).
-   `width` An integer value indicating the with of the output widget (default: 12).
-   `height` An integer value indicating the height of the output widget (default: 9).

### Details

Dolphin plots may be chosen to be plotted along with basic shark plots (for details see `dolphinPlot()` and `sharkPlot()`. Both plots are internally connected. By hovering on one of the clones, it is automatically highlighted in both, shark and dolphin plot.

Important note: extended shark plots and dolphin plots can NOT be visualized together.

### Examples

```{r SharkDolph}
##Basic shark plot linked to dolphin plot
combinedPlot(seaObject_tp, showLegend = TRUE, vlines = timepoints,
            vlab = timepoints, vlabSize = 2, ylab = 'Cancer cell fraction',
            separateIndependentClones = TRUE)
```

## plaicePlot()

Plaice plots provide an allele-aware visualization of clonal evolution. Plots show the development of all clones over time (x axis) and their clonal prevalences (y axis), and the ratio of remaining healthy alleles (lower plaice).

### Usage

    plaicePlot(seaObject, shape, borderCol, vlines, vlineCol, vlab, vlabSize,
                separateIndependentClones, clonesToFill, showLegend,
                markMeasuredTimepoints, main, mainPos, mainSize, xlab, ylab,
                pad.left, annotations, annotationsSize, interactivePlot)

-   `seaObject` A seaObject.
-   `shape` The type of shape to construct the plot out of. The options are "spline" and "polygon" (default: "spline"").
-   `borderCol` A color for the border line. If "NULL" then no border will be drawn (default: "black").
-   `vlines` A vector of positions at which to draw vertical lines (default: NULL).
-   `vlineCol` A color value for the vertical lines (default: "#6E6E66").
-   `vlab` A character vector containing labels for each of the vertical lines (default: NULL).
-   `vlabSize` An integer value for the vertical labels size (default: 3).
-   `separateIndependentClones` Boolean defining whether independently-arising clones (with parent 0) should be separated by blank space in the plot (default: FALSE).
-   `clonesToFill` An integer vector with the index of the clone's color to fill each clone. For example: clonesToFill \<- c(0,0,0,2,0,0) clone 4 (and its children) will be filled with clone 2 color (default: NULL).
-   `showLegend` A boolean indicating whether to show a legend at the left side of the plot (default: FALSE).
-   `markMeasuredTimepoints` A vector of x positions at which to draw triangles on the bottom of the plot (default: NULL).
-   `main` A string corresponding to the plot's main title (default: NULL).
-   `mainPos` A string defining the title's position. Options are 'left', 'middle' or 'right', always above the plot (default: "middle").
-   `mainSize` An integer value defining the size of the title (default: 5).
-   `xlab` A string defining the label of the x axis (default: NULL).
-   `ylab` A boolean defining whether or not to show the default y axis labels (default: FALSE).
-   `pad.left` The amount of "ramp-up" to the left of the first time point. Given as a fraction of the total plot width (default: 0.005).
-   `annotations` A data.frame with: columns "x" (x position), "y" (y position), "lab" (annotation text) and "col" (color of the text either black or white) (default: NULL).
-   `annotationsSize` An integer value defining the size of the annotations (default: 3).
-   `interactivePlot` A boolean defining whether the plot should be interactive (default: TRUE; if using this function to export the plaice plot, e.g. by png(), define interactivePlot = FALSE).

### Details

Plaice plots represent an option for visualizing clonal evolution on allelic- level. The upper half of the plot (=plaice) shows a common dolphin plot in bottom visualization. The lower plaice shows the percentage of healthy alleles. Clones that are characterized by bi-allelic events affecting one or more genes may be colored accordingly. Coloring of clones in the lower plaice is interpreted as follows:

-   An uncolored clone indicates that at least one copy of a healthy allele is present for all genes.
-   A colored clone indicates that a bi-allelic event took place. As a consequence, no healthy allele of the gene(s) affected by the bi-allelic event remains.
    -   If clone 1 carries a point mutation in TP53 and clone 2 carries an additional del17p (affecting the remaining healthy allele), clone 2 in the lower plaice should be colored in the same hue as clone 1 in the upper plaice, which is originally characterized by mutation in TP53, indicating that the double-hit event affected TP53.
    -   If clone 1 carries a deletion of 17p and clone 2 carries an additional point mutation in TP53 (affecting the remaining healthy allele), clone 2 in the lower plaice should be colored in the same hue as clone 2 in the upper plaice, which is originally charatcerized by mutation in TP53, indicating that the double-hit event affected TP53.
    -   If clone 2 carries two point mutations in TP53 affecting different alleles, clone 2 in the lower plaice should be colored in the same hue as clone 2 in the upper plaice, indicating that the double-hit event affected TP53.
    -   If clone 2 carries a variant affecting the X chromosome of a male subject, leading to a loss of the only available healthy allele, clone 2 in the lower plaice should be colored in the same hue as clone 2 in the upper plaice, indicating the hemizygous variant affecting a gene on the X chromosome.

### Examples

```{r plaice}
#Plaice plot when all genes have at least one healthy copy
plaicePlot(seaObject_tp, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE, 
            separateIndependentClones = TRUE)
```

```{r plaiceAnnots}
#Plaice plot showing biallelic events + annotations
annotsTable <- data.frame(x = c(24,55), y = c(-40,-5), 
                            col = c('black', 'white'), 
                            lab = c('TP53', 'UBA1'))
plaicePlot(seaObject_tp, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE, 
            separateIndependentClones = TRUE, clonesToFill = c(0,0,1,0,0,6,0),
            annotations = annotsTable)
```

```{r plaiceThpEff}
##Plaice plot with enabled therapy effect estimation, 
##all genes are assumed to have at least one healthy copy
plaicePlot(seaObject_te, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE, 
            separateIndependentClones = TRUE)
```

## exploreTrees()

Plaice plots provide an allele-aware visualization of clonal evolution. Plots show the development of all clones over time (x axis) and their clonal prevalences (y axis), and the ratio of remaining healthy alleles (lower plaice).

### Usage

    exploreTrees(fracTable,timepoints)

-   `fracTable` A numeric matrix containing tumor fraction estimates for all clones at all time points.
-   `timepoints` A numeric vector specifying the time points for each column of the matrix.

### Details

To create a seaObject, the basis for all plotting functions in clevRvis, a fracTable, a timepoints vector and a parents vector are required. clevRvis provides an approach to determine all valid parental relations on the basis of the information provided in fracTable and the timepoints vector. Thereby, alternative trees can be explored.

To optimize run-time, the analysis is devided into 3 step procedure:

1)  Possible parents are determined. If clone 1 has at any measured time point a lower CCF compared to clone 2, then clone 1 cannot be clone 2's parent.

2)  Possible branched dependent evolution is investigated. If clone 2 can only develope from clone 1, the difference in CCFs for clone 1 and clone 2 is calculated. Every remaining clone with a CCF larger than the difference cannot develop from clone 1.

3)  All remaining, possible parental relations are determined. An extensive validity check is performed using clevRvis (validity check when creating a seaObject). A maximum of 20,000 parental relations is investigated.

A list of numeric vectors containing valid parental relations, apt to explore alternative trees.

### Examples

```{r exploreTrees}
##Alternative, valid parental relations are determined
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                20, 00, 00, 00,
                30, 00, 20, 05,
                98, 00, 55, 40),
            ncol=length(timepoints))

trees <- exploreTrees(fracTable, timepoints)
trees
```

## clevRvisShiny()

clevRvis provides an additional shiny GUI to perform all analyses interactively.

### Usage

    clevRvisShiny()

### Examples

clevRvis supports two input file types - with and without information on parental relations.

#### Input - without parental relations

A file containing clones in the first column and information on the clones' cancer cell fractions (CCFs) for each time point in the subsequent columns may be uploaded. Subsequently, the parental relations are defined interactively in the shiny-app. Two exemplary input files, containing the CCFs for simulated clonal evolution, are available with the package.

Exemplary file Example1.xlsx is provided along with the package in 'inst/extdata'. The file contains an easy, manually generated example of clonal evolution (4 clones, 2 time points). The suggested parental relations, that have to be defined manually for the analysis with clevRvis, are:

```{r example1,echo=FALSE}
library(knitr)
kable(data.frame(Clone=c("clone A","clone B","clone C","clone D"),
                Parent=c("normal cell","clone A","clone B","clone C")))
```

A second exemplary file Example2.xlsx is also provided along with the package in 'inst/extdata'. The file contains a more complex, manually generated example of clonal evolution (5 clones, 5 time points). For Example2.xlsx the suggested parental relations are:

```{r example2,echo=FALSE}
library(knitr)
kable(data.frame(Clone=c("clone A","clone B","clone C","clone D","clone E"),
                Parent=c("normal cell","clone A","clone B","clone C",
                        "clone B")))
```

Alternatively, previously defined vector `timepoints` and matrix `fracTable` can be easily used to generate adequate input data for use of the clevRvis shiny-app.

```{r example3}
##Data used in a previous example
timepoints <- c(0,50,100)
fracTable <- matrix(c(20,10,0,0,0,0,0,
                    40,20,15,0,30,10,0,
                    50,25,15,10,40,20,15),
                    ncol = length(timepoints))

##Define input for shiny-app
input<-data.frame(Clones=paste0("Clone ",LETTERS[1:nrow(fracTable)]),
                    as.data.frame(fracTable))
names(input)[2:ncol(input)]<-timepoints

##Export data.frame 'input', e.g. as csv-file
##write.table(input, "Example3.csv", sep = ",", row.names = FALSE,quote = FALSE)
```

Subsequently, Example3.csv can be used as input for the clevRvis shiny-app. Parental relations have to be defined interactively:

```{r example3Parents,echo=FALSE}
library(knitr)
kable(data.frame(Clone=c("clone A","clone B","clone C","clone D","clone E",
                        "Clone F","Clone G"),
                Parent=c("normal cell","clone A","clone A","clone C",
                        "Normal cell","Clone E","Clone F")))
```

#### Input - with parental relations

A file containing clones in the first column, information on the clones' CCFs for each time point in the subsequent columns and information on the parental relations in a column 'parent' may be uploaded. The position of the 'parent'-column in the input file is not restricted. Two exemplary input files - Example1_withParents.xlsx and Example2_withParents.xlsx -, containing the CCFs for simulated clonal evolution and parental relations in an additional column, are available with the package in 'inst/extdata'. The files correspond to files Example1.xlsx and Example2.xlsx, just containing the additional column 'parent'.

Alternatively, previously defined vectors `timepoints` and `parents` and matrix `fracTable` can be easily used to generate adequate input data for use of the clevRvis shiny-app.

```{r example4}
##Data used in a previous example
timepoints <- c(0,50,100)
parents <- c(0,1,1,3,0,5,6)
fracTable <- matrix(c(20,10,0,0,0,0,0,
                    40,20,15,0,30,10,0,
                    50,25,15,10,40,20,15),
                    ncol = length(timepoints))

##Define input for shiny-app
input<-data.frame(Clones=paste0("Clone ",LETTERS[1:nrow(fracTable)]),
                    as.data.frame(fracTable),parents=parents)
names(input)[2:(ncol(input)-1)]<-timepoints

##Export data.frame 'input', e.g. as csv-file
##write.table(input, "Example3_withParents.csv", sep = ",", row.names = FALSE,
##quote = FALSE)
```

Subsequently, Example3_withParents.csv can be used as input for the clevRvis shiny-app.

#### (opt.) Explore parental relations

Independent of whether a user uploads parental relations within an input file or not, clevRvis provides an option to explore parental relations. For a defined CCF table, all valid parental relations can be determined. The results are provided as a drop-down menu in the 'Inputs' panel (normal cells coded as 0). A user may choose any valid parental relations instead of the original one to perform all subsequent analyses.

#### Subsequent analyses

Once an input file has been read in, further analysis options are available:

As second step, a seaObject has to be created, using the second panel on the left side of the clevRvis shiny-app - 'seaObject options'. `Extra time points interpolation` and `Therapy Effect estimation` can be enabled. By executing `Submit`, a seaObject is created. The seaObject table is provided in the second output-tab 'seaObject table' on the right.

As a third step, customized plots can be generated, using the third panel on the left side of the clevRvis shiny-app - 'Plots'. All plots are provided in the third output-tab 'Plots' on the right.

Optionally, as a final step, annotations can be added to the previously generated plots using the fourth panel on the left side of the clevRvis shiny-app - 'Annotations'.

A detailed walk-through is provided in the 'Tutorial' panel within the clevRvis shiny-app.

## Data

To generate appropriate input for the analysis with clevRvis, called variants have to be clustered (ideally also considering copy-number variants) and phylogenetic trees reconstructed to determine parental information. According to our experience, algorithms for automatic clustering and tree reconstruction, considering bulk DNA-seq data, do not always generate valid results (see Sandmann et al. 2022 <https://cgp.iiarjournals.org/content/19/2/194.long>). Currently, we suggest manual analysis for both analysis steps (compare e.g. Reutter, Sandmann et al. 2021 <https://www.nature.com/articles/s41375-020-0862-5> or Sandmann, Behrens et al. 2022 <https://www.frontiersin.org/articles/10.3389/fonc.2022.888114/full>).

# Session information

```{r session}
sessionInfo()
```

\name{extSharkPlot}
\alias{extSharkPlot}
\title{
Generate an extended graph visualization of clonal evolution
}
\description{
An extended shark plot shows the basic graph visualization of clonal evolution
with nodes representing clones and edges indicating their evolutionary relations
and additional visualization of CCFs.
}
\usage{
extSharkPlot(seaObject, 
            showLegend = FALSE, 
            main = NULL, 
            timepoints = NULL, 
            width = 10, 
            interactivePlot = TRUE)
}
\arguments{
\item{seaObject}{
A seaObject.
}
\item{showLegend}{
A boolean indicating whether to show the legend or not (default: FALSE).
}
\item{main}{
A string corresponding to the plot's main title (default: NULL).
}
\item{timepoints}{
By default, all time points available in the seaObject are visualized.
Optionally, a selected set of available time points can be chosen
(default: NULL).
}
\item{width}{
An integer value indicating the width of the widget plot (default: 10).
}
\item{interactivePlot}{
A boolean defining whether the plot should be interactive (default: TRUE; if
using this function to export the extended shark plot, e.g. by png(), define
interactivePlot = FALSE).
}
}
\details{
An extended shark plots consists of two elements:

1) A basic shark plot: common trees with nodes representing clones and edges
indicating their evolutionary relation. Phylogeny can be directly deduced from
these plots.

2) Additionally, CCFs of each clone (rows) at each time point (columns) are
shown as points next to the basic shark plot. The size of each point correlates
with the CCF at the corresponding clone and time point.

Both plots are linked in an interactive widget.
}
\value{
None
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}

\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints)

extSharkPlot(seaObject, showLegend = TRUE, main = 'Example Shark Plot')
}

\name{combinedPlot}
\alias{combinedPlot}
\title{
Generate a combined basic graph and detailed visualization of clonal evolution
}
\description{
Given a sea object containing layout information, a shark and dolphin plot can
be plotted together - linked and interactive.
}
\usage{
combinedPlot(seaObject, 
            shark = TRUE, 
            dolphin = TRUE, 
            shape = "spline", 
            borderCol = NULL, 
            vlines = NULL, 
            vlineCol = "#6E6E66", 
            vlab = NULL,
            vlabSize = 3, 
            pos = "center", 
            separateIndependentClones = FALSE, 
            showLegend = FALSE,
            markMeasuredTimepoints = NULL, 
            downloadWidget = NULL, 
            mainDph = NULL, 
            mainPosDph = "middle",
            mainSizeDph = 5, 
            mainShk = NULL, 
            xlab = NULL, 
            ylab = NULL, 
            pad.left = 0.005, 
            annotations = NULL, 
            width = 12,
            height = 9)
}
\arguments{
\item{seaObject}{
A seaObject.
}
\item{shark}{
A boolean defining whether or not to draw a shark plot (default: TRUE).
}
\item{dolphin}{
A boolean defining whether or not to draw a dolphin plot (default: TRUE).
}
\item{shape}{
The type of shape to construct the plot out of. The options are "spline" and
"polygon" (default: "spline"").
}
\item{borderCol}{
A color for the border line. If "NULL" then no border will be drawn (default:
NULL).
}
\item{vlines}{
A vector of positions at which to draw vertical lines (default: NULL).
}
\item{vlineCol}{
A color value for the vertical lines (default: "#6E6E66").
}
\item{vlab}{
A character vector containing labels for each of the vertical lines (default:
NULL).
}
\item{vlabSize}{
An integer value for the vertical labels size (default: 3).
}
\item{pos}{
Plotting position of the clones. Options are "center", "bottom" or "top"
(default: "center").
}
\item{separateIndependentClones}{
Boolean defining whether independently-arising clones (with parent 0) should be
separated by blank space in the plot (default: FALSE).
}
\item{showLegend}{
A boolean indicating whether to show a legend at the left side of the plot
(default: FALSE).
}
\item{markMeasuredTimepoints}{
A vector of x positions at which to draw triangles on the bottom of the plot
(default: NULL).
}
\item{downloadWidget}{
File to safe HTML to (default: NULL).
}
\item{mainDph}{
A string corresponding to the dolphin plot's main title (default: NULL).
}
\item{mainPosDph}{
A string defining the dolphin plot's title position. Options are 'left',
'middle' or 'right', always above the plot (default: "middle").
}
\item{mainSizeDph}{
An integer value defining the size of the dolphin plot's title (default: 5).
}
\item{mainShk}{
A string corresponding to the shark plot's main title (default: NULL).
}
\item{xlab}{
A string defining the label of the x axis (default: NULL).
}
\item{ylab}{
A string defining the label of the y axis. Automatically, a vertical line
showing 100\% will be plotted (default: NULL).
}
\item{pad.left}{
The amount of "ramp-up" to the left of the first time point. Given as a fraction
of the total plot width (default: 0.005).
}
\item{annotations}{
A data.frame with: columns "x" (x position), "y" (y position), "lab" (annotation
text) and "col" (color of the text either black or white) (default: NULL).
}
\item{width}{
An integer value indicating the with of the output widget (default: 12).
}
\item{height}{
An integer value indicating the height of the output widget (default: 9).
}
}
\details{
Dolphin plots may be chosen to be plotted along with basic shark plots (for
details see \code{\link{dolphinPlot}} and \code{\link{sharkPlot}}). Both plots
are internally connected. By hovering on one of the clones, it is automatically
highlighted in both, shark and dolphin plot.

Important note: extended shark plots and dolphin plots can NOT be visualized
together.
}
\value{
None
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}

\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints,
                            timepointInterpolation = TRUE)

combinedPlot(seaObject, borderCol = 'white', showLegend = TRUE)
}

\name{dolphinPlot}
\alias{dolphinPlot}
\title{
Generate a detailed visualization of clonal evolution
}
\description{
Dolphin plots provide a detailed visualization of clonal evolution. Plots show
the development of all clones over time (x axis) and their clonal prevalences
(y axis).
}
\usage{
dolphinPlot(seaObject, 
            shape = "spline", 
            borderCol = NULL, 
            pos = "center", 
            vlines = NULL, 
            vlineCol = "#6E6E66", 
            vlab = NULL, 
            vlabSize = 3,
            separateIndependentClones = FALSE, 
            showLegend = FALSE, 
            markMeasuredTimepoints = NULL,
            main = NULL, 
            mainPos = "middle", 
            mainSize = 5, 
            xlab = NULL, 
            ylab = NULL, 
            pad.left = 0.005, 
            annotations = NULL,
            annotSize = 3)
}
\arguments{
\item{seaObject}{
A seaObject.
}
\item{shape}{
The type of shape to construct the plot out of. The options are "spline" and
"polygon" (default: "spline").
}
\item{borderCol}{
A color for the border line. If "NULL" then no border will be drawn (default:
NULL).
}
\item{pos}{
Plotting position of the clones. Options are "center" or "bottom" (default:
"center").
}
\item{vlines}{
A vector of positions at which to draw vertical lines (default: NULL).
}
\item{vlineCol}{
A color value for the vertical lines (default: "#6E6E66").
}
\item{vlab}{
A character vector containing labels for each of the vertical lines (default:
NULL).
}
\item{vlabSize}{
An integer value for the vertical labels size (default: 3).
}
\item{separateIndependentClones}{
Boolean defining whether independently-arising clones (with parent 0) should be
separated by blank space in the plot (default: FALSE).
}
\item{showLegend}{
A boolean indicating whether to show a legend at the left side of the plot
(default: FALSE).
}
\item{markMeasuredTimepoints}{
A vector of x positions at which to draw triangles on the bottom of the plot
(default: NULL).
}
\item{main}{
A string corresponding to the plot's main title (default: NULL).
}
\item{mainPos}{
A string defining the title's position. Options are 'left', 'middle' or 'right',
always above the plot (default: "middle").
}
\item{mainSize}{
An integer value defining the size of the title (default: 5).
}
\item{xlab}{
A string defining the label of the x axis (default: NULL).
}
\item{ylab}{
A string defining the label of the y axis. Automatically, a vertical line
showing 100\% will be plotted (default: NULL).
}
\item{pad.left}{
The amount of "ramp-up" to the left of the first time point. Given as a fraction
of the total plot width (default: 0.005).
}
\item{annotations}{
A data.frame with: columns "x" (x position), "y" (y position), "lab" (annotation
text) and "col" (color of the text either black or white) (default: NULL).
}
\item{annotSize}{
An integer value defining the size of the annotations (default: 3).
}

}
\details{
Dolphin plots displays detailed information on clonal evolution, showing the
development of all clones over time (x axis) and their clonal prevalence
(y axis). Information on phylogeny, CCFs and time course characterizing a clonal
evolution are jointly visualized in this single plot.

Several basic options for customizing dolphin plots are available, e.g.
switching between spline and polygon shape, bottom or central visualization,
annotations, separating independent clones, adding vertical lines and labels,
changing border and vertical lines colors, etc.

Dolphin plots may be chosen to be plotted along with basic shark plots (see
\code{\link{combinedPlot}}).
}
\value{
None
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}

\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints,
                            timepointInterpolation = TRUE)

dolphinPlot(seaObject, main = 'Example Dolphin Plot', pos = 'center')
}

\name{plaicePlot}
\alias{plaicePlot}
\title{
Generate an allele-aware visualization of clonal evolution
}
\description{
Plaice plots provide an allele-aware visualization of clonal evolution. Plots
show the development of all clones over time (x axis) and their clonal
prevalences (y axis), and the ratio of remaining healthy alleles (lower plaice).
}
\usage{
plaicePlot(seaObject, 
            shape = "spline", 
            borderCol = "black", 
            vlines = NULL, 
            vlineCol = "#6E6E66", 
            vlab = NULL, 
            vlabSize = 3,
            separateIndependentClones = FALSE, 
            clonesToFill = NULL, 
            showLegend = FALSE,
            markMeasuredTimepoints = NULL, 
            main = NULL, 
            mainPos = "middle", 
            mainSize = 5, 
            xlab = NULL, 
            ylab = FALSE,
            pad.left = 0.005, 
            annotations = NULL, 
            annotationsSize = 3, 
            interactivePlot = TRUE)
}
\arguments{
\item{seaObject}{
A seaObject.
}
\item{shape}{
The type of shape to construct the plot out of. The options are "spline" and
"polygon" (default: "spline"").
}
\item{borderCol}{
A color for the border line. If "NULL" then no border will be drawn (default:
"black").
}
\item{vlines}{
A vector of positions at which to draw vertical lines (default: NULL).
}
\item{vlineCol}{
A color value for the vertical lines (default: "#6E6E66").
}
\item{vlab}{
A character vector containing labels for each of the vertical lines (default:
NULL).
}
\item{vlabSize}{
An integer value for the vertical labels size (default: 3).
}
\item{separateIndependentClones}{
Boolean defining whether independently-arising clones (with parent 0) should be
separated by blank space in the plot (default: FALSE).
}
\item{clonesToFill}{
An integer vector with the index of the clone's color to fill each clone. For
example: clonesToFill <- c(0,0,0,2,0,0) clone 4 (and its children) will be
filled with clone 2 color (default: NULL).
}
\item{showLegend}{
A boolean indicating whether to show a legend at the left side of the plot
(default: FALSE).
}
\item{markMeasuredTimepoints}{
A vector of x positions at which to draw triangles on the bottom of the plot
(default: NULL).
}
\item{main}{
A string corresponding to the plot's main title (default: NULL).
}
\item{mainPos}{
A string defining the title's position. Options are 'left', 'middle' or 'right',
always above the plot (default: "middle").
}
\item{mainSize}{
An integer value defining the size of the title (default: 5).
}
\item{xlab}{
A string defining the label of the x axis (default: NULL).
}
\item{ylab}{
A boolean defining whether or not to show the default y axis labels (default:
FALSE).
}
\item{pad.left}{
The amount of "ramp-up" to the left of the first time point. Given as a fraction
of the total plot width (default: 0.005).
}
\item{annotations}{
A data.frame with: columns "x" (x position), "y" (y position), "lab" (annotation
text) and "col" (color of the text either black or white) (default: NULL).
}
\item{annotationsSize}{
An integer value defining the size of the annotations (default: 3).
}
\item{interactivePlot}{
A boolean defining whether the plot should be interactive (default: TRUE; if
using this function to export the plaice plot, e.g. by png(), define
interactivePlot = FALSE).
}

}
\details{
Plaice plots are based on the bottom visualization of dolphin plots ("flatfish"
= plaice), mirrored above and below the x-axis . They have been developed to
improve the visualization of biallelic events.

Clonal evolution can be visualized in the upper half of the plot. Several
options similar to the ones for dolphin plot are available as well for plaice
plot: switching between spline and polygon shape, annotations, separating
independent clones (recommended), adding vertical lines and labels, etc.

On the bottom half of the plot, a not-colored mirrored representation of clonal
evolution is plotted. The user may choose the clones and their color, in order
to show which clones caused biallelic events. When a clone is not colored, it
indicates that a healthy allele remains.

Variants affecting the only available X- or Y-chromosome in male subjects can
also be visualized using plaice plots.
}
\value{
None
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}

\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints,
                            timepointInterpolation = TRUE)

plaicePlot(seaObject, shape = "spline",
            vlines = c(0,150), vlab = c("day 0","day 150"),
            main = 'Example plot', clonesToFill = c(0,1,0))
}

\docType{class}
\name{seaObject-class}
\alias{seaObject-class}
\alias{show}
\alias{show,seaObject-method}
\alias{ytop}
\alias{ytop,seaObject-method}
\alias{ytop<-}
\alias{ytop<-,seaObject-method}
\alias{ybtm}
\alias{ybtm,seaObject-method}
\alias{ybtm<-}
\alias{ybtm<-,seaObject-method}
\alias{xpos}
\alias{xpos,seaObject-method}
\alias{xpos<-}
\alias{xpos<-,seaObject-method}
\alias{col}
\alias{col,seaObject-method}
\alias{col<-}
\alias{col<-,seaObject-method}
\alias{timepoints}
\alias{timepoints,seaObject-method}
\alias{timepoints<-}
\alias{timepoints<-,seaObject-method}
\alias{fracTable}
\alias{fracTable,seaObject-method}
\alias{fracTable<-}
\alias{fracTable<-,seaObject-method}
\alias{parents}
\alias{parents,seaObject-method}
\alias{parents<-}
\alias{parents<-,seaObject-method}
\alias{nestLevels}
\alias{nestLevels,seaObject-method}
\alias{nestLevels<-}
\alias{nestLevels<-,seaObject-method}
\alias{cloneFamily}
\alias{cloneFamily,seaObject-method}
\alias{cloneFamily<-}
\alias{cloneFamily<-,seaObject-method}
\alias{cloneLabels}
\alias{cloneLabels,seaObject-method}
\alias{cloneLabels<-}
\alias{cloneLabels<-,seaObject-method}
\alias{defaultLabels}
\alias{defaultLabels,seaObject-method}
\alias{defaultLabels<-}
\alias{defaultLabels<-,seaObject-method}
\alias{originTimepoint}
\alias{originTimepoint,seaObject-method}
\alias{originTimepoint<-}
\alias{originTimepoint<-,seaObject-method}
\title{
Class seaObject
}
\description{
Represents a seaObject class, containing all necessary input to generate shark
plots, dolphin plots and plaice plots
}
\section{Methods}{
    \describe{
    \item{show}{
    \code{show(seaObject)}: summary of the seaObject.
    }
    \item{ytop}{
    \code{ytop(seaObject)}: get value of ytop.
    }
    \item{ytop<-}{
    \code{ytop(seaObject)<-}: assign value to ytop.
    }
    \item{ybtm}{
    \code{ybtm(seaObject)}: get value of ybtm.
    }
    \item{ybtm<-}{
    \code{ybtm(seaObject)<-}: assign value to ybtm.
    }
    \item{xpos(x)}{
    \code{xpos(seaObject)}: get value of xpos
    }
    \item{xpos(x)<-}{
    \code{xpos(seaObject)}: assign value to xpos
    }
    \item{col(x)}{
    \code{col(seaObject)}: get value of col
    }
    \item{col(x)<-}{
    \code{col(seaObject)}: assign value to col
    }
    \item{timepoints(x)}{
    \code{timepoints(seaObject)}: get value of timepoints
    }
    \item{timepoints(x)<-}{
    \code{timepoints(seaObject)}: assign value to timepoints
    }
    \item{fracTable(x)}{
    \code{fracTable(seaObject)}: get value of fracTable
    }
    \item{fracTable(x)<-}{
    \code{fracTable(seaObject)}: assign value to fracTable
    }
    \item{parents(x)}{
    \code{parents(seaObject)}: get value of parents
    }
    \item{parents(x)<-}{
    \code{parents(seaObject)}: assign value to parents
    }
    \item{nestLevels(x)}{
    \code{nestLevels(seaObject)}: get value of nestLevels
    }
    \item{nestLevels(x)<-}{
    \code{nestLevels(seaObject)}: assign value to nestLevels
    }
    \item{cloneFamily(x)}{
    \code{cloneFamily(seaObject)}: get value of cloneFamily
    }
    \item{cloneFamily(x)<-}{
    \code{cloneFamily(seaObject)}: assign value to cloneFamily
    }
    \item{cloneLabels(x)}{
    \code{cloneLabels(seaObject)}: get value of cloneLabels
    }
    \item{cloneLabels(x)<-}{
    \code{cloneLabels(seaObject)}: assign value to cloneLabels
    }
    \item{defaultLabels(x)}{
    \code{defaultLabels(seaObject)}: get value of defaultLabels
    }
    \item{defaultLabels(x)<-}{
    \code{defaultLabels(seaObject)}: assign value to defaultLabels
    }
    \item{originTimepoint(x)}{
    \code{originTimepoint(seaObject)}: get value of originTimepoint
    }
    \item{originTimepoint(x)<-}{
    \code{originTimepoint(seaObject)}: assign value to originTimepoint
    }
}
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}
\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints)
timepoints(seaObject)
timepoints(seaObject) <- c(0,20,75,150)
}

\name{exploreTrees}
\alias{exploreTrees}
\title{
Explore alternative trees
}
\description{
clevRvis allows to explore alternative trees, i.e. alternative parental
relations. Taking information on the CCFs and available time points as input,
all possible parental relations are investigated and checked for validity.
}
\usage{
exploreTrees(fracTable, 
            timepoints)
}
\arguments{
\item{fracTable}{
A numeric matrix containing tumor fraction estimates for all clones at all
time points.
}
\item{timepoints}{
A numeric vector specifying the timepoints for each column of the matrix.
}
}
\details{
To create a seaObject, the basis for all plotting functions in clevRvis, a
fracTable, a timepoints vector and a parents vector are required. clevRvis
provides an approach to determine all valid parental relations on the basis
of the information provided in fracTable and the timepoints vector. Thereby,
alternative trees can be explored.

To optimize run-time, the analysis is devided into 3 step procedure:

1) Possible parents are determined. If clone 1 has at any measured time point a
lower CCF compared to clone 2, then clone 1 cannot be clone 2's parent.

2) Possible branched dependent evolution is investigated. If clone 2 can only
develope from clone 1, the difference in CCFs for clone 1 and clone 2 is
calculated. Every remaining clone with a CCF larger than the difference
cannot develop from clone 1.

3) All remaining, possible parental relations are determined. An extensive
validity check is performed using clevRvis (validity check when creating a
seaObject). A maximum of 20,000 parental relations is investigated.


}
\value{
A list of numeric vectors containing valid parental relations, apt to explore
alternative trees.
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}


\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45,  0,  0,
                20,  0,  0,  0,
                30,  0, 20,  5,
                98,  0, 55, 40),
            ncol=length(timepoints))

trees <- exploreTrees(fracTable, timepoints)
}

\name{clevRvis_package}
\alias{clevRvis_package}
\alias{clevRvis}
\docType{package}
\title{
\packageTitle{clevRvis}
}
\description{
clevRvis is a tool that allows you to visualize changes in the subclonal
architecture of tumors. Simple tree visualization (shark plot) a more detailed
visualization (dolphin plot) and an allele-aware visualization (plaice plot)
are available.
Moreover, this tool provides fully automatic algorithms for interpolating time 
points and estimating therapy effect in the presence of lacking input data.
}
\details{
The package contains a function performing the whole analysis using a shiny user
interface - \code{\link{clevRvisShiny}}.

Additionally, all functions for classical use in R are available:

1) \code{createSeaObject}: create the seaObject needed for further visualization
options

2) \code{sharkPlot}: basic graph visualization (shark plot) of clonal evolution.

3) \code{extSharkplot}: basic graph visualization with extension showing Cancer
Cell Fractions (CCFs) as point size for each clone at each selected time point.

4) \code{dolphinPlot}: detailed visualization (dolphin plot) of clonal evolution
showing the development of all clones over time and their clonal prevalences.

5) \code{combinedPlot}: interactive linked visualization of shark and dolphin plot
together.

6) \code{plaicePlot}: detailed visualization on the allelic level (plaice plot)
of clonal evolution. Clonal prevalences on the top half and percentage of
remaining healthy alleles on the bottom half.

7) \code{exploreTrees}: Generate alternative parental relations to explore
    alternative trees
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}

\seealso{
\code{\link{clevRvisShiny}}, \code{\link{createSeaObject}},
\code{\link{sharkPlot}}, \code{\link{extSharkPlot}}, \code{\link{dolphinPlot}},
\code{\link{combinedPlot}}, \code{\link{plaicePlot}}, \code{\link{exploreTrees}}
}
\examples{
timepoints <- c(0,50,100)
parents <- c(0,1,1,3,0,5,6)
fracTable <- matrix(
                    c(  20, 10,  0,  0,  0,  0,  0,
                        40, 20, 15,  0, 30, 10,  0,
                        50, 25, 15, 10, 40, 20, 15),
                    ncol = length(timepoints))

#Generating a seaObject with timepoint interpolation
seaObject <- createSeaObject(fracTable, parents, timepoints)


#Basic shark plot showing legend and title
#png('basicShark.png',height = 800, width=600)
sharkPlot(seaObject, showLegend = TRUE, main = 'Example Shark plot')
#dev.off()

#extended shark plot, showing CCF as point size only for measured timepoints,
#legend and title
#png('extendedShark.png',height = 700, width = 1500)
extSharkPlot(seaObject, timepoints = timepoints, showLegend = TRUE,
            main = 'Example Extended Shark plot')
#dev.off()

#Default dolphin plot, with vertical lines showing all time points, custom
#y axis label and triangles indicating the measured time points
#png('dolphinPlot.png',height = 800, width=1750)
dolphinPlot(seaObject, showLegend = TRUE, vlines = slot(seaObject,"timepoints"),
            vlab = slot(seaObject,"timepoints"), vlabSize = 2,
            ylab = 'Cancer cell fractions (CCFs)',
            markMeasuredTimepoints = timepoints)
#dev.off()

#Basic shark plot linked to dolphin plot
combinedPlot(seaObject, showLegend = TRUE, vlines = timepoints,
            vlab = timepoints, vlabSize = 2, ylab = 'Cancer cell fraction',
            separateIndependentClones = TRUE)

#static plaice plot showing biallelic events + annotations
annotsTable <- data.frame(x = c(24,55), y = c(-40,-5),
                        col = c('black', 'white'), lab = c('TP53', 'UBA1'))
#png('plaicePlot.png',height = 800, width = 1750)
plaicePlot(seaObject, showLegend = TRUE,  vlines = timepoints,
            vlab = timepoints, vlabSize = 4, ylab = TRUE,
            separateIndependentClones = TRUE, clonesToFill = c(0,0,1,0,0,6,0),
            annotations = annotsTable, interactivePlot = FALSE)
#dev.off()

#seaObject with enabled timepoint interpolation and therapy effect estimation
#between timepoint 50 and 100
seaObject <- createSeaObject(fracTable, parents, timepoints,
                            timepointInterpolation = TRUE,
                            therapyEffect = c(50,100))

#Default dolphin plot showing estimated therapy effect, with vertical
#lines showing all time points, custom y axis label and triangles indicating
#the measured time points
#png('dolphinPlotTherapy.png',height = 800, width=1750)
dolphinPlot(seaObject, showLegend = TRUE, vlines = slot(seaObject,"timepoints"),
            vlab = slot(seaObject,"timepoints"), vlabSize = 2,
            ylab = 'Cancer cell fractions (CCFs)',
            markMeasuredTimepoints = timepoints)
#dev.off()

#Explore alternative valid trees
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45,  0,  0,
                20,  0,  0,  0,
                30,  0, 20,  5,
                98,  0, 55, 40),
            ncol=length(timepoints))
trees <- exploreTrees(fracTable, timepoints)

}

\name{createSeaObject}
\alias{createSeaObject}
\title{
Create a seaObject
}
\description{
clevRvis needs a seaObject for the visualization of clonal evolution by means
of any available plot. When generating the seaObject, extra time points may be
interpolated, therapy effect may be estimated.
}
\usage{
createSeaObject(fracTable, 
                parents, 
                timepoints, 
                col = NULL, 
                cloneLabels = NULL,
                originTimepoint = NULL, 
                timepointInterpolation = TRUE, 
                therapyEffect = NULL)
}
\arguments{
\item{fracTable}{
A numeric matrix containing tumor fraction estimates for all clones at all
time points.
}
\item{parents}{
An integer vector specifying parental relationships between clones.
}
\item{timepoints}{
A numeric vector specifying the time points for each column of the matrix.
}
\item{col}{
A vector of colors to use when plotting each clone (default: NULL).
}
\item{cloneLabels}{
A character vector of names to assign to each clone when plotting a legend
(default: NULL).
}
\item{originTimepoint}{
Time point when the first clone emerges (must be before the first measured time
point) (default: NULL).
}
\item{timepointInterpolation}{
When set to true extra time points will be estimated between measured time
points and before the first measure time point to improve the visualization
(default: TRUE).
}
\item{therapyEffect}{
A single numeric value indicating the time point when to estimate the effect
of therapy or a numeric vector containing two consecutive measured time points,
therapy effect time point will be in the middle (default: NULL).
}
}
\details{
The basis for all plotting functions included in clevRvis is a seaObject. It
contains information on the CCFs for all clones, at all measured time points.
Additionally, parental information on the clones is included.

Additional time points may be interpolated, therapy effect may be estimated when
generating seaObjects.

Time point interpolation is generally recommended to improve visualization of
clonal evolution. When having less time points than clones, or many new clones
emerging in one single measured time point, the extra time point estimation is
strongly recommended to visualize the clonal evolution properly. If there is
only one measured time point, time point interpolation is required and the time
point of origin has to be specified manually, as there is no way of calculating
it.

To visualize the effect of therapy on the clones' CCFs in case of missing
measured data, a fully automatic approach for therapy effect estimation is
available. When creating the seaObject, a specific time point can be defined
(between two measured time points) or two measured time points can be selected
(new therapy effect time point will be in the middle) for the estimation of the
therapy effect.

}
\value{
A sea Object with all relevant slots filled.
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}

\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints)

}

\name{sharkPlot}
\alias{sharkPlot}
\title{
Generate a basic graph visualization of clonal evolution
}
\description{
A shark plot shows the basic graph visualization of clonal evolution with nodes
representing clones and edges indicating their evolutionary relations.
}
\usage{
sharkPlot(seaObject, 
            showLegend = FALSE, 
            main = NULL)
}
\arguments{
\item{seaObject}{
A seaObject.
}
\item{showLegend}{
A boolean indicating whether to show the legend or not (default: FALSE).
}
\item{main}{
A string corresponding to the plot's main title (default: NULL).
}
}
\details{
A shark plot is the basic approach for visualization: common trees, with nodes
representing clones and edges indicating their evolutionary relation. Phylogeny
can be directly deduced from these plots.

Shark plots also offer an extension to visualize the changes in CCF along time
for each clone. CCFs of each clone (rows) at each time point (columns) are shown
as points next to the basic shark plot (see \code{\link{extSharkPlot}}).
}
\value{
None
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}


\examples{
timepoints <- c(0,30,75,150)
fracTable <- matrix(
            c( 100, 45, 00, 00,
                02, 00, 00, 00,
                03, 00, 02, 01,
                98, 00, 95, 40),
            ncol=length(timepoints))
parents <- c(0,1,1,3)
seaObject <- createSeaObject(fracTable = fracTable,
                            parents = parents,
                            timepoints = timepoints)

sharkPlot(seaObject, showLegend = TRUE, main = 'Example Shark Plot')
}

\name{clevRvisShiny}
\alias{clevRvisShiny}
\title{
A user interface to perform all analyses with clevRvis.
}
\description{
clevRvis is a tool that allows you to visualize changes in the subclonal
architecture of tumors. Simple tree visualization (shark plot) a more detailed
visualization (dolphin plot) and an allele-aware visualization (plaice plot)
are available.
Moreover, this tool provides fully automatic algorithms for time point
interpolation and therapy effect estimation in the presence of lacking input
data.
}
\usage{
clevRvisShiny()
}
\arguments{
None
}
\details{
Detailed information on the usage of clevRvisShiny are available within the
shiny app (see Tutorial).
}
\value{
None
}
\author{
Sarah Sandmann <sarah.sandmann@uni-muenster.de>
}


\examples{
if(interactive()){
    clevRvisShiny()
}}
